(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    188164,       5638]*)
(*NotebookOutlinePosition[    203122,       5992]*)
(*  CellTagsIndexPosition[    202749,       5977]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\<\
Introduction to Scientific Programming:  Computational Problem Solving Using \
Mathematica and C

Joseph L. Zachary\
\>", "Chapter"],

Cell["\<\
The Blocks, Integ, and Newton package are designed to accompany Introduction \
to Scientific Programming.\
\>", "Text"],

Cell["Blocks", "Section",
  CellTags->"Blocks"],

Cell[TextData[{
  "The Blocks package provides four functions useful for summing the even \
terms of the harmonic series, as discussed in Chapter 4 of Introduction to \
Scientific Programming.  \nThe even terms of the series are ",
  Cell[BoxData[
      \(TraditionalForm\`1\/2\)]],
  " + ",
  Cell[BoxData[
      \(TraditionalForm\`1\/4\)]],
  " + ",
  Cell[BoxData[
      \(TraditionalForm\`1\/6\)]],
  " + ",
  Cell[BoxData[
      \(TraditionalForm\`1\/8\)]],
  " + ...\nBefore the functions of the package can be used, the package must \
be loaded with"
}], "Text",
  CellTags->"Blocks"],

Cell[BoxData[
    \(Needs["\<ISP`Blocks`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"Blocks"],

Cell[CellGroupData[{

Cell["BlockFast", "Subsection",
  CellTags->{"Blocks", "BlockFast"}],

Cell[TextData[
"\[FilledSmallSquare] BlockFast[n] sums the first n even terms of the \
harmonic series by exploiting the EulerGamma constant and the PolyGamma \
function."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockFast"}],

Cell[TextData[
"\[FilledSmallSquare] The parameter n must be a non-negative integer."], 
  "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockFast"}],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockFast"}],

Cell[BoxData[
    \(Needs["\<ISP`Blocks`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  ShowCellBracket->True,
  CellTags->{"Blocks", "BlockFast"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlockFast[15]\)], "Input",
  CellLabel->"In[2]:=",
  ShowCellBracket->True,
  CellTags->{"Blocks", "BlockFast"}],

Cell[BoxData[
    \(1.65911449661449665`\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"Blocks", "BlockFast"}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["BlockFloat", "Subsection",
  CellTags->{"BlockFloat", "Blocks"}],

Cell[CellGroupData[{

Cell[TextData[
"\[FilledSmallSquare] BlockFloat[n] sums the first n even terms of the \
harmonic series using floating-point arithmetic."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"BlockFloat", "Blocks"}],

Cell[TextData[
"\[FilledSmallSquare] The parameter n must be a non-negative integer."], 
  "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockFloat"}],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"BlockFloat", "Blocks"}],

Cell[BoxData[
    \(Needs["\<ISP`Blocks`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"BlockFloat", "Blocks"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlockFloat[15]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"BlockFloat", "Blocks"}],

Cell[BoxData[
    \(1.65911449661449665`\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"BlockFloat", "Blocks"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["BlockPrecision", "Subsection",
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[CellGroupData[{

Cell[TextData[
"\[FilledSmallSquare] BlockPrecision[n, p] sums the first n even terms of the \
harmonic  series from left to right if n is non-negative.\n\
\[FilledSmallSquare] BlockPrecision[n,p] sums the first |n| even terms of the \
harmonic series from right to left if n is negative."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[TextData[{
  StyleBox["\[FilledSmallSquare]",
    FontFamily->"Math1"],
  " The parameter n must be an integer.\n",
  StyleBox["\[FilledSmallSquare]",
    FontFamily->"Math1"],
  " The parameter p must be a positive integer.\n",
  StyleBox["\[FilledSmallSquare]",
    FontFamily->"Math1"],
  " If p is between 1 and 10, inclusive, BlockPrecision uses simulated \
floating-point arithmetic with p-digit mantissas.\n",
  StyleBox["\[FilledSmallSquare]",
    FontFamily->"Math1"],
  " If p is larger than $MachinePrecision, BlockPrecision uses arbitrary \
floating-point arithmetic with a precision of p.\n",
  StyleBox["\[FilledSmallSquare]",
    FontFamily->"Math1"],
  " Otherwise, BlockPrecision uses floating-point arithmetic."
}], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[BoxData[
    \(Needs["\<ISP`Blocks`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  ShowCellBracket->True,
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlockPrecision[100, \ 3]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[BoxData[
    TagBox[
      InterpretationBox[\("2.720000000000000000"\),
        2.72,
        AutoDelete->True],
      (BaseForm[ #, 10]&)]], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlockPrecision[\(-100\), \ 3]\)], "Input",
  CellLabel->"In[3]:=",
  ShowCellBracket->True,
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[BoxData[
    TagBox[
      InterpretationBox[\("2.600000000000000000"\),
        2.6,
        AutoDelete->True],
      (BaseForm[ #, 10]&)]], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlockFloat[100, \ 16]\)], "Input",
  CellLabel->"In[4]:=",
  ShowCellBracket->True,
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[BoxData[
    \(2.59368875881981031`\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlockPrecision[100, \ 20]\)], "Input",
  CellLabel->"In[5]:=",
  ShowCellBracket->True,
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[BoxData[
    \(2.5936887588198101304025588378291`20\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["BlockRat", "Subsection",
  CellTags->{"Blocks", "BlockRat"}],

Cell[TextData[
"\[FilledSmallSquare] BlockRat[n] sums the first n even terms of the harmonic \
series using rational arithmetic."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockRat"}],

Cell[TextData[
"\[FilledSmallSquare] The parameter n must be a non-negative integer."], 
  "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockRat"}],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Blocks", "BlockRat"}],

Cell[BoxData[
    \(Needs["\<ISP`Blocks`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"Blocks", "BlockRat"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlockRat[15]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"Blocks", "BlockRat"}],

Cell[BoxData[
    \(1195757\/720720\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"Blocks", "BlockRat"}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell["", "Text"],

Cell["Integ", "Section",
  CellTags->"Integ"],

Cell["\<\
The Integ package provides two functions useful for visualizing numerical \
integration, as discussed in Chapter 15 of Introduction to Scientific \
Programming.
Before the functions can be used, the package must be loaded with\
\>", "Text",
  CellTags->"Integ"],

Cell[BoxData[
    \(\(\nNeeds["\<ISP`Integ`\>"]\)\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"Integ"],

Cell["AnimateRectangular", "Subsection",
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[TextData[
"\[FilledSmallSquare] AnimateRectangular[g, a, b, n] produces an animation \
with n frames illustrating the use of the rectangular method to approximte \
the integral of g[x] between x=a and x=b."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[TextData[
"\[FilledSmallSquare] The parameter g must be a single-parameter, real-valued \
function.\n\[FilledSmallSquare] The parameters a and b must be real-valued, \
with a < b.\n\[FilledSmallSquare] The paramter n must be a positive \
integer."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Integ", "AnimateRectangular"}],

Cell["Examples", "Subsubsection",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[BoxData[
    \(Needs["\<ISP`Integ`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[BoxData[
    \(f[x_]\  := \ Sin[x] + 2\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnimateRectangular[f, \ 0, \ 8, \ 3]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.0147151 0.196201 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11282 -18 -4.5 ]
[.01131 .11282 0 4.5 ]
[.01131 .21092 -6 -4.5 ]
[.01131 .21092 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .50522 -18 -4.5 ]
[.01131 .50522 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.2619 .01472 m
.2619 .02097 L
s
[(2)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(4)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(6)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(8)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11282 m
.03006 .11282 L
s
[(0.5)] .01131 .11282 1 0 Mshowa
.02381 .21092 m
.03006 .21092 L
s
[(1)] .01131 .21092 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1.5)] .01131 .30902 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(2)] .01131 .40712 1 0 Mshowa
.02381 .50522 m
.03006 .50522 L
s
[(2.5)] .01131 .50522 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .0932 m
.02756 .0932 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .32864 m
.02756 .32864 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .56408 m
.02756 .56408 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02381 .25863 L
.5 .25863 L
.5 .01472 L
.5 .60123 L
.97619 .60123 L
.97619 .01472 L
s
.02381 .40712 m
.06244 .46968 L
.10458 .53025 L
.12507 .55459 L
.14415 .57336 L
.16254 .58741 L
.17284 .59342 L
.18221 .59768 L
.18704 .59942 L
.19216 .60092 L
.19654 .60191 L
.19901 .60236 L
.20135 .6027 L
.20265 .60286 L
.20405 .603 L
.20535 .60311 L
.20652 .60319 L
.20719 .60323 L
.20792 .60326 L
.20869 .60329 L
.20941 .60331 L
.21009 .60332 L
.2108 .60332 L
.21209 .60331 L
.21328 .60328 L
.21456 .60322 L
.21577 .60315 L
.21688 .60306 L
.21958 .60279 L
.22214 .60243 L
.22673 .60157 L
.23179 .60028 L
.24205 .5966 L
.25155 .59194 L
.26047 .5865 L
.28065 .57051 L
.29915 .55173 L
.33878 .50047 L
.37689 .44142 L
.41745 .3749 L
.45649 .31427 L
.47641 .28679 L
.49799 .26082 L
.51703 .24187 L
.52785 .23295 L
.53797 .2259 L
.54836 .22004 L
.55785 .21593 L
.56305 .21418 L
.56799 .21287 L
Mistroke
.57321 .21185 L
.57616 .21143 L
.57749 .21129 L
.57889 .21116 L
.58016 .21107 L
.58083 .21103 L
.58155 .21099 L
.58281 .21094 L
.58398 .21092 L
.58469 .21092 L
.58546 .21092 L
.58619 .21093 L
.58686 .21095 L
.58814 .21099 L
.58878 .21103 L
.58949 .21107 L
.59196 .21127 L
.59336 .21142 L
.59463 .21158 L
.59945 .2124 L
.60379 .21341 L
.60853 .2148 L
.61807 .21853 L
.62838 .22391 L
.63772 .22998 L
.65869 .2475 L
.69872 .29409 L
.73724 .35094 L
.77821 .41767 L
.81767 .48084 L
.85561 .5341 L
.87673 .5585 L
.896 .57664 L
.90555 .58401 L
.91595 .59074 L
.92571 .59578 L
.93487 .59937 L
.93939 .60071 L
.94426 .60186 L
.94689 .60234 L
.94966 .60274 L
.95203 .603 L
.95337 .60311 L
.95462 .6032 L
.95593 .60326 L
.95667 .60329 L
.95736 .6033 L
.95861 .60332 L
.95924 .60332 L
.95994 .60331 L
Mistroke
.96114 .60328 L
.96244 .60323 L
.96366 .60316 L
.96477 .60307 L
.96743 .6028 L
.97034 .6024 L
.97562 .60136 L
.97619 .60123 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3ooooooooQ8N4005CQ0`0oh@<0?N4203oQ00<0hN40
1^4005CQ00<0hN40@>400`3Qh@0lh@030>4003gQ00<0h@001n4005GQ00<0hN40?N4403gQ00<0h@00
?N400`3Q0007h@00EN400`3Qh@0mh@030>4003kQ0P0oh@030>7Q00KQ001Dh@030>4003oQ0P0nh@03
0>7Q03gQ00<0h@001n4005GQ00<0hN40?n400`3Qh@0mh@80?^400`3Qh@06h@00on4Qh@005N400`3Q
h@3oh@WQ000Eh@030>7Q0?oQ2N4001GQ00<0hN40on49h@005N400`3Qh@3oh@WQ000?hOl04@01h@00
5N400`3Qh@0=h@030>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@03
0>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@030>7Q00kQ00<0hN40
3N400`3Qh@0=h@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N4207oQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40
O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@03
0>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@80On400`3Qh@1oh@03
0>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN40
1N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N4207oQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40
O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@03
0>7Q07oQ00<0hN401N4001GQ0P1oh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@03
0>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN40
1N4000GQ00<0hN400^400`3Qh@02h@030>7Q00?Q00<0hN40O^400`3Qh@1oh@030>7Q00GQ0004h@03
0>4000OQ00<0h@001>400`3Qh@1nh@030>7Q07oQ00<0hN401N4000CQ00<0h@002N400`3Qh@02h@<0
O^400`3Qh@1oh@030>7Q00GQ0004h@030>4000OQ0P05h@030>7Q07kQ00<0hN40On400`3Qh@05h@00
1>400`3Q0007h@030>7Q00CQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ0005h@030>7Q00KQ0`04h@03
0>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ0P1oh@03
0>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40
On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@03
0>7Q00GQ000Eh@80On400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@03
0>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ0P1oh@03
0>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40
On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@03
0>7Q00GQ000Eh@80On400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q07oQ00<0hN401N4000gQ1@03h@030>7Q07kQ00<0hN40On400`3Qh@05h@00
3n400`3Qh@03h@030>7Q07kQ00<0hN40On400`3Qh@05h@003n400`3Qh@03h@<0O^400`3Qh@0Bh@H0
In400`3Qh@05h@003n400`3Qh@03h@030>7Q07kQ00<0hN403n4300KQ1@1Rh@030>7Q00GQ000?h@03
0>7Q00?Q00<0hN40O^400`3Qh@0=h@803^42063Q00<0hN401N4000kQ0P05h@030>7Q07kQ00<0hN40
2^4301;Q00<0hN40GN400`3Qh@05h@005N400`3Qh@1nh@030>7Q00SQ0P0Fh@80GN400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q00KQ0P0Jh@80Fn400`3Qh@05h@005N4207oQ00<0hN401N400`3Qh@0Lh@03
0>7Q05SQ00<0hN401N4001GQ00<0hN40O^400`3Qh@04h@030>7Q01kQ00<0hN40En400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q00?Q00<0hN408>4205OQ00<0hN401N4001GQ00<0hN40O^400`3Qh@02h@03
0>7Q02?Q00<0hN40E>400`3Qh@05h@005N400`3Qh@1nh@050>7Qh@009n400`3Qh@1Ch@030>7Q00GQ
000Eh@80On40103Qh@0Yh@030>7Q05;Q00<0hN401N4001GQ00<0hN40O^400`3Q000[h@030>7Q057Q
00<0hN401N4001GQP`0/h@030>7Q057Q00<0hN401N4001GQ00<0hN40O^400`3Qh@0/h@030>7Q053Q
00<0hN401N4001GQ00<0hN40ON4202oQ00<0hN40Cn400`3Qh@05h@005N4207gQ00<0h@00<>400`3Q
h@1>h@030>7Q00GQ000Eh@030>7Q07_Q00@0hN40<>400`3Qh@1>h@030>7Q00GQ000Eh@030>7Q07_Q
00@0hN40<N400`3Qh@1=h@030>7Q00GQ000Eh@030>7Q07[Q00D0hN7Q000bh@030>7Q04cQ00<0hN40
1N4001GQ00<0hN40NN400`3Qh@02h@030>7Q037Q00<0hN40Bn400`3Qh@05h@005N400`3Qh@1hh@03
0>7Q00?Q00<0hN40<N400`3Qh@1;h@030>7Q00GQ000Eh@80N>400`3Qh@04h@030>7Q03;Q00<0hN40
B^400`3Qh@05h@005N400`3Qh@1gh@030>7Q00CQ00<0hN40<n400`3Qh@19h@030>7Q00GQ000Eh@03
0>7Q07KQ00<0hN401N400`3Qh@0dh@030>7Q04SQ00<0hN401N4000?Q1@02h@030>7Q00;Q00<0hN40
0n400`3Qh@1eh@030>7Q00KQ00<0hN40=>400`3Qh@18h@030>7Q00GQ0005h@030>7Q00KQ00<0h@00
1>400`3Qh@1dh@030>7Q00OQ00<0hN40=N400`3Qh@17h@030>7Q00GQ0005h@030>7Q00SQ00<0hN40
0^4307CQ00<0hN401n400`3Qh@0fh@030>7Q04KQ00<0hN401N4000GQ00<0hN401^4200GQ00<0hN40
Ln400`3Qh@08h@030>7Q03OQ00<0hN40AN400`3Qh@05h@001N400`3Qh@06h@030>7Q00CQ00<0hN40
L^400`3Qh@09h@030>7Q03OQ00<0hN40AN400`3Qh@05h@001>4200SQ0`04h@030>7Q07;Q00<0hN40
2N400`3Qh@0hh@030>7Q04CQ00<0hN401N4001GQ00<0hN40LN400`3Qh@0:h@030>7Q03WQ00<0hN40
@n400`3Qh@05h@005N42077Q00<0hN402n400`3Qh@0jh@030>7Q04;Q00<0hN401N4001GQ00<0hN40
L>400`3Qh@0;h@030>7Q03[Q00<0hN40@^400`3Qh@05h@005N400`3Qh@1_h@030>7Q00cQ00<0hN40
>n400`3Qh@11h@030>7Q00GQ000Eh@030>7Q06kQ00<0hN403N400`3Qh@0lh@030>7Q043Q00<0hN40
1N4001GQ00<0hN40K^400`3Qh@0=h@030>7Q03gQ00<0hN40?n400`3Qh@05h@005N400`3Qh@1]h@03
0>7Q00kQ00<0hN40?N400`3Qh@0oh@030>7Q00GQ000Eh@80K^400`3Qh@0>h@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N400`3Qh@1/h@030>7Q00oQ00<0hN40?n400`3Qh@0mh@030>7Q00GQ000Eh@03
0>7Q06_Q00<0hN404>400`3Qh@0oh@030>7Q03gQ00<0hN401N4001GQ00<0hN40Jn400`3Qh@0@h@03
0>7Q043Q00<0hN40?>400`3Qh@05h@005N400`3Qh@1Zh@030>7Q017Q00<0hN40@>400`3Qh@0lh@03
0>7Q00GQ000Eh@80J^400`3Qh@0Bh@030>7Q047Q00<0hN40>n400`3Qh@05h@005N400`3Qh@1Yh@03
0>7Q01;Q00<0hN40@N400`3Qh@0kh@030>7Q00GQ000Eh@030>7Q06SQ00<0hN404n400`3Qh@12h@03
0>7Q03[Q00<0hN401N4001GQ00<0hN40In400`3Qh@0Dh@030>7Q04?Q00<0hN40>N400`3Qh@05h@00
5N400`3Qh@1Wh@030>7Q01CQ00<0hN40@n400`3Qh@0ih@030>7Q00GQ000Eh@80In400`3Qh@0Eh@03
0>7Q04CQ00<0hN40>>400`3Qh@05h@005N400`3Qh@1Vh@030>7Q01GQ00<0hN40A>400`3Qh@0hh@03
0>7Q00GQ000Eh@030>7Q06GQ00<0hN405^400`3Qh@15h@030>7Q03OQ00<0hN401N4001GQ00<0hN40
I>400`3Qh@0Gh@030>7Q04KQ00<0hN40=^400`3Qh@05h@003^4300CQ00<0hN40I>400`3Qh@0Gh@03
0>7Q04KQ00<0hN40=^400`3Qh@05h@003^400`3Qh@04h@030>7Q06?Q00<0hN406>400`3Qh@17h@03
0>7Q03GQ00<0hN401N4000oQ00<0hN400n4306;Q00<0hN406N400`3Qh@17h@030>7Q03GQ00<0hN40
1N4000oQ00<0hN400n4206?Q00<0hN406N400`3Qh@18h@030>7Q03CQ00<0hN401N4000kQ00<0h@00
1>4206;Q00<0hN406^400`3Qh@18h@030>7Q03CQ00<0hN401N4000oQ00<0hN400n400`3Q001Qh@03
0>7Q01[Q00<0hN40BN400`3Qh@0ch@030>7Q00GQ000Eh@030>40063Q00<0hN406n400`3Qh@1:h@03
0>7Q03;Q00<0hN401N4001GQ0P000n40h@1Mh@030>7Q01cQ00<0hN40B^400`3Qh@0bh@030>7Q00GQ
000Eh@050>7Qh@00GN400`3Qh@0Lh@030>7Q04_Q00<0hN40<N400`3Qh@05h@005N401@3QhN4005cQ
00<0hN407N400`3Qh@1<h@030>7Q033Q00<0hN401N4001GQ00<0hN400^400`3Qh@1Ih@030>7Q01gQ
00<0hN40C>400`3Qh@0`h@030>7Q00GQ000Eh@030>7Q00;Q00<0hN40F>400`3Qh@0Nh@030>7Q04gQ
00<0hN40;n400`3Qh@05h@005N4200CQ00<0hN40E^400`3Qh@0Oh@030>7Q04kQ00<0hN40;^400`3Q
h@05h@005N400`3Qh@03h@030>7Q05KQ00<0hN407n400`3Qh@1>h@030>7Q02kQ00<0hN401N4001GQ
00<0hN401>400`3Qh@1Dh@030>7Q023Q00<0hN40Cn400`3Qh@0]h@030>7Q00GQ000Eh@030>7Q00GQ
00<0hN40D^400`3Qh@0Qh@030>7Q04oQ00<0hN40;N400`3Qh@05h@005N400`3Qh@05h@030>7Q05;Q
00<0hN408N400`3Qh@1@h@030>7Q02cQ00<0hN401N4001GQ00<0hN401^400`3Qh@1@h@030>7Q02;Q
00<0hN40DN400`3Qh@0[h@030>7Q00GQ000Eh@801n400`3Qh@1@h@030>7Q02;Q00<0hN40DN400`3Q
h@0[h@030>7Q00GQ000Eh@030>7Q00OQ00<0hN40C^400`3Qh@0Sh@030>7Q05;Q00<0hN40:^400`3Q
h@05h@005N400`3Qh@08h@030>7Q04cQ00<0hN409>400`3Qh@1Ch@030>7Q02WQ00<0hN401N4001GQ
00<0hN402>400`3Qh@1<h@030>7Q02CQ00<0hN40Dn400`3Qh@0Yh@030>7Q00GQ000Eh@030>7Q00WQ
00<0hN40B^400`3Qh@0Uh@030>7Q05CQ00<0hN40:>400`3Qh@05h@005N4200_Q00<0hN40B>400`3Q
h@0Vh@030>7Q05GQ00<0hN409n400`3Qh@05h@005N400`3Qh@0;h@030>7Q04OQ00<0hN409^400`3Q
h@1Eh@030>7Q02OQ00<0hN401N4001GQ00<0hN402n400`3Qh@16h@030>7Q02OQ00<0hN40E^400`3Q
h@0Vh@030>7Q00GQ0004h@<00n400`3Qh@02h@030>7Q00?Q00<0hN403>400`3Qh@15h@030>7Q02OQ
00<0hN40En400`3Qh@0Uh@030>7Q00GQ0004h@030>7Q00OQ00<0h@001>400`3Qh@0=h@030>7Q04?Q
00<0hN40:>400`3Qh@1Hh@030>7Q02CQ00<0hN401N4000GQ00<0hN402>400`3Qh@02h@<03^400`3Q
h@11h@030>7Q02WQ00<0hN40F>400`3Qh@0Th@030>7Q00GQ0005h@030>7Q00KQ0P05h@030>7Q00kQ
00<0hN40@>400`3Qh@0Zh@030>7Q05WQ00<0hN408n400`3Qh@05h@001>400`3Q0007h@030>7Q00CQ
00<0hN403n400`3Qh@0oh@030>7Q02[Q00<0hN40F^400`3Qh@0Rh@030>7Q00GQ0005h@030>7Q00KQ
0`04h@030>7Q013Q00<0hN40?N400`3Qh@0[h@030>7Q05[Q00<0hN408^400`3Qh@05h@005N400`3Q
h@0Ah@030>7Q03_Q00<0hN40;>400`3Qh@1Kh@030>7Q027Q00<0hN401N4001GQ00<0hN404N400`3Q
h@0jh@030>7Q02gQ00<0hN40G>400`3Qh@0Ph@030>7Q00GQ000Eh@804n400`3Qh@0hh@030>7Q02kQ
00<0hN40GN400`3Qh@0Oh@030>7Q00GQ000Eh@030>7Q01?Q00<0hN40=n400`3Qh@0^h@030>7Q05gQ
00<0hN407n400`3Qh@05h@005N400`3Qh@0Dh@030>7Q03GQ00<0hN40;n400`3Qh@1Nh@030>7Q01kQ
00<0hN401N4001GQ00<0hN405N400`3Qh@0ch@030>7Q033Q00<0hN40Gn400`3Qh@0Mh@030>7Q00GQ
000Eh@030>7Q01KQ00<0hN40<N400`3Qh@0ah@030>7Q063Q00<0hN407>400`3Qh@05h@005N4201OQ
00<0hN40<N400`3Qh@0ah@030>7Q067Q00<0hN406n400`3Qh@05h@005N400`3Qh@0Gh@030>7Q02oQ
00<0hN40<^400`3Qh@1Qh@030>7Q01_Q00<0hN401N4001GQ00<0hN406>400`3Qh@0]h@030>7Q03?Q
00<0hN40H^400`3Qh@0Jh@030>7Q00GQ000Eh@030>7Q01WQ00<0hN40:n400`3Qh@0dh@030>7Q06?Q
00<0hN406N400`3Qh@05h@005N400`3Qh@0Jh@030>7Q02WQ00<0hN40=N400`3Qh@1Th@030>7Q01SQ
00<0hN401N4001GQ0P0Lh@030>7Q02OQ00<0hN40=^400`3Qh@1Uh@030>7Q01OQ00<0hN401N4001GQ
00<0hN407>400`3Qh@0Uh@030>7Q03OQ00<0hN40I^400`3Qh@0Fh@030>7Q00GQ000Eh@030>7Q01gQ
00<0hN408n400`3Qh@0hh@030>7Q06OQ00<0hN405N400`3Qh@05h@005N400`3Qh@0Nh@030>7Q023Q
0P0kh@030>7Q06SQ00<0hN405>400`3Qh@05h@005N400`3Qh@0Oh@030>7Q01kQ00<0hN40>n400`3Q
h@1Yh@030>7Q01?Q00<0hN401N4001GQ00<0hN408>4201cQ0P0nh@030>7Q06[Q0P0Ch@030>7Q00GQ
000Eh@808n400`3Qh@0Hh@030>7Q03kQ00<0hN40K>400`3Qh@0@h@030>7Q00GQ000Eh@030>7Q02?Q
0P0Gh@030>7Q03oQ00<0hN40KN42013Q00<0hN401N4001GQ00<0hN409N4201?Q0P12h@030>7Q06oQ
00<0hN403N400`3Qh@05h@003^4200GQ00<0hN409n4200oQ0P14h@030>7Q073Q0`0<h@030>7Q00GQ
000@h@030>7Q00;Q00<0hN40:N4300[Q0P16hH<01n4000oQ00<0hN400n4302cQ2P31h@T02>40013Q
00<0hN400^400`3Qh@3oh@WQ000>h@030>4000CQ00<0hN40on49h@003n400`3Qh@03h@030>7Q0?oQ
2N4001GQ00<0hN40on49h@00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.66388, -0.23241, 
  0.0308942, 0.0187455}},
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.0147151 0.196201 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11282 -18 -4.5 ]
[.01131 .11282 0 4.5 ]
[.01131 .21092 -6 -4.5 ]
[.01131 .21092 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .50522 -18 -4.5 ]
[.01131 .50522 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.2619 .01472 m
.2619 .02097 L
s
[(2)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(4)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(6)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(8)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11282 m
.03006 .11282 L
s
[(0.5)] .01131 .11282 1 0 Mshowa
.02381 .21092 m
.03006 .21092 L
s
[(1)] .01131 .21092 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1.5)] .01131 .30902 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(2)] .01131 .40712 1 0 Mshowa
.02381 .50522 m
.03006 .50522 L
s
[(2.5)] .01131 .50522 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .0932 m
.02756 .0932 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .32864 m
.02756 .32864 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .56408 m
.02756 .56408 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02381 .58552 L
.2619 .58552 L
.2619 .01472 L
.2619 .25863 L
.5 .25863 L
.5 .01472 L
.5 .3523 L
.7381 .3523 L
.7381 .01472 L
.7381 .60123 L
.97619 .60123 L
.97619 .01472 L
s
.02381 .40712 m
.06244 .46968 L
.10458 .53025 L
.12507 .55459 L
.14415 .57336 L
.16254 .58741 L
.17284 .59342 L
.18221 .59768 L
.18704 .59942 L
.19216 .60092 L
.19654 .60191 L
.19901 .60236 L
.20135 .6027 L
.20265 .60286 L
.20405 .603 L
.20535 .60311 L
.20652 .60319 L
.20719 .60323 L
.20792 .60326 L
.20869 .60329 L
.20941 .60331 L
.21009 .60332 L
.2108 .60332 L
.21209 .60331 L
.21328 .60328 L
.21456 .60322 L
.21577 .60315 L
.21688 .60306 L
.21958 .60279 L
.22214 .60243 L
.22673 .60157 L
.23179 .60028 L
.24205 .5966 L
.25155 .59194 L
.26047 .5865 L
.28065 .57051 L
.29915 .55173 L
.33878 .50047 L
.37689 .44142 L
.41745 .3749 L
.45649 .31427 L
.47641 .28679 L
.49799 .26082 L
.51703 .24187 L
.52785 .23295 L
.53797 .2259 L
.54836 .22004 L
.55785 .21593 L
.56305 .21418 L
.56799 .21287 L
Mistroke
.57321 .21185 L
.57616 .21143 L
.57749 .21129 L
.57889 .21116 L
.58016 .21107 L
.58083 .21103 L
.58155 .21099 L
.58281 .21094 L
.58398 .21092 L
.58469 .21092 L
.58546 .21092 L
.58619 .21093 L
.58686 .21095 L
.58814 .21099 L
.58878 .21103 L
.58949 .21107 L
.59196 .21127 L
.59336 .21142 L
.59463 .21158 L
.59945 .2124 L
.60379 .21341 L
.60853 .2148 L
.61807 .21853 L
.62838 .22391 L
.63772 .22998 L
.65869 .2475 L
.69872 .29409 L
.73724 .35094 L
.77821 .41767 L
.81767 .48084 L
.85561 .5341 L
.87673 .5585 L
.896 .57664 L
.90555 .58401 L
.91595 .59074 L
.92571 .59578 L
.93487 .59937 L
.93939 .60071 L
.94426 .60186 L
.94689 .60234 L
.94966 .60274 L
.95203 .603 L
.95337 .60311 L
.95462 .6032 L
.95593 .60326 L
.95667 .60329 L
.95736 .6033 L
.95861 .60332 L
.95924 .60332 L
.95994 .60331 L
Mistroke
.96114 .60328 L
.96244 .60323 L
.96366 .60316 L
.96477 .60307 L
.96743 .6028 L
.97034 .6024 L
.97562 .60136 L
.97619 .60123 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3ooooooooQ8N4005CQ0`0oh@<0?N4203oQ00<0hN40
1^4005CQ00<0hN40@>400`3Qh@0lh@030>4003gQ00<0h@001n4005GQ00<0hN40?N4403gQ00<0h@00
?N400`3Q0007h@00EN400`3Qh@0mh@030>4003kQ0P0oh@030>7Q00KQ001Dh@030>4003oQ0P0nh@03
0>7Q03gQ00<0h@001n4005GQ00<0hN40?n400`3Qh@0mh@80?^400`3Qh@06h@00on4Qh@005N400`3Q
h@3oh@WQ000Eh@030>7Q0?oQ2N4001GQ00<0hN40on49h@005N400`3Qh@3oh@WQ000?hOl04@01h@00
5N400`3Qh@0=h@030>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@03
0>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@030>7Q00kQ00<0hN40
3N400`3Qh@0=h@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@03
0>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@03
0>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@03
0>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ0P0oh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@03
0>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@00
5N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ0P0oh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@03
0>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40
?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40
?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ0P0oh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ0005h@03
0>7Q00;Q00<0hN400^400`3Qh@03h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4000CQ00<0h@001n400`3Q0004h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4000CQ00<0h@002N400`3Qh@02h@<0?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@00
1>400`3Q0007h@801N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ0004h@03
0>4000OQ00<0hN401>400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ0005h@03
0>7Q00KQ0`04h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40
?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N4203oQ00<0hN40?N400`3Qh@0nh@03
0>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@00
5N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N4203oQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40
?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40
?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@03
0>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@03
0>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@03
0>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@03
0>7Q03kQ00<0hN401N4000gQ1@03h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4000oQ00<0hN400n400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000?h@03
0>7Q00?Q0`0nh@030>7Q03gQ00<0hN404^4602KQ00<0hN40?^400`3Qh@05h@003n400`3Qh@03h@03
0>7Q03kQ00<0hN40?N400`3Qh@0?h@<01^45027Q00<0hN40?^400`3Qh@05h@003n400`3Qh@03h@03
0>7Q03kQ00<0hN40?N400`3Qh@0=h@803^4201oQ00<0hN40?^400`3Qh@05h@003^4200GQ00<0hN40
?^400`3Qh@0mh@030>7Q00[Q0`0Bh@030>7Q01cQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN402>4201KQ0P0Lh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@03
0>7Q00KQ0P0Jh@806^400`3Qh@0nh@030>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q00GQ00<0hN40
7>400`3Qh@0Gh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q00CQ00<0hN40
7^400`3Qh@0Fh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q00?Q00<0hN40
8>4201KQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN400^400`3Qh@0Sh@03
0>7Q01?Q00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00D0hN7Q000Wh@030>7Q01;Q
00<0hN40?^400`3Qh@05h@005N4203oQ00<0hN40?N40103Qh@0Yh@030>7Q017Q00<0hN40?^400`3Q
h@05h@005N400`3Qh@0nh@030>7Q03gQ00<0h@00:n400`3Qh@0@h@030>7Q03kQ00<0hN401N4001GQ
00<0hN40?^5202cQ00<0hN404>400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Q
h@0/h@030>7Q00oQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03cQ0P0_h@030>7Q00kQ
00<0hN40?^400`3Qh@05h@005N4203oQ00<0hN40>n400`3Q000`h@030>7Q00gQ00<0hN40?^400`3Q
h@05h@005N400`3Qh@0nh@030>7Q03[Q00@0hN40<>400`3Qh@0=h@030>7Q03kQ00<0hN401N4001GQ
00<0hN40?^400`3Qh@0jh@040>7Q037Q00<0hN403>400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ
00<0hN40>N401@3QhN4003;Q00<0hN402n400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
>>400`3Qh@02h@030>7Q037Q00<0hN402^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
=n400`3Qh@03h@030>7Q037Q00<0hN402^400`3Qh@0nh@030>7Q00GQ000Eh@80?n400`3Qh@0fh@03
0>7Q00CQ00<0hN40<^400`3Qh@09h@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0fh@03
0>7Q00CQ00<0hN40<n400`3Qh@08h@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0eh@03
0>7Q00GQ00<0hN40=>400`3Qh@07h@030>7Q03kQ00<0hN401N4000?Q1@02h@030>7Q00;Q00<0hN40
0n400`3Qh@0nh@030>7Q03CQ00<0hN401^400`3Qh@0dh@030>7Q00OQ00<0hN40?^400`3Qh@05h@00
1N400`3Qh@06h@030>4000CQ00<0hN40?^400`3Qh@0ch@030>7Q00OQ00<0hN40=N400`3Qh@06h@03
0>7Q03kQ00<0hN401N4000GQ00<0hN402>400`3Qh@02h@<0?^400`3Qh@0ch@030>7Q00OQ00<0hN40
=^400`3Qh@05h@030>7Q03kQ00<0hN401N4000GQ00<0hN401^4200GQ00<0hN40?^400`3Qh@0bh@03
0>7Q00SQ00<0hN40=n400`3Qh@04h@030>7Q03kQ00<0hN401N4000GQ00<0hN401^400`3Qh@04h@03
0>7Q03kQ00<0hN40<N400`3Qh@09h@030>7Q03OQ00<0hN401>400`3Qh@0nh@030>7Q00GQ0004h@80
2>4300CQ00<0hN40?^400`3Qh@0ah@030>7Q00WQ00<0hN40>>400`3Qh@03h@030>7Q03kQ00<0hN40
1N4001GQ00<0hN40?^400`3Qh@0`h@030>7Q00[Q00<0hN40>N400`3Qh@02h@030>7Q03kQ00<0hN40
1N4001GQ0P0oh@030>7Q02oQ00<0hN402n400`3Qh@0jh@050>7Qh@00@>400`3Qh@05h@005N400`3Q
h@0nh@030>7Q02oQ00<0hN402n400`3Qh@0jh@050>7Qh@00@>400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q02kQ00<0hN403>400`3Qh@0kh@040>7Q043Q00<0hN401N4001GQ00<0hN40?^400`3Qh@0]h@03
0>7Q00gQ00<0hN40?>400`3Q0010h@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40;N400`3Qh@0=h@03
0>7Q03gQ0P10h@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40;>400`3Qh@0>h@030>7Q03gQ0P10h@03
0>7Q00GQ000Eh@80?n400`3Qh@0/h@030>7Q00kQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@03
0>7Q03kQ00<0hN40:n400`3Qh@0?hD<0?n400`3Qh@05h@005N400`3Qh@0nh@030>7Q02[Q00<0hN40
DN4203oQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0Zh@030>7Q057Q00<0h@00?^400`3Qh@05h@00
5N400`3Qh@0nh@030>7Q02WQ00<0hN40D^400`3Q000nh@030>7Q00GQ000Eh@80?n400`3Qh@0Xh@03
0>7Q05?Q00@0hN40?N400`3Qh@05h@005N400`3Qh@0nh@030>7Q02SQ00<0hN40Dn40103Qh@0mh@03
0>7Q00GQ000Eh@030>7Q03kQ00<0hN409n400`3Qh@1Dh@050>7Qh@00?>400`3Qh@05h@005N400`3Q
h@0nh@030>7Q02KQ00<0hN40EN400`3Qh@02h@030>7Q03WQ00<0hN401N4001GQ00<0hN40?^400`3Q
h@0Vh@030>7Q05GQ00<0hN400^400`3Qh@0ih@030>7Q00GQ000Eh@80?n400`3Qh@0Uh@030>7Q05KQ
00<0hN400n400`3Qh@0hh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN409N400`3Qh@1Fh@030>7Q00?Q
00<0hN40>>400`3Qh@05h@005N400`3Qh@0nh@030>7Q02CQ00<0hN40En400`3Qh@04h@030>7Q03OQ
00<0hN401N4001GQ00<0hN40?^400`3Qh@0Sh@030>7Q05SQ00<0hN401N400`3Qh@0fh@030>7Q00GQ
000>h@<01>400`3Qh@0nh@030>7Q02?Q00<0hN40F>400`3Qh@05h@030>7Q03KQ00<0hN401N4000kQ
00<0hN401>400`3Qh@0nh@030>7Q02;Q00<0hN40FN400`3Qh@06h@030>7Q03GQ00<0hN401N4000oQ
00<0hN400n4303kQ00<0hN408N400`3Qh@1Jh@030>7Q00KQ00<0hN40=N400`3Qh@05h@003n400`3Q
h@03h@80?n400`3Qh@0Qh@030>7Q05[Q00<0hN401n400`3Qh@0dh@030>7Q00GQ000>h@030>4000CQ
0P0oh@030>7Q023Q00<0hN40Fn400`3Qh@07h@030>7Q03CQ00<0hN401N4000oQ00<0hN400n400`3Q
000nh@030>7Q023Q00<0hN40Fn400`3Qh@08h@030>7Q03?Q00<0hN401N4001GQ00<0h@00?^400`3Q
h@0Oh@030>7Q05cQ00<0hN402N400`3Qh@0bh@030>7Q00GQ000Eh@8000?Q0>40?>400`3Qh@0Nh@03
0>7Q05gQ00<0hN402N400`3Qh@0bh@030>7Q00GQ000Eh@050>7Qh@00?>400`3Qh@0Nh@030>7Q05gQ
00<0hN402^400`3Qh@0ah@030>7Q00GQ000Eh@050>7Qh@00?>400`3Qh@0Mh@030>7Q05kQ00<0hN40
2n400`3Qh@0`h@030>7Q00GQ000Eh@030>7Q00;Q00<0hN40>N400`3Qh@0Mh@030>7Q05kQ00<0hN40
2n400`3Qh@0`h@030>7Q00GQ000Eh@030>7Q00;Q00<0hN40>N400`3Qh@0Lh@030>7Q05oQ00<0hN40
3>400`3Qh@0_h@030>7Q00GQ000Eh@801>400`3Qh@0hh@030>7Q01_Q00<0hN40H>400`3Qh@0=h@03
0>7Q02kQ00<0hN401N4001GQ00<0hN400n400`3Qh@0hh@030>7Q01_Q00<0hN40H>400`3Qh@0=h@03
0>7Q02kQ00<0hN401N4001GQ00<0hN401>400`3Qh@0gh@030>7Q01[Q00<0hN40HN400`3Qh@0>h@03
0>7Q02gQ00<0hN401N4001GQ00<0hN401N400`3Qh@0fh@030>7Q01WQ00<0hN40H^400`3Qh@0>h@03
0>7Q02gQ00<0hN401N4001GQ00<0hN401N400`3Qh@0fh@030>7Q01WQ00<0hN40H^400`3Qh@0?h@03
0>7Q02cQ00<0hN401N4001GQ00<0hN401^400`3Qh@0eh@030>7Q01SQ00<0hN40Hn400`3Qh@0@h@03
0>7Q02_Q00<0hN401N4001GQ0P07h@030>7Q03GQ00<0hN406>400`3Qh@1Sh@030>7Q013Q00<0hN40
:n400`3Qh@05h@005N400`3Qh@07h@030>7Q03CQ00<0hN405n400`3Qh@1Th@030>7Q017Q00<0hN40
:^400`3Qh@05h@005N400`3Qh@08h@030>7Q03?Q00<0hN405^400`3Qh@1Uh@030>7Q01;Q00<0hN40
:N400`3Qh@05h@005N400`3Qh@08h@030>7Q03?Q00<0hN405^400`3Qh@1Uh@030>7Q01;Q00<0hN40
:N400`3Qh@05h@005N400`3Qh@09h@030>7Q03;Q00<0hN405N400`3Qh@1Vh@030>7Q01?Q00<0hN40
:>400`3Qh@05h@005N4200_Q00<0hN40<N400`3Qh@0Dh@030>7Q06OQ00<0hN405>400`3Qh@0Wh@03
0>7Q00GQ000Eh@030>7Q00_Q00<0hN40<>400`3Qh@0Dh@030>7Q06OQ00<0hN405>400`3Qh@0Wh@03
0>7Q00GQ000Eh@030>7Q00_Q00<0hN40<>400`3Qh@0Ch@030>7Q06SQ00<0hN405N400`3Qh@0Vh@03
0>7Q00GQ0004h@<00n400`3Qh@02h@030>7Q00?Q00<0hN403>400`3Qh@0_h@030>7Q01?Q00<0hN40
J>400`3Qh@0Fh@030>7Q02GQ00<0hN401N4000CQ00<0hN401n400`3Q0004h@030>7Q00gQ00<0hN40
;^400`3Qh@0Bh@030>7Q06WQ00<0hN405n400`3Qh@0Th@030>7Q00GQ0005h@030>7Q00SQ00<0hN40
0^4300kQ00<0hN40;N400`3Qh@0Ah@030>7Q06[Q00<0hN405n400`3Qh@0Th@030>7Q00GQ0005h@03
0>7Q00KQ0P05h@030>7Q00kQ00<0hN40;N400`3Qh@0@h@030>7Q06_Q00<0hN406>400`3Qh@0Sh@03
0>7Q00GQ0004h@030>4000OQ00<0hN401>400`3Qh@0?h@030>7Q02cQ00<0hN404>400`3Qh@1[h@03
0>7Q01WQ00<0hN408^400`3Qh@05h@001N400`3Qh@06h@<01>400`3Qh@0@h@030>7Q02_Q00<0hN40
3n400`3Qh@1/h@030>7Q01WQ00<0hN408^400`3Qh@05h@005N400`3Qh@0Ah@030>7Q02[Q00<0hN40
3^400`3Qh@1]h@030>7Q01[Q00<0hN408N400`3Qh@05h@005N400`3Qh@0Ah@030>7Q02[Q00<0hN40
3N400`3Qh@1^h@030>7Q01_Q00<0hN408>400`3Qh@05h@005N4201?Q00<0hN40:N400`3Qh@0<h@03
0>7Q06oQ00<0hN407>400`3Qh@0Oh@030>7Q00GQ000Eh@030>7Q01?Q00<0hN40:>400`3Qh@0<h@03
0>7Q06oQ00<0hN407>400`3Qh@0Oh@030>7Q00GQ000Eh@030>7Q01CQ00<0hN409n400`3Qh@0;h@03
0>7Q073Q00<0hN407N400`3Qh@0Nh@030>7Q00GQ000Eh@030>7Q01GQ00<0hN409^400`3Qh@0:h@03
0>7Q077Q00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01KQ00<0hN409N400`3Qh@09h@03
0>7Q07;Q00<0hN407n400`3Qh@0Lh@030>7Q00GQ000Eh@805n400`3Qh@0Uh@030>7Q00WQ00<0hN40
L^400`3Qh@0Ph@030>7Q01_Q00<0hN401N4001GQ00<0hN405n400`3Qh@0Th@030>7Q00SQ00<0hN40
Ln400`3Qh@0Ph@030>7Q01_Q00<0hN401N4001GQ00<0hN406>400`3Qh@0Sh@030>7Q00OQ00<0hN40
M>400`3Qh@0Qh@030>7Q01[Q00<0hN401N4001GQ00<0hN406N400`3Qh@0Rh@030>7Q00KQ00<0hN40
MN400`3Qh@0Rh@030>7Q01WQ00<0hN401N4001GQ00<0hN406^400`3Qh@0Qh@030>7Q00GQ00<0hN40
M^400`3Qh@0Sh@030>7Q01SQ00<0hN401N4001GQ0P0Lh@030>7Q023Q00<0hN401>400`3Qh@1gh@03
0>7Q02CQ00<0hN405n400`3Qh@05h@005N400`3Qh@0Lh@030>7Q01oQ00<0hN400n400`3Qh@1hh@03
0>7Q02GQ00<0hN405^400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN400^400`3Qh@1ih@03
0>7Q02KQ00<0hN405N400`3Qh@05h@005N400`3Qh@0Nh@030>7Q01gQ00<0hN400P1lh@030>7Q02OQ
00<0hN405>400`3Qh@05h@005N400`3Qh@0Oh@030>7Q01cQ00<0h@00O^400`3Qh@0Xh@030>7Q01?Q
00<0hN401N4001GQ00<0hN408>4201cQ0P1oh@030>7Q02WQ0P0Ch@030>7Q00GQ000EhD80P>400`3Q
h@0[h@030>7Q013Q00<0hN401N4001GQ00<0hN408n4201OQ00<0hN40P>400`3Qh@0/h@804>400`3Q
h@05h@005N400`3Qh@0Uh@804n4208?Q00<0hN40;^400`3Qh@0=h@030>7Q00GQ000>h@801N400`3Q
h@0Wh@803n4208GQ00<0hN40;n4300cQ00<0hN401N40013Q00<0hN400^400`3Qh@0Yh@<02^4208OQ
@P07h@003n400`3Qh@03h@<0;>4:0<7Q2@08h@004>400`3Qh@02h@030>7Q0?oQ2N4000kQ00<0h@00
1>400`3Qh@3oh@WQ000?h@030>7Q00?Q00<0hN40on49h@005N400`3Qh@3oh@WQ0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.66388, -0.23241, 
  0.0308942, 0.0187455}},
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.0147151 0.196201 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11282 -18 -4.5 ]
[.01131 .11282 0 4.5 ]
[.01131 .21092 -6 -4.5 ]
[.01131 .21092 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .50522 -18 -4.5 ]
[.01131 .50522 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.2619 .01472 m
.2619 .02097 L
s
[(2)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(4)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(6)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(8)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11282 m
.03006 .11282 L
s
[(0.5)] .01131 .11282 1 0 Mshowa
.02381 .21092 m
.03006 .21092 L
s
[(1)] .01131 .21092 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1.5)] .01131 .30902 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(2)] .01131 .40712 1 0 Mshowa
.02381 .50522 m
.03006 .50522 L
s
[(2.5)] .01131 .50522 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .0932 m
.02756 .0932 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .32864 m
.02756 .32864 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .56408 m
.02756 .56408 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02381 .57222 L
.14286 .57222 L
.14286 .01472 L
.14286 .58552 L
.2619 .58552 L
.2619 .01472 L
.2619 .43481 L
.38095 .43481 L
.38095 .01472 L
.38095 .25863 L
.5 .25863 L
.5 .01472 L
.5 .21898 L
.61905 .21898 L
.61905 .01472 L
.61905 .3523 L
.7381 .3523 L
.7381 .01472 L
.7381 .53602 L
.85714 .53602 L
.85714 .01472 L
.85714 .60123 L
.97619 .60123 L
.97619 .01472 L
s
.02381 .40712 m
.06244 .46968 L
.10458 .53025 L
.12507 .55459 L
.14415 .57336 L
.16254 .58741 L
.17284 .59342 L
.18221 .59768 L
.18704 .59942 L
.19216 .60092 L
.19654 .60191 L
.19901 .60236 L
.20135 .6027 L
.20265 .60286 L
.20405 .603 L
.20535 .60311 L
.20652 .60319 L
.20719 .60323 L
.20792 .60326 L
.20869 .60329 L
.20941 .60331 L
.21009 .60332 L
.2108 .60332 L
.21209 .60331 L
.21328 .60328 L
.21456 .60322 L
.21577 .60315 L
.21688 .60306 L
.21958 .60279 L
.22214 .60243 L
.22673 .60157 L
.23179 .60028 L
.24205 .5966 L
.25155 .59194 L
.26047 .5865 L
.28065 .57051 L
.29915 .55173 L
.33878 .50047 L
.37689 .44142 L
.41745 .3749 L
.45649 .31427 L
.47641 .28679 L
.49799 .26082 L
.51703 .24187 L
.52785 .23295 L
.53797 .2259 L
.54836 .22004 L
.55785 .21593 L
.56305 .21418 L
.56799 .21287 L
Mistroke
.57321 .21185 L
.57616 .21143 L
.57749 .21129 L
.57889 .21116 L
.58016 .21107 L
.58083 .21103 L
.58155 .21099 L
.58281 .21094 L
.58398 .21092 L
.58469 .21092 L
.58546 .21092 L
.58619 .21093 L
.58686 .21095 L
.58814 .21099 L
.58878 .21103 L
.58949 .21107 L
.59196 .21127 L
.59336 .21142 L
.59463 .21158 L
.59945 .2124 L
.60379 .21341 L
.60853 .2148 L
.61807 .21853 L
.62838 .22391 L
.63772 .22998 L
.65869 .2475 L
.69872 .29409 L
.73724 .35094 L
.77821 .41767 L
.81767 .48084 L
.85561 .5341 L
.87673 .5585 L
.896 .57664 L
.90555 .58401 L
.91595 .59074 L
.92571 .59578 L
.93487 .59937 L
.93939 .60071 L
.94426 .60186 L
.94689 .60234 L
.94966 .60274 L
.95203 .603 L
.95337 .60311 L
.95462 .6032 L
.95593 .60326 L
.95667 .60329 L
.95736 .6033 L
.95861 .60332 L
.95924 .60332 L
.95994 .60331 L
Mistroke
.96114 .60328 L
.96244 .60323 L
.96366 .60316 L
.96477 .60307 L
.96743 .6028 L
.97034 .6024 L
.97562 .60136 L
.97619 .60123 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3ooooooooQ8N4005CQ0`0oh@<0?N4203oQ00<0hN40
1^4005CQ00<0hN40@>400`3Qh@0lh@030>4003gQ00<0h@001n4005GQ00<0hN40?N4403gQ00<0h@00
?N400`3Q0007h@00EN400`3Qh@0mh@030>4003kQ0P0oh@030>7Q00KQ001Dh@030>4003oQ0P0nh@03
0>7Q03gQ00<0h@001n4005GQ00<0hN40?n400`3Qh@0mh@80?^400`3Qh@06h@00on4Qh@005N400`3Q
h@3oh@WQ000Eh@030>7Q0?oQ2N4001GQ00<0hN40on49h@005N400`3Qh@3oh@WQ000?hOl04@01h@00
5N400`3Qh@0=h@030>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@03
0>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@030>7Q00kQ00<0hN40
3N400`3Qh@0=h@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@807^400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@80
7^400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@05h@001N400`3Qh@02h@030>7Q00;Q00<0hN400n400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4000CQ00<0h@00
1n400`3Q0004h@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@001>400`3Q0009h@030>7Q00;Q0`0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4000CQ00<0h@00
1n4200GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ0004h@030>4000OQ00<0hN401>400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4000GQ00<0hN40
1^4300CQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
1N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@03
0>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
1N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@003N4500?Q00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000?h@030>7Q00?Q00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000?h@030>7Q00?Q0`0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q01;Q1P06h@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000?h@030>7Q00?Q00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0?h@<01^450003h@3Q01kQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ
000?h@030>7Q00?Q00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N4R01oQ00<0hN407^400`3Q
h@0Mh@030>7Q00GQ000>h@801N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q00[Q
0`0Ah@807^400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ
00<0hN407N400`3Qh@08h@805>40103Q000Lh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q00KQ0P0Fh@040>7Qh@806^400`3Qh@0Nh@03
0>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q00GQ00<0hN40
5^400`3Qh@03h@030>7Q01OQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01gQ00<0hN401>400`3Qh@0Gh@030>7Q00CQ00<0hN405^400`3Qh@0Nh@03
0>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@03h@03
0>7Q01SQ00<0hN401N4201KQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01gQ00<0hN400^400`3Qh@0Ih@030>7Q00OQ00<0hN404n400`3Qh@0Nh@03
0>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N401@3QhN4001cQ
00<0hN402>400`3Qh@0Bh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N4201kQ00<0hN407^400`3Q
h@0Mh@030>7Q01gQ00@0hN407N400`3Qh@09h@030>7Q017Q00<0hN407^400`3Qh@0Mh@030>7Q00GQ
000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0h@007^400`3Qh@0:h@030>7Q013Q
00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0MhB807n400`3Q
h@0:h@030>7Q013Q00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Q
h@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0;h@030>7Q00oQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ
000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01cQ00<0hN407n400`3Qh@0<h@030>7Q00kQ
00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@807^400`3Qh@0Nh@030>7Q01gQ00<0hN406n400`3Q
h@0Ph@030>7Q00gQ00<0hN403N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Q
h@0Nh@030>7Q01gQ00<0hN406^400`3Qh@0Qh@030>7Q00gQ00<0hN403N400`3Qh@0Nh@030>7Q01gQ
00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN406^400`3Qh@0Qh@030>7Q00kQ
00<0hN403>400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ
00<0hN406N400`3Qh@0Rh@030>7Q00oQ00<0hN402n400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ
00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN406>400`3Qh@0Sh@030>7Q013Q00<0hN402^400`3Q
h@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN405n400`3Q
h@0Th@030>7Q013Q00<0hN402^400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ
00<0hN407N400`3Qh@0Fh@030>7Q02GQ00<0hN404N400`3Qh@09h@030>7Q01kQ00<0hN407N400`3Q
h@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Fh@030>7Q02GQ00<0hN404^400`3Q
h@08h@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Q
h@0Eh@030>7Q02KQ00<0hN404n400`3Qh@07h@030>7Q01kQ00<0hN407N400`3Qh@05h@000n4500;Q
00<0hN400^400`3Qh@03h@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01CQ00<0hN409n400`3Q
h@0Ch@030>7Q00OQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ0005h@030>7Q00KQ00<0h@001>400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Ch@030>7Q02SQ00<0hN405>400`3Qh@06h@030>7Q01kQ
00<0hN407N400`3Qh@05h@001N400`3Qh@08h@030>7Q00;Q0`0Mh@030>7Q01kQ00<0hN407N400`3Q
h@0Ch@030>7Q02SQ00<0hN405N400`3Qh@05h@030>7Q01kQ00<0hN407N400`3Qh@05h@001N400`3Q
h@06h@801N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Bh@030>7Q02WQ00<0hN405^400`3Q
h@04h@030>7Q01kQ00<0hN407N400`3Qh@05h@001N400`3Qh@06h@030>7Q00CQ00<0hN407N400`3Q
h@0Nh@030>7Q01gQ00<0hN404N400`3Qh@0Zh@030>7Q01KQ00<0hN401>400`3Qh@0Nh@030>7Q01gQ
00<0hN401N4000CQ0P08h@<01>400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Ah@030>7Q02[Q
00<0hN405n400`3Qh@03h@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ
00<0hN407N400`3Qh@0@h@030>7Q02_Q00<0hN406>400`3Qh@02h@030>7Q01kQ00<0hN407N400`3Q
h@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@030>7Q00oQ00<0hN40;>400`3Qh@0Ih@050>7Qh@00
8>400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00oQ00<0hN40
;>400`3Qh@0Ih@050>7Qh@008>400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Q
h@0Mh@030>7Q00kQ00<0hN40;N400`3Qh@0Jh@040>7Q023Q00<0hN407N400`3Qh@05h@005N400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0=h@030>7Q02kQ00<0hN406n400`3Q000Ph@030>7Q01gQ
00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN403N400`3Qh@0^h@030>7Q01cQ
0P0Ph@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN403>400`3Q
h@0_h@030>7Q01cQ0P0Ph@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN407N400`3Q
h@0<h@030>7Q02oQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Q
h@0Nh@030>7Q01gQ00<0hN402n400`3Qh@0`hB807n400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ
00<0hN407^400`3Qh@0Mh@030>7Q00[Q00<0hN40DN4201oQ00<0hN407N400`3Qh@05h@005N400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0:h@030>7Q057Q00<0h@007^400`3Qh@0Mh@030>7Q00GQ
000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00WQ00<0hN40D^400`3Q000Nh@030>7Q01gQ
00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN407N400`3Qh@08h@030>7Q05?Q00@0hN407N400`3Q
h@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00SQ00<0hN40Dn40103Q
h@0Mh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401n400`3Q
h@1Dh@050>7Qh@007>400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00KQ00<0hN40EN400`3Qh@02h@030>7Q01WQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@06h@030>7Q05GQ00<0hN400^400`3Qh@0Ih@030>7Q01gQ00<0hN40
1N4001GQ0P0Nh@030>7Q01kQ00<0hN407N400`3Qh@05h@030>7Q05KQ00<0hN400n400`3Qh@0Hh@03
0>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N400`3Qh@1Fh@03
0>7Q00?Q00<0hN406>400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00CQ00<0hN40En400`3Qh@04h@030>7Q01OQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@03h@030>7Q05SQ00<0hN401N400`3Qh@0Fh@030>7Q01gQ00<0hN40
1N4000kQ0`04h@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00?Q00<0hN40F>400`3Qh@05h@03
0>7Q01KQ00<0hN407N400`3Qh@05h@003^400`3Qh@04h@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00;Q00<0hN40FN400`3Qh@06h@030>7Q01GQ00<0hN407N400`3Qh@05h@003n400`3Qh@03h@<0
7N400`3Qh@0Nh@030>7Q01gQ00D0hN7Q001Lh@030>7Q00KQ00<0hN405N400`3Qh@0Mh@030>7Q00GQ
000?h@030>7Q00?Q0P0Nh@030>7Q01kQ00<0hN407N401@3QhN4005cQ00<0hN401n400`3Qh@0Dh@03
0>7Q01gQ00<0hN401N4000kQ00<0h@001>4201kQ00<0hN407^400`3Qh@0Mh@040>7Q05gQ00<0hN40
1n400`3Qh@0Dh@030>7Q01gQ00<0hN401N4000oQ00<0hN400n400`3Q000Mh@030>7Q01kQ00<0hN40
7N40103Qh@1Mh@030>7Q00SQ00<0hN404n400`3Qh@0Mh@030>7Q00GQ000Eh@030>4001gQ00<0hN40
7^400`3Qh@0Mh@030>4005kQ00<0hN402N400`3Qh@0Bh@030>7Q01gQ00<0hN401N4001GQ0P000n40
h@0Kh@030>7Q01kQ00<0hN407N4205oQ00<0hN402N400`3Qh@0Bh@030>7Q01gQ00<0hN401N4001GQ
00D0hN7Q000Kh@030>7Q01kQ00<0hN407N4205oQ00<0hN402^400`3Qh@0Ah@030>7Q01gQ00<0hN40
1N4001GQ00D0hN7Q000Kh@030>7Q01kQ8@1Ph@030>7Q00_Q00<0hN404>400`3Qh@0Mh@030>7Q00GQ
000Eh@030>7Q00;Q00<0hN406>400`3Qh@0Nh@030>7Q01gQ00<0hN40G^400`3Qh@0;h@030>7Q013Q
00<0hN407N400`3Qh@05h@005N400`3Qh@02h@030>7Q01SQ00<0hN407^400`3Qh@0Lh@030>7Q05oQ
00<0hN403>400`3Qh@0?h@030>7Q01gQ00<0hN401N4001GQ0P04h@030>7Q01OQ00<0hN407^400`3Q
h@0Kh@030>7Q063Q00<0hN403N400`3Qh@0>h@030>7Q01gQ00<0hN401N4001GQ00<0hN400n400`3Q
h@0Gh@030>7Q01kQ00<0hN406n400`3Qh@1Ph@030>7Q00gQ00<0hN403^400`3Qh@0Mh@030>7Q00GQ
000Eh@030>7Q00CQ00<0hN405^400`3Qh@0Nh@030>7Q01[Q00<0hN40HN400`3Qh@0>h@030>7Q00gQ
00<0hN407N400`3Qh@05h@005N400`3Qh@05h@030>7Q01GQ00<0hN407^400`3Qh@0Ih@030>7Q06;Q
00<0hN403^400`3Qh@0=h@030>7Q01gQ00<0hN401N4001GQ00<0hN401N400`3Qh@0Eh@030>7Q01kQ
00<0hN406N400`3Qh@1Rh@030>7Q00oQ00<0hN403>400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00KQ
00<0hN405>400`3Qh@0Nh@030>7Q01SQ00<0hN40Hn400`3Qh@0@h@030>7Q00_Q00<0hN407N400`3Q
h@05h@005N4200OQ00<0hN405>400`3Qh@0Nh@030>7Q01SQ00<0hN40Hn400`3Qh@0@h@030>7Q00_Q
00<0hN407N400`3Qh@05h@005N400`3Qh@07h@030>7Q01?Q00<0hN407^400`3Qh@0Gh@030>7Q06CQ
00<0hN404N400`3Qh@0:h@030>7Q01gQ00<0hN401N4001GQ00<0hN402>400`3Qh@0Bh@030>7Q01kQ
00<0hN405^400`3Qh@1Uh@030>7Q01;Q00<0hN402N400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00SQ
00<0hN404^400`3Qh@0Nh@030>7Q01KQ00<0hN40IN400`3Qh@0Bh@030>7Q00WQ00<0hN407N400`3Q
h@05h@005N400`3Qh@09h@030>7Q017Q00<0hN407^400`3Qh@0Eh@030>7Q06KQ00<0hN404n400`3Q
h@08h@030>7Q01gQ00<0hN401N4001GQ0P0;h@030>7Q013Q00<0hN407^400`3Qh@0Dh@030>7Q06OQ
00<0hN405>400`3Qh@07h@030>7Q01gQ00<0hN401N4001GQ00<0hN402n400`3Qh@0?h@030>7Q01kQ
00<0hN405>400`3Qh@1Wh@030>7Q01CQ00<0hN401n400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00_Q
00<0hN403n400`3Qh@0Nh@030>7Q01?Q00<0hN40J>400`3Qh@0Eh@030>7Q00KQ00<0hN407N400`3Q
h@05h@001>4300?Q00<0hN400^400`3Qh@03h@030>7Q00cQ00<0hN403^400`3Qh@0Nh@030>7Q01?Q
00<0hN40J>400`3Qh@0Fh@030>7Q00GQ00<0hN407N400`3Qh@05h@001>400`3Qh@07h@030>4000CQ
00<0hN403N400`3Qh@0=h@030>7Q01kQ00<0hN404^400`3Qh@1Yh@030>7Q01OQ00<0hN401>400`3Q
h@0Mh@030>7Q00GQ0005h@030>7Q00SQ00<0hN400^4300kQ00<0hN403>400`3Qh@0Nh@030>7Q017Q
00<0hN40J^400`3Qh@0Gh@030>7Q00CQ00<0hN407N400`3Qh@05h@001N400`3Qh@06h@801N400`3Q
h@0>h@030>7Q00cQ00<0hN407^400`3Qh@0@h@030>7Q06_Q00<0hN406>400`3Qh@03h@030>7Q01gQ
00<0hN401N4000CQ00<0h@001n400`3Qh@04h@030>7Q00oQ00<0hN402n400`3Qh@0Nh@030>7Q013Q
00<0hN40Jn400`3Qh@0Ih@030>7Q00;Q00<0hN407N400`3Qh@05h@001N400`3Qh@06h@<01>400`3Q
h@0@h@030>7Q00[Q00<0hN407^400`3Qh@0?h@030>7Q06cQ00<0hN406N400`3Qh@02h@030>7Q01gQ
00<0hN401N4001GQ00<0hN404N400`3Qh@09h@030>7Q01kQ00<0hN403^400`3Qh@1]h@030>7Q01[Q
00D0hN7Q000Oh@030>7Q00GQ000Eh@030>7Q017Q00<0hN402N400`3Qh@0Nh@030>7Q00gQ00<0hN40
K^400`3Qh@0Kh@040>7Q01oQ00<0hN401N4001GQ0P0Ch@030>7Q00SQ00<0hN407^400`3Qh@0<h@03
0>7Q06oQ00<0hN407>400`3Q000Oh@030>7Q00GQ000Eh@030>7Q01?Q00<0hN401n400`3Qh@0Nh@03
0>7Q00cQ00<0hN40Kn400`3Qh@0Lh@030>4001oQ00<0hN401N4001GQ00<0hN405>400`3Qh@06h@03
0>7Q01kQ00<0hN402n400`3Qh@1`h@030>7Q01gQ0P0Oh@030>7Q00GQ000Eh@030>7Q01GQ00<0hN40
1N400`3Qh@0Nh@030>7Q00[Q00<0hN40LN4R01oQ00<0hN401N4001GQ00<0hN405^400`3Qh@04h@03
0>7Q01kQ00<0hN402N400`3Qh@2Ch@807^400`3Qh@05h@005N4201OQ00<0hN401>400`3Qh@0Nh@03
0>7Q00WQ00<0hN40Tn400`3Q000Mh@030>7Q00GQ000Eh@030>7Q01OQ00<0hN400n400`3Qh@0Nh@03
0>7Q00SQ00<0hN40U>400`3Q000Mh@030>7Q00GQ000Eh@030>7Q01SQ00<0hN400^400`3Qh@0Nh@03
0>7Q00OQ00<0hN40UN40103Qh@0Lh@030>7Q00GQ000Eh@030>7Q01WQ00D0hN7Q000Ph@030>7Q00KQ
00<0hN40U^401@3QhN4001_Q00<0hN401N4001GQ00<0hN406^40103Qh@0Ph@030>7Q00GQ00<0hN40
Un400`3Qh@02h@030>7Q01SQ00<0hN401N4001GQ0P0Lh@030>40023Q00<0hN401>400`3Qh@2Hh@03
0>7Q00?Q00<0hN405n400`3Qh@05h@005N400`3Qh@0Lh@808>400`3Qh@03h@030>7Q09WQ00<0hN40
1>400`3Qh@0Fh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@02h@030>7Q09[Q00<0hN40
1N400`3Qh@0Eh@030>7Q00GQ000EhB807n400`3Qh@0209gQ00<0hN401^400`3Qh@0Dh@030>7Q00GQ
000Eh@030>7Q01gQ00<0h@007^400`3Q002Oh@030>7Q00OQ00<0hN404n400`3Qh@05h@005N400`3Q
h@0Mh@030>7Q00807>420:3Q00<0hN402>4201?Q00<0hN401N4001GQ0P0NhB80XN400`3Qh@0:h@03
0>7Q013Q00<0hN401N4001GQ00<0hN408n4201OQ00<0hN40XN400`3Qh@0;h@804>400`3Qh@05h@00
5N400`3Qh@0Uh@804n420:CQ00<0hN403N400`3Qh@0=h@030>7Q00GQ000>h@801N400`3Qh@0Wh@80
3n420:KQ00<0hN403^4300cQ00<0hN401N40013Q00<0hN400^400`3Qh@0Yh@<02^420:SQ8@07h@00
3n400`3Qh@03h@<0;>4:0<7Q2@08h@004>400`3Qh@02h@030>7Q0?oQ2N4000kQ00<0h@001>400`3Q
h@3oh@WQ000?h@030>7Q00?Q00<0hN40on49h@005N400`3Qh@3oh@WQ0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.66388, -0.23241, 
  0.0308942, 0.0187455}},
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"Integ", "AnimateRectangular"}]
}, Open  ]],

Cell["AnimateTrapezoidal", "Subsection",
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[TextData[
"\[FilledSmallSquare] AnimateTrapezoidal produces an animation with n frames \
illustrating the use of the trapezoidal method to approximte the integral of \
g[x] between x=a and x=b."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[TextData[
"\[FilledSmallSquare] The parameter g must be a single-parameter, real-valued \
function.\n\[FilledSmallSquare] The parameters a and b must be real-valued, \
with a < b.\n\[FilledSmallSquare] The parameter n must be a positive \
integer."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell["Examples", "Subsubsection",
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[BoxData[
    \(Needs["\<ISP`Integ`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[BoxData[
    \(f[x_]\  := \ Sin[x] + 2\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnimateTrapezoidal[f, \ 0, \ 8, \ 3]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.0147151 0.196201 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11282 -18 -4.5 ]
[.01131 .11282 0 4.5 ]
[.01131 .21092 -6 -4.5 ]
[.01131 .21092 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .50522 -18 -4.5 ]
[.01131 .50522 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.2619 .01472 m
.2619 .02097 L
s
[(2)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(4)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(6)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(8)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11282 m
.03006 .11282 L
s
[(0.5)] .01131 .11282 1 0 Mshowa
.02381 .21092 m
.03006 .21092 L
s
[(1)] .01131 .21092 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1.5)] .01131 .30902 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(2)] .01131 .40712 1 0 Mshowa
.02381 .50522 m
.03006 .50522 L
s
[(2.5)] .01131 .50522 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .0932 m
.02756 .0932 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .32864 m
.02756 .32864 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .56408 m
.02756 .56408 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02381 .40712 L
.5 .25863 L
.5 .01472 L
.5 .25863 L
.97619 .60123 L
.97619 .01472 L
s
.02381 .40712 m
.06244 .46968 L
.10458 .53025 L
.12507 .55459 L
.14415 .57336 L
.16254 .58741 L
.17284 .59342 L
.18221 .59768 L
.18704 .59942 L
.19216 .60092 L
.19654 .60191 L
.19901 .60236 L
.20135 .6027 L
.20265 .60286 L
.20405 .603 L
.20535 .60311 L
.20652 .60319 L
.20719 .60323 L
.20792 .60326 L
.20869 .60329 L
.20941 .60331 L
.21009 .60332 L
.2108 .60332 L
.21209 .60331 L
.21328 .60328 L
.21456 .60322 L
.21577 .60315 L
.21688 .60306 L
.21958 .60279 L
.22214 .60243 L
.22673 .60157 L
.23179 .60028 L
.24205 .5966 L
.25155 .59194 L
.26047 .5865 L
.28065 .57051 L
.29915 .55173 L
.33878 .50047 L
.37689 .44142 L
.41745 .3749 L
.45649 .31427 L
.47641 .28679 L
.49799 .26082 L
.51703 .24187 L
.52785 .23295 L
.53797 .2259 L
.54836 .22004 L
.55785 .21593 L
.56305 .21418 L
.56799 .21287 L
Mistroke
.57321 .21185 L
.57616 .21143 L
.57749 .21129 L
.57889 .21116 L
.58016 .21107 L
.58083 .21103 L
.58155 .21099 L
.58281 .21094 L
.58398 .21092 L
.58469 .21092 L
.58546 .21092 L
.58619 .21093 L
.58686 .21095 L
.58814 .21099 L
.58878 .21103 L
.58949 .21107 L
.59196 .21127 L
.59336 .21142 L
.59463 .21158 L
.59945 .2124 L
.60379 .21341 L
.60853 .2148 L
.61807 .21853 L
.62838 .22391 L
.63772 .22998 L
.65869 .2475 L
.69872 .29409 L
.73724 .35094 L
.77821 .41767 L
.81767 .48084 L
.85561 .5341 L
.87673 .5585 L
.896 .57664 L
.90555 .58401 L
.91595 .59074 L
.92571 .59578 L
.93487 .59937 L
.93939 .60071 L
.94426 .60186 L
.94689 .60234 L
.94966 .60274 L
.95203 .603 L
.95337 .60311 L
.95462 .6032 L
.95593 .60326 L
.95667 .60329 L
.95736 .6033 L
.95861 .60332 L
.95924 .60332 L
.95994 .60331 L
Mistroke
.96114 .60328 L
.96244 .60323 L
.96366 .60316 L
.96477 .60307 L
.96743 .6028 L
.97034 .6024 L
.97562 .60136 L
.97619 .60123 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3ooooooooQ8N4005CQ0`0oh@<0?N4203oQ00<0hN40
1^4005CQ00<0hN40@>400`3Qh@0lh@030>4003gQ00<0h@001n4005GQ00<0hN40?N4403gQ00<0h@00
?N400`3Q0007h@00EN400`3Qh@0mh@030>4003kQ0P0oh@030>7Q00KQ001Dh@030>4003oQ0P0nh@03
0>7Q03gQ00<0h@001n4005GQ00<0hN40?n400`3Qh@0mh@80?^400`3Qh@06h@00on4Qh@005N400`3Q
h@3oh@WQ000Eh@030>7Q0?oQ2N4001GQ00<0hN40on49h@005N400`3Qh@3oh@WQ000?hOl04@01h@00
5N400`3Qh@0=h@030>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@03
0>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@030>7Q00kQ00<0hN40
3N400`3Qh@0=h@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N4207oQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40
O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@03
0>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@80On400`3Qh@1oh@03
0>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN40
1N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N4207oQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40
O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@03
0>7Q07oQ00<0hN401N4001GQ0P1oh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@03
0>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN40
1N4000GQ00<0hN400^400`3Qh@02h@030>7Q00?Q00<0hN40O^400`3Qh@1oh@030>7Q00GQ0004h@03
0>4000OQ00<0h@001>400`3Qh@1nh@030>7Q07oQ00<0hN401N4000CQ00<0h@002N400`3Qh@02h@<0
O^400`3Qh@1oh@030>7Q00GQ0004h@030>4000OQ0P05h@030>7Q07kQ00<0hN40On400`3Qh@05h@00
1>400`3Q0007h@030>7Q00CQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ0005h@030>7Q00KQ0`04h@03
0>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ0P1oh@03
0>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40
On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@03
0>7Q00GQ000Eh@80On400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@03
0>7Q07kQ00<0hN40On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ0P1oh@03
0>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40
On400`3Qh@05h@005N400`3Qh@1nh@030>7Q07oQ00<0hN401N4001GQ00<0hN40O^400`3Qh@1oh@03
0>7Q00GQ000Eh@80On400`3Qh@1oh@030>7Q00GQ000Eh@030>7Q07kQ00<0hN40On400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q07oQ00<0hN401N4000gQ1@03h@030>7Q07kQ00<0hN40On400`3Qh@05h@00
3n400`3Qh@03h@030>7Q07kQ00<0hN40On400`3Qh@05h@003n400`3Qh@03h@<0O^400`3Qh@0Bh@H0
In400`3Qh@05h@003n400`3Qh@03h@030>7Q07kQ00<0hN403n4300KQ1@1Rh@030>7Q00GQ000?h@03
0>7Q00?Q00<0hN40O^400`3Qh@0=h@803^42063Q00<0hN401N4000kQ0P05h@030>7Q07kQ00<0hN40
2^4301;Q00<0hN40GN400`3Qh@05h@005N400`3Qh@1nh@030>7Q00SQ0P0Fh@80GN400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q00KQ0P0Jh@80Fn400`3Qh@05h@005N4207oQ00<0hN401N400`3Qh@0Lh@03
0>7Q05SQ00<0hN401N4001GQ00<0hN40O^400`3Qh@04h@030>7Q01kQ00<0hN40En400`3Qh@05h@00
5N400`3Qh@1nh@030>7Q00?Q00<0hN408>4205OQ00<0hN401N4001GQ00<0hN40O^400`3Qh@02h@03
0>7Q02?Q00<0hN40E>400`3Qh@05h@005N400`3Qh@1nh@050>7Qh@009n400`3Qh@1Ch@030>7Q00GQ
000Eh@80On40103Qh@0Yh@030>7Q05;Q00<0hN401N4001GQ00<0hN40O^400`3Q000[h@030>7Q057Q
00<0hN401N4001GQ00<0hN40ON4302cQ00<0hN40DN400`3Qh@05h@005N400`3Qh@1jh@<000CQ0000
;>400`3Qh@1@h@030>7Q00GQ000Eh@030>7Q07OQ0`03h@050>7Qh@00;>400`3Qh@1?h@030>7Q00GQ
000Eh@80M>4400GQ00<0hN400n400`3Qh@0Zh@030>7Q04kQ00<0hN401N4001GQ00<0hN40L>4300SQ
00<0hN401N4202[Q00<0hN40C^400`3Qh@05h@005N400`3Qh@1]h@<02n400`3Qh@07h@030>7Q02SQ
00<0hN40CN400`3Qh@05h@005N400`3Qh@1Zh@<03N400`3Qh@09h@80:N400`3Qh@1<h@030>7Q00GQ
000Eh@030>7Q06OQ0`0?h@030>7Q00cQ00<0hN409n400`3Qh@1;h@030>7Q00GQ000Eh@030>7Q06CQ
0`0Ah@030>7Q00kQ00<0hN409^400`3Qh@1;h@030>7Q00GQ000Eh@80H^4301?Q00<0hN404>4202OQ
00<0hN40B^400`3Qh@05h@005N400`3Qh@1Mh@@05^400`3Qh@0Bh@030>7Q02GQ00<0hN40BN400`3Q
h@05h@005N400`3Qh@1Jh@<06N400`3Qh@0Dh@809^400`3Qh@18h@030>7Q00GQ0003h@D00^400`3Q
h@02h@030>7Q00?Q00<0hN40En4301_Q00<0hN405n400`3Qh@0Sh@030>7Q04SQ00<0hN401N4000GQ
00<0hN401^400`3Q0004h@030>7Q05CQ0`0Mh@030>7Q01WQ00<0hN408n400`3Qh@17h@030>7Q00GQ
0005h@030>7Q00SQ00<0hN400^43057Q0`0Ph@030>7Q01[Q0P0Th@030>7Q04KQ00<0hN401N4000GQ
00<0hN401^4200GQ00<0hN40C^4302;Q00<0hN407N400`3Qh@0Rh@030>7Q04GQ00<0hN401N4000GQ
00<0hN401^400`3Qh@04h@030>7Q04_Q0`0Th@030>7Q01oQ0P0Rh@030>7Q04GQ00<0hN401N4000CQ
0P08h@<01>400`3Qh@17h@@09n400`3Qh@0Qh@030>7Q023Q00<0hN40A>400`3Qh@05h@005N400`3Q
h@14h@<0:^400`3Qh@0Sh@030>7Q023Q00<0hN40@n400`3Qh@05h@005N4204;Q0`0/h@030>7Q02GQ
0P0Qh@030>7Q04;Q00<0hN401N4001GQ00<0hN40?^4302oQ00<0hN409n400`3Qh@0Nh@030>7Q04;Q
00<0hN401N4001GQ00<0hN40>n43037Q00<0hN40:N4201oQ00<0hN40@N400`3Qh@05h@005N400`3Q
h@0hh@<0<n400`3Qh@0/h@030>7Q01gQ00<0hN40@>400`3Qh@05h@005N400`3Qh@0eh@<0=^400`3Q
h@0]h@030>7Q01gQ00<0hN40?n400`3Qh@05h@005N400`3Qh@0ah@@0>>400`3Qh@0_h@807N400`3Q
h@0oh@030>7Q00GQ000Eh@80;n4303cQ00<0hN40<N400`3Qh@0Kh@030>7Q03kQ00<0hN401N4001GQ
00<0hN40:n4303kQ00<0hN40<n4201cQ00<0hN40?N400`3Qh@05h@005N400`3Qh@0Xh@<0@>400`3Q
h@0fh@030>7Q01WQ00<0hN40?N400`3Qh@05h@005N400`3Qh@0Uh@<0@n400`3Qh@0gh@030>7Q01WQ
00<0hN40?>400`3Qh@05h@005N400`3Qh@0Rh@<0AN400`3Qh@0ih@806N400`3Qh@0lh@030>7Q00GQ
000Eh@808>4304OQ00<0hN40?>400`3Qh@0Gh@030>7Q03_Q00<0hN401N4001GQ00<0hN406n4404[Q
00<0hN40?N4201OQ00<0hN40>n400`3Qh@05h@005N400`3Qh@0Hh@<0CN400`3Qh@10h@030>7Q01GQ
00<0hN40>^400`3Qh@05h@005N400`3Qh@0Eh@<0Cn400`3Qh@12h@030>7Q01GQ00<0hN40>N400`3Q
h@05h@005N400`3Qh@0Bh@<0D^400`3Qh@13h@805N400`3Qh@0ih@030>7Q00GQ000Eh@804>4305CQ
00<0hN40A^400`3Qh@0Ch@030>7Q03SQ00<0hN401N4001GQ00<0hN403>4305OQ00<0hN40An4201?Q
00<0hN40>>400`3Qh@05h@005N400`3Qh@09h@<0FN400`3Qh@1:h@030>7Q017Q00<0hN40=n400`3Q
h@05h@005N400`3Qh@05h@@0Fn400`3Qh@1<h@030>7Q017Q00<0hN40=^400`3Qh@05h@003^4300CQ
00<0hN400^4305oQ00<0hN40CN42017Q00<0hN40=^400`3Qh@05h@003^400`3Qh@04h@030>400080
HN400`3Qh@1@h@030>7Q00oQ00<0hN40=N400`3Qh@05h@003n400`3Qh@03h@<0H^400`3Qh@1Bh@80
3n400`3Qh@0eh@030>7Q00GQ000?h@030>7Q00?Q0P1Sh@030>7Q05CQ00<0hN403N400`3Qh@0dh@03
0>7Q00GQ000>h@030>4000CQ0P1Rh@030>7Q05KQ00<0hN403>400`3Qh@0dh@030>7Q00GQ000?h@03
0>7Q00?Q00<0h@00HN400`3Qh@1Gh@803N400`3Qh@0ch@030>7Q00GQ000Eh@030>40063Q00<0hN40
F^400`3Qh@0;h@030>7Q03;Q00<0hN401N4001GQ0P000n40h@1Mh@030>7Q05cQ00<0hN402^400`3Q
h@0bh@030>7Q00GQ000Eh@050>7Qh@00GN400`3Qh@1Mh@802n400`3Qh@0ah@030>7Q00GQ000Eh@05
0>7Qh@00G>400`3Qh@1Ph@030>7Q00WQ00<0hN40<>400`3Qh@05h@005N400`3Qh@02h@030>7Q05WQ
00<0hN40HN4200WQ00<0hN40<>400`3Qh@05h@005N400`3Qh@02h@030>7Q05SQ00<0hN40I>400`3Q
h@07h@030>7Q02oQ00<0hN401N4001GQ0P04h@030>7Q05KQ00<0hN40I^400`3Qh@07h@030>7Q02kQ
00<0hN401N4001GQ00<0hN400n400`3Qh@1Fh@030>7Q06OQ0P07h@030>7Q02kQ00<0hN401N4001GQ
00<0hN401>400`3Qh@1Dh@030>7Q06[Q00<0hN401N400`3Qh@0]h@030>7Q00GQ000Eh@030>7Q00GQ
00<0hN40D^400`3Qh@1/h@801N400`3Qh@0]h@030>7Q00GQ000Eh@030>7Q00GQ00<0hN40D^400`3Q
h@1^h@030>7Q00?Q00<0hN40;>400`3Qh@05h@005N400`3Qh@06h@030>7Q053Q00<0hN40L>400`3Q
h@03h@030>7Q02_Q00<0hN401N4001GQ0P07h@030>7Q053Q00<0hN40LN4200?Q00<0hN40:n400`3Q
h@05h@005N400`3Qh@07h@030>7Q04kQ00<0hN40M>401@3QhN4002cQ00<0hN401N4001GQ00<0hN40
2>400`3Qh@1<h@030>7Q07KQ0P02h@030>7Q02WQ00<0hN401N4001GQ00<0hN402>400`3Qh@1<h@03
0>7Q07SQ00<0h@00:n400`3Qh@05h@005N400`3Qh@09h@030>7Q04[Q00<0hN40N^400`3Q000Zh@03
0>7Q00GQ000Eh@802n400`3Qh@18h@030>7Q07cQ0`0Yh@030>7Q00GQ000Eh@030>7Q00_Q00<0hN40
An400`3Qh@1nh@030>7Q02OQ00<0hN401N4001GQ00<0hN402n400`3Qh@16h@030>7Q083Q0P0Wh@03
0>7Q00GQ0004h@<00n400`3Qh@02h@030>7Q00?Q00<0hN403>400`3Qh@15h@030>7Q087Q0P0Vh@03
0>7Q00GQ0004h@030>7Q00OQ00<0h@001>400`3Qh@0=h@030>7Q04?Q00<0hN40Pn4202GQ00<0hN40
1N4000GQ00<0hN402>400`3Qh@02h@<03^400`3Qh@11h@030>7Q08CQ00@0h@008n400`3Qh@05h@00
1N400`3Qh@06h@801N400`3Qh@0>h@030>7Q043Q00<0hN40Q^40103Qh@0Rh@030>7Q00GQ0004h@03
0>4000OQ00<0hN401>400`3Qh@0?h@030>7Q03oQ00<0hN40Qn400`3Qh@02023Q00<0hN401N4000GQ
00<0hN401^4300CQ00<0hN404>400`3Qh@0mh@030>7Q08SQ00<0hN400^400`3Qh@0Mh@030>7Q00GQ
000Eh@030>7Q017Q00<0hN40>n400`3Qh@2:h@030>7Q00;Q00<0hN407>400`3Qh@05h@005N400`3Q
h@0Ah@030>7Q03[Q00<0hN40S>400`3Qh@02h@807>400`3Qh@05h@005N4201?Q00<0hN40>>400`3Q
h@2>h@030>7Q00?Q00<0hN406N400`3Qh@05h@005N400`3Qh@0Ch@030>7Q03OQ00<0hN40S^400`3Q
h@04h@806N400`3Qh@05h@005N400`3Qh@0Dh@030>7Q03GQ00<0hN40T>400`3Qh@05h@030>7Q01KQ
00<0hN401N4001GQ00<0hN405N400`3Qh@0ch@030>7Q09;Q00<0hN401N400`3Qh@0Eh@030>7Q00GQ
000Eh@030>7Q01KQ00<0hN40<N400`3Qh@2Dh@030>7Q00GQ0P0Eh@030>7Q00GQ000Eh@805n400`3Q
h@0ah@030>7Q09GQ00<0hN401^400`3Qh@0Bh@030>7Q00GQ000Eh@030>7Q01OQ00<0hN40;n400`3Q
h@2Fh@030>7Q00OQ0P0Bh@030>7Q00GQ000Eh@030>7Q01SQ00<0hN40;N400`3Qh@2Hh@030>7Q00SQ
00<0hN403n400`3Qh@05h@005N400`3Qh@0Ih@030>7Q02_Q00<0hN40V^400`3Qh@08h@030>7Q00kQ
00<0hN401N4001GQ00<0hN406^400`3Qh@0Yh@030>7Q09cQ00<0hN402>4200kQ00<0hN401N4001GQ
0P0Lh@030>7Q02OQ00<0hN40W^400`3Qh@09h@030>7Q00_Q00<0hN401N4001GQ00<0hN407>400`3Q
h@0Uh@030>7Q0:3Q00<0hN402N4200_Q00<0hN401N4001GQ00<0hN407N400`3Qh@0Sh@030>7Q0:;Q
00<0hN402^400`3Qh@08h@030>7Q00GQ000Eh@030>7Q01kQ00<0hN408>420:KQ00<0hN402^400`3Q
h@07h@030>7Q00GQ000Eh@030>7Q01oQ00<0hN407^400`3Qh@2Wh@030>7Q00[Q0P07h@030>7Q00GQ
000Eh@030>7Q023Q0P0Lh@80Zn4200cQ00<0hN401>400`3Qh@05h@005N4202?Q00<0hN406>400`3Q
h@2]h@030>7Q00[Q0P04h@030>7Q00GQ000Eh@030>7Q02?Q0P0Gh@030>7Q0:oQ0P0<h@050>7Qh@00
1n4001GQ00<0hN409N4201?Q0P2dh@030>7Q00[Q00@0hN401n4000kQ0P05h@030>7Q02OQ0P0?h@80
]n4300[Q0`07h@004>400`3Qh@02h@030>7Q02WQ0`0:h@80_>4300WQ00<0hN401N4000oQ00<0hN40
0n4302cQ2P31h@T02>40013Q00<0hN400^400`3Qh@3oh@WQ000>h@030>4000CQ00<0hN40on49h@00
3n400`3Qh@03h@030>7Q0?oQ2N4001GQ00<0hN40on49h@00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.66388, -0.23241, 
  0.0308942, 0.0187455}},
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.0147151 0.196201 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11282 -18 -4.5 ]
[.01131 .11282 0 4.5 ]
[.01131 .21092 -6 -4.5 ]
[.01131 .21092 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .50522 -18 -4.5 ]
[.01131 .50522 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.2619 .01472 m
.2619 .02097 L
s
[(2)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(4)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(6)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(8)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11282 m
.03006 .11282 L
s
[(0.5)] .01131 .11282 1 0 Mshowa
.02381 .21092 m
.03006 .21092 L
s
[(1)] .01131 .21092 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1.5)] .01131 .30902 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(2)] .01131 .40712 1 0 Mshowa
.02381 .50522 m
.03006 .50522 L
s
[(2.5)] .01131 .50522 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .0932 m
.02756 .0932 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .32864 m
.02756 .32864 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .56408 m
.02756 .56408 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02381 .40712 L
.2619 .58552 L
.2619 .01472 L
.2619 .58552 L
.5 .25863 L
.5 .01472 L
.5 .25863 L
.7381 .3523 L
.7381 .01472 L
.7381 .3523 L
.97619 .60123 L
.97619 .01472 L
s
.02381 .40712 m
.06244 .46968 L
.10458 .53025 L
.12507 .55459 L
.14415 .57336 L
.16254 .58741 L
.17284 .59342 L
.18221 .59768 L
.18704 .59942 L
.19216 .60092 L
.19654 .60191 L
.19901 .60236 L
.20135 .6027 L
.20265 .60286 L
.20405 .603 L
.20535 .60311 L
.20652 .60319 L
.20719 .60323 L
.20792 .60326 L
.20869 .60329 L
.20941 .60331 L
.21009 .60332 L
.2108 .60332 L
.21209 .60331 L
.21328 .60328 L
.21456 .60322 L
.21577 .60315 L
.21688 .60306 L
.21958 .60279 L
.22214 .60243 L
.22673 .60157 L
.23179 .60028 L
.24205 .5966 L
.25155 .59194 L
.26047 .5865 L
.28065 .57051 L
.29915 .55173 L
.33878 .50047 L
.37689 .44142 L
.41745 .3749 L
.45649 .31427 L
.47641 .28679 L
.49799 .26082 L
.51703 .24187 L
.52785 .23295 L
.53797 .2259 L
.54836 .22004 L
.55785 .21593 L
.56305 .21418 L
.56799 .21287 L
Mistroke
.57321 .21185 L
.57616 .21143 L
.57749 .21129 L
.57889 .21116 L
.58016 .21107 L
.58083 .21103 L
.58155 .21099 L
.58281 .21094 L
.58398 .21092 L
.58469 .21092 L
.58546 .21092 L
.58619 .21093 L
.58686 .21095 L
.58814 .21099 L
.58878 .21103 L
.58949 .21107 L
.59196 .21127 L
.59336 .21142 L
.59463 .21158 L
.59945 .2124 L
.60379 .21341 L
.60853 .2148 L
.61807 .21853 L
.62838 .22391 L
.63772 .22998 L
.65869 .2475 L
.69872 .29409 L
.73724 .35094 L
.77821 .41767 L
.81767 .48084 L
.85561 .5341 L
.87673 .5585 L
.896 .57664 L
.90555 .58401 L
.91595 .59074 L
.92571 .59578 L
.93487 .59937 L
.93939 .60071 L
.94426 .60186 L
.94689 .60234 L
.94966 .60274 L
.95203 .603 L
.95337 .60311 L
.95462 .6032 L
.95593 .60326 L
.95667 .60329 L
.95736 .6033 L
.95861 .60332 L
.95924 .60332 L
.95994 .60331 L
Mistroke
.96114 .60328 L
.96244 .60323 L
.96366 .60316 L
.96477 .60307 L
.96743 .6028 L
.97034 .6024 L
.97562 .60136 L
.97619 .60123 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3ooooooooQ8N4005CQ0`0oh@<0?N4203oQ00<0hN40
1^4005CQ00<0hN40@>400`3Qh@0lh@030>4003gQ00<0h@001n4005GQ00<0hN40?N4403gQ00<0h@00
?N400`3Q0007h@00EN400`3Qh@0mh@030>4003kQ0P0oh@030>7Q00KQ001Dh@030>4003oQ0P0nh@03
0>7Q03gQ00<0h@001n4005GQ00<0hN40?n400`3Qh@0mh@80?^400`3Qh@06h@00on4Qh@005N400`3Q
h@3oh@WQ000Eh@030>7Q0?oQ2N4001GQ00<0hN40on49h@005N400`3Qh@3oh@WQ000?hOl04@01h@00
5N400`3Qh@0=h@030>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@03
0>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@030>7Q00kQ00<0hN40
3N400`3Qh@0=h@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@03
0>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@03
0>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@03
0>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ0P0oh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@03
0>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@00
5N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ0P0oh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@03
0>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40
?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40
?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ0P0oh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ0005h@03
0>7Q00;Q00<0hN400^400`3Qh@03h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4000CQ00<0h@001n400`3Q0004h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4000CQ00<0h@002N400`3Qh@02h@<0?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@00
1>400`3Q0007h@801N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ0004h@03
0>4000OQ00<0hN401>400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ0005h@03
0>7Q00KQ0`04h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40
?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N4203oQ00<0hN40?N400`3Qh@0nh@03
0>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@00
5N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40
?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N4203oQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40
?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40
?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q03kQ00<0hN40
?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@03
0>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@03
0>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@03
0>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q03kQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40?N400`3Qh@0nh@03
0>7Q03kQ00<0hN401N4000gQ1@03h@030>7Q03kQ00<0hN40?N400`3Qh@0nh@030>7Q03kQ00<0hN40
1N4000oQ00<0hN400n400`3Qh@0nh@030>7Q03gQ00<0hN40?^400`3Qh@0nh@030>7Q00GQ000?h@03
0>7Q00?Q0`0nh@030>7Q03gQ00<0hN404^4602KQ00<0hN40?^400`3Qh@05h@003n400`3Qh@03h@03
0>7Q03kQ00<0hN40?N400`3Qh@0?h@<01^45027Q00<0hN40?^400`3Qh@05h@003n400`3Qh@03h@03
0>7Q03kQ00<0hN40?N400`3Qh@0=h@803^4201oQ00<0hN40?^400`3Qh@05h@003^4200GQ00<0hN40
?^400`3Qh@0mh@030>7Q00[Q0`0Bh@030>7Q01cQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03gQ00<0hN402>4201KQ0P0Lh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@03
0>7Q00KQ0P0Jh@806^400`3Qh@0nh@030>7Q00GQ000Eh@80?n400`3Qh@0mh@030>7Q00GQ00<0hN40
7>400`3Qh@0Gh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q00CQ00<0hN40
7^400`3Qh@0Fh@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0mh@030>7Q00?Q00<0hN40
8>4201KQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00<0hN400^400`3Qh@0Sh@03
0>7Q01?Q00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03gQ00D0hN7Q000Wh@030>7Q01;Q
00<0hN40?^400`3Qh@05h@005N4203oQ00<0hN40?N40103Qh@0Yh@030>7Q017Q00<0hN40?^400`3Q
h@05h@005N400`3Qh@0nh@030>7Q03gQ00<0h@00:n400`3Qh@0@h@030>7Q03kQ00<0hN401N4001GQ
00<0hN40?^400`3Qh@0mh@80;>400`3Qh@0@h@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Q
h@0lh@8000?Q0000:n400`3Qh@0?h@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0lh@03
0>7Q00;Q0`0Yh@030>7Q00kQ00<0hN40?^400`3Qh@05h@005N4203oQ00<0hN40>n400`3Qh@06h@80
:>400`3Qh@0=h@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0jh@030>7Q00WQ0`0Uh@03
0>7Q00gQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03WQ0P0>h@809>400`3Qh@0<h@03
0>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0ih@030>7Q00oQ0`0Rh@030>7Q00_Q00<0hN40
?^400`3Qh@05h@005N400`3Qh@0nh@030>7Q03SQ00<0hN404n42027Q00<0hN402^400`3Qh@0nh@03
0>7Q00GQ000Eh@030>7Q03kQ00<0hN40=n400`3Qh@0Fh@<07^400`3Qh@0:h@030>7Q03kQ00<0hN40
1N4001GQ0P0oh@030>7Q03KQ0P0Kh@807N400`3Qh@09h@030>7Q03kQ00<0hN401N4001GQ00<0hN40
?^400`3Qh@0fh@030>7Q01cQ0`0Kh@030>7Q00SQ00<0hN40?^400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q03GQ00<0hN408>4201[Q00<0hN401n400`3Qh@0nh@030>7Q00GQ0003h@D00^400`3Qh@02h@03
0>7Q00?Q00<0hN40?^400`3Qh@0dh@030>7Q02?Q0`0Gh@030>7Q00OQ00<0hN40?^400`3Qh@05h@00
1N400`3Qh@06h@030>4000CQ00<0hN40?^400`3Qh@0ch@80:>4201KQ00<0hN401^400`3Qh@0nh@03
0>7Q00GQ0005h@030>7Q00SQ00<0hN400^4303kQ00<0hN40<n400`3Qh@0Yh@<05>400`3Qh@05h@03
0>7Q03kQ00<0hN401N4000GQ00<0hN401^4200GQ00<0hN40?^400`3Qh@0bh@030>7Q02gQ0P0Ch@03
0>7Q00CQ00<0hN40?^400`3Qh@05h@001N400`3Qh@06h@030>7Q00CQ00<0hN40?^400`3Qh@0ah@03
0>7Q033Q0`0@h@030>7Q00CQ00<0hN40?^400`3Qh@05h@001>4200SQ0`04h@030>7Q03kQ00<0hN40
<N400`3Qh@0ch@803n400`3Qh@03h@030>7Q03kQ00<0hN401N4001GQ00<0hN40?^400`3Qh@0`h@03
0>7Q03KQ0`0=h@030>7Q00;Q00<0hN40?^400`3Qh@05h@005N4203oQ00<0hN40;n400`3Qh@0jh@80
3>401@3QhN40043Q00<0hN401N4001GQ00<0hN40?^400`3Qh@0_h@030>7Q03cQ0`09h@050>7Qh@00
@>400`3Qh@05h@005N400`3Qh@0nh@030>7Q02kQ00<0hN40@>4200SQ00@0hN40@>400`3Qh@05h@00
5N400`3Qh@0nh@030>7Q02gQ00<0hN40@n4300KQ00<0h@00@>400`3Qh@05h@005N400`3Qh@0nh@03
0>7Q02cQ0P18h@801N42043Q00<0hN401N4001GQ00<0hN40?^400`3Qh@0/h@030>7Q04WQ0`02h@80
@>400`3Qh@05h@005N4203oQ00<0hN40:n4204kQ0P000n40h@0oh@030>7Q00GQ000Eh@030>7Q03kQ
00<0hN40:^42057Q0`0oh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN40:^400`3Qh@1Bh@030>7Q03gQ
00<0hN401N4001GQ00<0hN40?^400`3Qh@0Yh@80EN400`3Qh@0lh@030>7Q00GQ000Eh@030>7Q03kQ
00<0hN40:>4205KQ0P0mh@030>7Q00GQ000Eh@80?n400`3Qh@0Wh@80F>4203cQ00<0hN401N4001GQ
00<0hN40?^400`3Qh@0Wh@80F>400`3Q000kh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN409^4205[Q
00<0h@00>^400`3Qh@05h@005N400`3Qh@0nh@030>7Q02GQ0P1Lh@030>4003WQ00<0hN401N4001GQ
00<0hN40?^400`3Qh@0Uh@80G>40103Qh@0hh@030>7Q00GQ000Eh@80?n400`3Qh@0Th@80G^40103Q
h@0gh@030>7Q00GQ000Eh@030>7Q03kQ00<0hN408n400`3Q001Nh@050>7Qh@00=^400`3Qh@05h@00
5N400`3Qh@0nh@030>7Q02;Q00<0h@00H>401@3QhN4003GQ00<0hN401N4001GQ00<0hN40?^400`3Q
h@0Rh@80H^401@3QhN4003CQ00<0hN401N4000kQ0`04h@030>7Q03kQ00<0hN408N400`3Q001Rh@03
0>7Q00;Q00<0hN40<N400`3Qh@05h@003^400`3Qh@04h@030>7Q03kQ00<0hN408>400`3Q001Th@03
0>7Q00;Q00<0hN40<>400`3Qh@05h@003n400`3Qh@03h@<0?^400`3Qh@0Ph@80IN400`3Qh@03h@03
0>7Q02oQ00<0hN401N4000oQ00<0hN400n4303kQ00<0hN407n400`3Q001Vh@030>7Q00?Q00<0hN40
;^400`3Qh@05h@003^400`3Q0004h@8000?Q0>40?>400`3Qh@0Nh@030>4006OQ00<0hN400n400`3Q
h@0^h@030>7Q00GQ000?h@030>7Q00?Q00D0h@3Q000lh@030>7Q01gQ00@0hN40J>400`3Qh@03h@03
0>7Q02gQ00<0hN401N4001GQ00<0h@000^4203[Q00<0hN407N400`3Q001Zh@030>7Q00?Q00<0hN40
;>400`3Qh@05h@005N420003h@3Q00;Q00<0hN40=n400`3Qh@0Lh@030>4006_Q00<0hN401>400`3Q
h@0[h@030>7Q00GQ000Eh@050>7Qh@000n400`3Qh@0fh@030>7Q01_Q00@0hN40K>400`3Qh@04h@03
0>7Q02[Q00<0hN401N4001GQ00D0hN7Q0004h@80=^400`3Qh@0Jh@040>7Q06kQ00<0hN401>400`3Q
h@0Yh@030>7Q00GQ000Eh@030>7Q00;Q00<0hN400n400`3Qh@0ch@030>7Q01[Q00@0hN40K^400`3Q
h@05h@030>7Q02SQ00<0hN401N4001GQ00<0hN400^400`3Qh@04h@030>7Q03;Q00<0hN406N40103Q
h@1`h@030>7Q00GQ00<0hN409n400`3Qh@05h@005N4200CQ00<0hN401>4203;Q00<0hN406>40103Q
h@1bh@030>7Q00GQ00<0hN409^400`3Qh@05h@005N400`3Qh@03h@030>7Q00KQ00<0hN40;n400`3Q
h@0Hh@040>7Q07;Q00<0hN401^400`3Qh@0Uh@030>7Q00GQ000Eh@030>7Q00CQ00<0hN401^400`3Q
h@0^h@030>7Q01OQ00@0hN40M>400`3Qh@06h@030>7Q02CQ00<0hN401N4001GQ00<0hN401N400`3Q
h@06h@80;^400`3Qh@0Fh@040>7Q07GQ00<0hN401n400`3Qh@0Sh@030>7Q00GQ000Eh@030>7Q00GQ
00<0hN402>400`3Qh@0[h@030>7Q01GQ00D0hN7Q001fh@030>7Q00OQ00<0hN408^400`3Qh@05h@00
5N400`3Qh@06h@030>7Q00SQ00<0hN40:^400`3Qh@0Eh@040>7Q07SQ00<0hN401n400`3Qh@0Qh@03
0>7Q00GQ000Eh@801n400`3Qh@09h@80:^400`3Qh@0Dh@050>7Qh@00N>400`3Qh@08h@030>7Q023Q
00<0hN401N4001GQ00<0hN401n400`3Qh@0:h@030>7Q02OQ00<0hN404n401@3QhN4007[Q00<0hN40
2>400`3Qh@0Oh@030>7Q00GQ000Eh@030>7Q00SQ00<0hN402^4202OQ00<0hN404n40103Qh@1lh@03
0>7Q00SQ00<0hN407^400`3Qh@05h@005N400`3Qh@08h@030>7Q00cQ00<0hN409>400`3Qh@0Bh@05
0>7Qh@00O>400`3Qh@09h@030>7Q01gQ00<0hN401N4001GQ00<0hN402N400`3Qh@0<h@030>7Q02?Q
00<0hN404N401@3QhN4007kQ00<0hN402N400`3Qh@0Lh@030>7Q00GQ000Eh@802n400`3Qh@0<h@80
8n400`3Qh@0@h@050>7Qh@00P>400`3Qh@09h@030>7Q01_Q00<0hN401N4001GQ00<0hN402n400`3Q
h@0=h@030>7Q023Q00<0hN404>401@3QhN40083Q00<0hN402^400`3Qh@0Jh@030>7Q00GQ000Eh@03
0>7Q00_Q00<0hN403^400`3Qh@0Oh@030>7Q00oQ00D0hN7Q0022h@030>7Q00[Q00<0hN406N400`3Q
h@05h@001>4300?Q00<0hN400^400`3Qh@03h@030>7Q00cQ00<0hN403^4201oQ00<0hN403^400`3Q
h@02h@030>7Q087Q00<0hN402^400`3Qh@0Hh@030>7Q00GQ0004h@030>7Q00OQ00<0h@001>400`3Q
h@0=h@030>7Q00oQ00<0hN407>400`3Qh@0>h@050>7Qh@00QN400`3Qh@0:h@030>7Q01OQ00<0hN40
1N4000GQ00<0hN402>400`3Qh@02h@<03^400`3Qh@0?h@030>7Q01_Q00<0hN403N401@3QhN4008KQ
00<0hN402n400`3Qh@0Fh@030>7Q00GQ0005h@030>7Q00KQ0P05h@030>7Q00kQ00<0hN404>4201_Q
00<0hN403>401@3QhN4008SQ00<0hN402n400`3Qh@0Eh@030>7Q00GQ0004h@030>4000OQ00<0hN40
1>400`3Qh@0?h@030>7Q017Q00<0hN406>400`3Qh@0;h@030>7Q00;Q00<0hN40Qn400`3Qh@0;h@03
0>7Q01CQ00<0hN401N4000GQ00<0hN401^4300CQ00<0hN404>400`3Qh@0Ah@030>7Q01OQ00<0hN40
2n401@3QhN4008[Q00<0hN403>400`3Qh@0Ch@030>7Q00GQ000Eh@030>7Q017Q00<0hN404N4201OQ
00<0hN402^401@3QhN4008cQ00<0hN403>400`3Qh@0Bh@030>7Q00GQ000Eh@030>7Q017Q00<0hN40
4n400`3Qh@0Dh@030>7Q00WQ00D0hN7Q002>h@030>7Q00cQ00<0hN404N400`3Qh@05h@005N4201?Q
00<0hN404n4201CQ00<0hN402N40103Qh@2@h@030>7Q00cQ00<0hN404>400`3Qh@05h@005N400`3Q
h@0Ch@030>7Q01CQ00<0hN404N400`3Qh@08h@050>7Qh@00T>400`3Qh@0=h@030>7Q00oQ00<0hN40
1N4001GQ00<0hN405>400`3Qh@0Dh@030>7Q013Q00<0hN401n401@3QhN4009;Q00<0hN403N400`3Q
h@0>h@030>7Q00GQ000Eh@030>7Q01GQ00<0hN405>42013Q00<0hN401^401@3QhN4009CQ00<0hN40
3N400`3Qh@0=h@030>7Q00GQ000Eh@030>7Q01KQ00<0hN405N400`3Qh@0=h@030>7Q00KQ00@0hN40
U^400`3Qh@0<h@030>7Q00gQ00<0hN401N4001GQ0P0Gh@030>7Q01KQ00<0hN403>400`3Qh@05h@05
0>7Qh@00Un400`3Qh@0<h@030>7Q00cQ00<0hN401N4001GQ00<0hN405n400`3Qh@0Fh@803>400`3Q
h@04h@050>7Qh@00V>400`3Qh@0=h@030>7Q00_Q00<0hN401N4001GQ00<0hN406>400`3Qh@0Gh@03
0>7Q00WQ00<0hN400n401@3QhN4009[Q00<0hN403N400`3Qh@0:h@030>7Q00GQ000Eh@030>7Q01WQ
00<0hN405n400`3Qh@08h@030>7Q00?Q00@0hN40W>400`3Qh@0=h@030>7Q00WQ00<0hN401N4001GQ
00<0hN406^400`3Qh@0Gh@802>400`3Qh@02h@040>7Q09kQ00<0hN403N400`3Qh@08h@030>7Q00GQ
000Eh@807>400`3Qh@0Hh@030>7Q00GQ00D0hN7Q0002h@030>7Q09kQ00<0hN403N400`3Qh@07h@03
0>7Q00GQ000Eh@030>7Q01cQ00<0hN406>400`3Qh@04h@070>7Qh@3Q002Rh@030>7Q00gQ00<0hN40
1^400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01SQ0P04h@060>7Q0>40Y>400`3Qh@0=h@030>7Q00GQ
00<0hN401N4001GQ00<0hN407^400`3Qh@0Ih@060>7Qh@3Q0`2Vh@030>7Q00gQ00<0hN401>400`3Q
h@05h@005N400`3Qh@0Oh@030>7Q01WQ00<0hN400`2Yh@030>7Q00gQ00<0hN400n400`3Qh@05h@00
5N400`3Qh@0Ph@806^440:_Q0P0>h@030>7Q00;Q00<0hN401N4001GQ0P0Sh@030>7Q01SQ0P2^h@03
0>7Q00cQ00D0hN7Q0007h@005N400`3Qh@0Sh@805n400`3Qh@2_h@803N40103Qh@07h@005N400`3Q
h@0Uh@804n420;CQ00<0hN402n400`3Q0007h@003^4200GQ00<0hN409n4200oQ0P2gh@<02n4200OQ
000@h@030>7Q00;Q00<0hN40:N4300[Q0P2lh@<02N400`3Qh@05h@003n400`3Qh@03h@<0;>4:0<7Q
2@08h@004>400`3Qh@02h@030>7Q0?oQ2N4000kQ00<0h@001>400`3Qh@3oh@WQ000?h@030>7Q00?Q
00<0hN40on49h@005N400`3Qh@3oh@WQ0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.66388, -0.23241, 
  0.0308942, 0.0187455}},
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.0147151 0.196201 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11282 -18 -4.5 ]
[.01131 .11282 0 4.5 ]
[.01131 .21092 -6 -4.5 ]
[.01131 .21092 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .50522 -18 -4.5 ]
[.01131 .50522 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.2619 .01472 m
.2619 .02097 L
s
[(2)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(4)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(6)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(8)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11282 m
.03006 .11282 L
s
[(0.5)] .01131 .11282 1 0 Mshowa
.02381 .21092 m
.03006 .21092 L
s
[(1)] .01131 .21092 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1.5)] .01131 .30902 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(2)] .01131 .40712 1 0 Mshowa
.02381 .50522 m
.03006 .50522 L
s
[(2.5)] .01131 .50522 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .0932 m
.02756 .0932 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .32864 m
.02756 .32864 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .56408 m
.02756 .56408 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02381 .40712 L
.14286 .57222 L
.14286 .01472 L
.14286 .57222 L
.2619 .58552 L
.2619 .01472 L
.2619 .58552 L
.38095 .43481 L
.38095 .01472 L
.38095 .43481 L
.5 .25863 L
.5 .01472 L
.5 .25863 L
.61905 .21898 L
.61905 .01472 L
.61905 .21898 L
.7381 .3523 L
.7381 .01472 L
.7381 .3523 L
.85714 .53602 L
.85714 .01472 L
.85714 .53602 L
.97619 .60123 L
.97619 .01472 L
s
.02381 .40712 m
.06244 .46968 L
.10458 .53025 L
.12507 .55459 L
.14415 .57336 L
.16254 .58741 L
.17284 .59342 L
.18221 .59768 L
.18704 .59942 L
.19216 .60092 L
.19654 .60191 L
.19901 .60236 L
.20135 .6027 L
.20265 .60286 L
.20405 .603 L
.20535 .60311 L
.20652 .60319 L
.20719 .60323 L
.20792 .60326 L
.20869 .60329 L
.20941 .60331 L
.21009 .60332 L
.2108 .60332 L
.21209 .60331 L
.21328 .60328 L
.21456 .60322 L
.21577 .60315 L
.21688 .60306 L
.21958 .60279 L
.22214 .60243 L
.22673 .60157 L
.23179 .60028 L
.24205 .5966 L
.25155 .59194 L
.26047 .5865 L
.28065 .57051 L
.29915 .55173 L
.33878 .50047 L
.37689 .44142 L
.41745 .3749 L
.45649 .31427 L
.47641 .28679 L
.49799 .26082 L
.51703 .24187 L
.52785 .23295 L
.53797 .2259 L
.54836 .22004 L
.55785 .21593 L
.56305 .21418 L
.56799 .21287 L
Mistroke
.57321 .21185 L
.57616 .21143 L
.57749 .21129 L
.57889 .21116 L
.58016 .21107 L
.58083 .21103 L
.58155 .21099 L
.58281 .21094 L
.58398 .21092 L
.58469 .21092 L
.58546 .21092 L
.58619 .21093 L
.58686 .21095 L
.58814 .21099 L
.58878 .21103 L
.58949 .21107 L
.59196 .21127 L
.59336 .21142 L
.59463 .21158 L
.59945 .2124 L
.60379 .21341 L
.60853 .2148 L
.61807 .21853 L
.62838 .22391 L
.63772 .22998 L
.65869 .2475 L
.69872 .29409 L
.73724 .35094 L
.77821 .41767 L
.81767 .48084 L
.85561 .5341 L
.87673 .5585 L
.896 .57664 L
.90555 .58401 L
.91595 .59074 L
.92571 .59578 L
.93487 .59937 L
.93939 .60071 L
.94426 .60186 L
.94689 .60234 L
.94966 .60274 L
.95203 .603 L
.95337 .60311 L
.95462 .6032 L
.95593 .60326 L
.95667 .60329 L
.95736 .6033 L
.95861 .60332 L
.95924 .60332 L
.95994 .60331 L
Mistroke
.96114 .60328 L
.96244 .60323 L
.96366 .60316 L
.96477 .60307 L
.96743 .6028 L
.97034 .6024 L
.97562 .60136 L
.97619 .60123 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3ooooooooQ8N4005CQ0`0oh@<0?N4203oQ00<0hN40
1^4005CQ00<0hN40@>400`3Qh@0lh@030>4003gQ00<0h@001n4005GQ00<0hN40?N4403gQ00<0h@00
?N400`3Q0007h@00EN400`3Qh@0mh@030>4003kQ0P0oh@030>7Q00KQ001Dh@030>4003oQ0P0nh@03
0>7Q03gQ00<0h@001n4005GQ00<0hN40?n400`3Qh@0mh@80?^400`3Qh@06h@00on4Qh@005N400`3Q
h@3oh@WQ000Eh@030>7Q0?oQ2N4001GQ00<0hN40on49h@005N400`3Qh@3oh@WQ000?hOl04@01h@00
5N400`3Qh@0=h@030>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@03
0>7Q00gQ00<0hN403^400`3Qh@0=h@030>7Q00gQ00<0hN403N400`3Qh@0=h@030>7Q00kQ00<0hN40
3N400`3Qh@0=h@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@807^400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@80
7^400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@05h@001N400`3Qh@02h@030>7Q00;Q00<0hN400n400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4000CQ00<0h@00
1n400`3Q0004h@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@001>400`3Q0009h@030>7Q00;Q0`0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4000CQ00<0h@00
1n4200GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ0004h@030>4000OQ00<0hN401>400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4000GQ00<0hN40
1^4300CQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
1N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@03
0>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
1N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN40
7N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@05h@003N4500?Q00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000?h@030>7Q00?Q00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000?h@030>7Q00?Q0`0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q01;Q1P06h@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000?h@030>7Q00?Q00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407N400`3Qh@0?h@<01^450003h@3Q01kQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ
000?h@030>7Q00?Q00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@0=h@803^4201oQ
00<0hN407^400`3Qh@0Mh@030>7Q00GQ000>h@801N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Q
h@0Mh@030>7Q00[Q0`0=h@<00^400`3Qh@0Lh@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Mh@030>7Q00SQ0P0=h@<01^4201cQ00<0hN407^400`3Q
h@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN401^4200cQ
0`0:h@<06^400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN407N400`3Q
h@0Mh@030>7Q00GQ00<0hN402N4300gQ00@0hN406N400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ
00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN407N400`3Qh@04h@030>7Q00OQ0`0Ah@040>7Q01SQ
00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ
00<0hN400n400`3Qh@05h@<05N400`3Qh@0201KQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@03
0>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0hN400^400`3Qh@03h@<06N401@3QhN4001GQ
00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ
00D0hN7Q0003h@<07N401@3QhN4001CQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@807^400`3Q
h@0Nh@030>7Q01gQ00<0hN407N401@3Qh@3Q00<08N401@3QhN4001?Q00<0hN407^400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01gQ00<0h@000P0Uh@050>7Qh@00
4^400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
7N4202WQ00@0hN404^400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN407>4202[Q00D0hN7Q000Ah@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Lh@030>7Q02[Q00D0hN7Q000@h@030>7Q01kQ00<0hN40
7N400`3Qh@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@030>7Q01_Q00<0hN40;>401@3QhN4000oQ
00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01[Q
00<0hN40;^40103Qh@0?h@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ
00<0hN407N400`3Qh@0Jh@030>7Q02oQ00@0hN403^400`3Qh@0Nh@030>7Q01gQ00<0hN401N4001GQ
00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN406N400`3Qh@0ah@040>7Q00gQ00<0hN407^400`3Q
h@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q01SQ00<0hN40<n40103Q
h@0<h@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Q
h@0Gh@80=N40103Qh@0<h@030>7Q01kQ00<0hN407N400`3Qh@05h@005N4201kQ00<0hN407^400`3Q
h@0Mh@030>7Q01KQ0P0gh@040>7Q00_Q00<0hN407^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ
00<0hN407^400`3Qh@0Mh@030>7Q01KQ00<0hN40=n40103Qh@0:h@030>7Q01kQ00<0hN407N400`3Q
h@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Eh@80>^40103Qh@09h@030>7Q01kQ
00<0hN407N400`3Qh@05h@000n4500;Q00<0hN400^400`3Qh@03h@030>7Q01gQ00<0hN407^400`3Q
h@0Mh@030>7Q01CQ0P0lh@030>4000WQ00<0hN407^400`3Qh@0Mh@030>7Q00GQ0005h@030>7Q00KQ
00<0h@001>400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Ch@80?^400`3Q0008h@030>7Q01kQ
00<0hN407N400`3Qh@05h@001N400`3Qh@08h@030>7Q00;Q0`0Mh@030>7Q01kQ00<0hN407N400`3Q
h@0Ch@80?n400`3Q0007h@030>7Q01kQ00<0hN407N400`3Qh@05h@001N400`3Qh@06h@801N400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0Bh@80@>40103Qh@06h@030>7Q01kQ00<0hN407N400`3Q
h@05h@001N400`3Qh@06h@030>7Q00CQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN404N4204;Q
00<0h@001^400`3Qh@0Nh@030>7Q01gQ00<0hN401N4000CQ0P08h@<01>400`3Qh@0Mh@030>7Q01kQ
00<0hN407N400`3Qh@0Ah@80@n400`3Q0005h@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Q
h@0Mh@030>7Q01kQ00<0hN407N400`3Qh@0@h@80AN400`3Q0004h@030>7Q01kQ00<0hN407N400`3Q
h@05h@005N4201kQ00<0hN407^400`3Qh@0Mh@030>7Q00oQ0P17h@030>4000?Q00<0hN407^400`3Q
h@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00oQ0P18h@800n400`3Q
h@0Nh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN403^4204[Q
0P02h@030>7Q01kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Q
h@0=h@80C>420003h@3Q01oQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@0=h@80C>40103Q000Ph@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN403>4204kQ0`0Ph@030>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN40
7N400`3Qh@0<h@030>7Q04kQ0P0Ph@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN402n42057Q0P0Oh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@03
0>7Q01gQ00<0hN402^4205?Q00<0hN407N400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN40
7^400`3Qh@0Mh@030>7Q00[Q00<0hN40D^4201kQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@09h@80EN400`3Qh@0Lh@030>7Q01gQ00<0hN401N4001GQ0P0Nh@03
0>7Q01kQ00<0hN407N400`3Qh@08h@80En400`3Qh@0Kh@030>7Q01gQ00<0hN401N4001GQ00<0hN40
7N400`3Qh@0Nh@030>7Q01gQ00<0hN402>400`3Qh@1Fh@030>7Q01_Q00<0hN407N400`3Qh@05h@00
5N400`3Qh@0Mh@030>7Q01kQ00<0hN407N400`3Qh@07h@80FN400`3Qh@0Jh@030>7Q01gQ00<0hN40
1N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401^4205_Q00<0hN406N400`3Qh@0Mh@03
0>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00KQ00<0hN40F^400`3Qh@0Ih@03
0>7Q01gQ00<0hN401N4001GQ0P0Nh@030>7Q01kQ00<0hN407N400`3Qh@05h@80GN400`3Qh@0Hh@03
0>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN401N400`3Qh@1Lh@80
6N400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q01gQ00<0hN407^400`3Qh@0Mh@030>7Q00CQ00<0hN40
G^400`3Qh@0Gh@030>7Q01gQ00<0hN401N4001GQ00<0hN407N400`3Qh@0Nh@030>7Q01gQ00<0hN40
0n42067Q00<0hN405^400`3Qh@0Mh@030>7Q00GQ000>h@<01>400`3Qh@0Mh@030>7Q01kQ00<0hN40
7N400`3Qh@03h@030>7Q063Q0P0Gh@030>7Q01gQ00<0hN401N4000kQ00<0hN401>400`3Qh@0Mh@03
0>7Q01kQ00<0hN407N400`3Qh@02h@030>7Q06;Q00<0hN405N400`3Qh@0Mh@030>7Q00GQ000?h@03
0>7Q00?Q0`0Mh@030>7Q01kQ00<0hN407N40103QhN4206CQ0P0Fh@030>7Q01gQ00<0hN401N4000oQ
00<0hN400n4201kQ00<0hN407^400`3Qh@0Mh@050>7Qh@00I^4201GQ00<0hN407N400`3Qh@05h@00
3^400`3Q0004h@807^400`3Qh@0Nh@030>7Q01gQ00@0hN40In4201GQ00<0hN407N400`3Qh@05h@00
3n400`3Qh@03h@030>4001gQ00<0hN407^400`3Qh@0Mh@040>7Q06SQ0P0Dh@030>7Q01gQ00<0hN40
1N4001GQ00@0h@007>400`3Qh@0Nh@030>7Q01gQ00<0h@00J^4201?Q00<0hN407N400`3Qh@05h@00
5N420003h@0001_Q00<0hN407^400`3Qh@0Mh@80Jn4201?Q00<0hN407N400`3Qh@05h@005N401@3Q
hN4001_Q00<0hN407^400`3Qh@0Mh@80K>4201;Q00<0hN407N400`3Qh@05h@005N40103QhN4201[Q
00<0hN407^400`3Qh@0Mh@030>7Q06cQ0P0Ah@030>7Q01gQ00<0hN401N4001GQ00<0hN400^4201WQ
00<0hN407^400`3Qh@0Lh@80K^42017Q00<0hN407N400`3Qh@05h@005N400`3Qh@02h@806N400`3Q
h@0Nh@030>7Q01_Q0P1`h@804>400`3Qh@0Mh@030>7Q00GQ000Eh@801>4201SQ00<0hN407^400`3Q
h@0Kh@030>7Q073Q00<0hN403^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00?Q00<0h@005n400`3Q
h@0Nh@030>7Q01[Q0P1bh@803n400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00CQ00<0h@005^400`3Q
h@0Nh@030>7Q01WQ0P1dh@803^400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00GQ0P0Fh@030>7Q01kQ
00<0hN406>4207GQ0P0>h@030>7Q01gQ00<0hN401N4001GQ00<0hN401N400`3Q000Eh@030>7Q01kQ
00<0hN406>4207KQ0P0=h@030>7Q01gQ00<0hN401N4001GQ00<0hN401^400`3Q000Dh@030>7Q01kQ
00<0hN405n4207SQ0P0<h@030>7Q01gQ00<0hN401N4001GQ0P07h@030>4001CQ00<0hN407^400`3Q
h@0Fh@030>4007SQ0P0<h@030>7Q01gQ00<0hN401N4001GQ00<0hN401n400`3Q000Ch@030>7Q01kQ
00<0hN405N400`3Q001jh@802n400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00SQ00<0h@004^400`3Q
h@0Nh@030>7Q01CQ00<0h@00O>4200[Q00<0hN407N400`3Qh@05h@005N400`3Qh@08h@040>7Q017Q
00<0hN407^400`3Qh@0Dh@030>4007cQ0P0:h@030>7Q01gQ00<0hN401N4001GQ00<0hN402N400`3Q
000Ah@030>7Q01kQ00<0hN404n400`3Q001nh@802N400`3Qh@0Mh@030>7Q00GQ000Eh@802n400`3Q
000@h@030>7Q01kQ00<0hN404^400`3Q0020h@802>400`3Qh@0Mh@030>7Q00GQ000Eh@030>7Q00_Q
00<0h@003n400`3Qh@0Nh@030>7Q017Q00@0hN40P>4200SQ00<0hN407N400`3Qh@05h@005N400`3Q
h@0;h@030>4000oQ00<0hN407^400`3Qh@0Ah@030>4008;Q0P07h@030>7Q01gQ00<0hN401N4000CQ
0`03h@030>7Q00;Q00<0hN400n400`3Qh@0<h@030>4000kQ00<0hN407^400`3Qh@0@h@040>7Q08?Q
0P06h@030>7Q01gQ00<0hN401N4000CQ00<0hN401n400`3Q0004h@030>7Q00gQ00<0h@003N400`3Q
h@0Nh@030>7Q00oQ00@0hN40QN400`3Qh@04h@030>7Q01gQ00<0hN401N4000GQ00<0hN402>400`3Q
h@02h@<03^4200gQ00<0hN407^400`3Qh@0>h@040>7Q08KQ0P05h@030>7Q01gQ00<0hN401N4000GQ
00<0hN401^4200GQ00<0hN403^400`3Q000<h@030>7Q01kQ00<0hN403N40103Qh@28h@801>400`3Q
h@0Mh@030>7Q00GQ0004h@030>4000OQ00<0hN401>400`3Qh@0?h@030>4000_Q00<0hN407^400`3Q
h@0=h@040>7Q08WQ00<0hN400^400`3Qh@0Mh@030>7Q00GQ0005h@030>7Q00KQ0`04h@030>7Q013Q
00<0h@002^400`3Qh@0Nh@030>7Q00cQ00@0hN40R^4200?Q00<0hN407N400`3Qh@05h@005N400`3Q
h@0Ah@802^400`3Qh@0Nh@030>7Q00_Q00@0hN40S>4200;Q00<0hN407N400`3Qh@05h@005N400`3Q
h@0Ah@030>4000WQ00<0hN407^400`3Qh@0:h@040>7Q08kQ00@0hN407n400`3Qh@05h@005N4201?Q
00<0h@002>400`3Qh@0Nh@030>7Q00WQ00@0hN40T>400`3Q000Oh@030>7Q00GQ000Eh@030>7Q01?Q
0P08h@030>7Q01kQ00<0hN402N40103Qh@2@h@<07n400`3Qh@05h@005N400`3Qh@0Dh@801n400`3Q
h@0Nh@030>7Q00SQ00@0hN40T^4201oQ00<0hN401N4001GQ00<0hN405N4200KQ00<0hN407^400`3Q
h@07h@040>7Q09CQ00<0hN407N400`3Qh@05h@005N400`3Qh@0Fh@801N400`3Qh@0Nh@030>7Q00KQ
00@0hN40U^4201gQ00<0hN401N4001GQ0P0Gh@801N400`3Qh@0Nh@030>7Q00KQ00@0hN40Un4301_Q
00<0hN401N4001GQ00<0hN405n4200CQ00<0hN407^400`3Qh@05h@040>7Q09SQ00<0hN400P0Ih@03
0>7Q00GQ000Eh@030>7Q01SQ0P03h@030>7Q01kQ00<0hN401>40103Qh@2Jh@040>7Qh@805n400`3Q
h@05h@005N400`3Qh@0Ih@050>7Qh@008>400`3Qh@03h@040>7Q09cQ00<0hN400^4201GQ00<0hN40
1N4001GQ00<0hN406^40103Qh@0Ph@030>7Q00;Q00@0hN40W^400`3Qh@03h@804n400`3Qh@05h@00
5N4201cQ00<0h@008>400`3Qh@02h@030>400:3Q00<0hN401>42017Q00<0hN401N4001GQ00<0hN40
7>42023Q00L0hN7Q0>400:;Q00<0hN401N400`3Qh@0>h@030>7Q00GQ000Eh@030>7Q01cQ0P0Ph@06
0>7Q0>40Y>400`3Qh@05h@803^400`3Qh@05h@005N400`3Qh@0Mh@H06n400`3Q00020:KQ00<0hN40
1^4200cQ00<0hN401N4001GQ00<0hN407n40103QhN4;013Q00<0h@00ZN400`3Qh@07h@802^400`3Q
h@05h@005N400`3Qh@0Ph@803>4;00GQ0P2[h@802N4200SQ00<0hN401N4001GQ0P0Sh@030>7Q01CQ
1P2^h@030>7Q00SQ0P06h@030>7Q00GQ000Eh@030>7Q02?Q0P0Gh@030>7Q0:oQ0P0:h@801>400`3Q
h@05h@005N400`3Qh@0Uh@804n420;CQ00<0hN402N4200;Q00<0hN401N4000kQ0P05h@030>7Q02OQ
0P0?h@80]n4300[Q0`07h@004>400`3Qh@02h@030>7Q02WQ0`0:h@80_>4300WQ00<0hN401N4000oQ
00<0hN400n4302cQ2P31h@T02>40013Q00<0hN400^400`3Qh@3oh@WQ000>h@030>4000CQ00<0hN40
on49h@003n400`3Qh@03h@030>7Q0?oQ2N4001GQ00<0hN40on49h@00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.66388, -0.23241, 
  0.0308942, 0.0187455}},
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"Integ", "AnimateTrapezoidal"}]
}, Open  ]],

Cell["", "Text"],

Cell["Newton", "Section",
  CellTags->"Newton"],

Cell["\<\
The Newton package provides a function useful for visualizing Newton's \
method, as discussed in Chapter 14 of Introduction to Scientific Programming.
Before the functions can be used, the package must be loaded with\
\>", "Text",
  CellTags->"Newton"],

Cell[BoxData[
    \(Needs["\<ISP`Newton`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"Newton"],

Cell["AnimateNewton", "Subsection",
  CellTags->{"Newton", "AnimateNewton"}],

Cell[TextData[
"\[FilledSmallSquare] AnimateNewton[f, guess, n, {a,b}] produces an animation \
with n+1 frames illustrating the use of Newton's method to find the root of \
f[x] beginning with a guess of x=guess.  The x-axis between a and b is \
shown."], "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Newton", "AnimateNewton"}],

Cell[TextData[
"\[FilledSmallSquare] The parameter f must be a single-parameter, real-valued \
function.\n\[FilledSmallSquare] The parameter guess must be a real number.\n\
\[FilledSmallSquare] The parameter n must be a positive integer.\n\
\[FilledSmallSquare] The parameters a and b must be real-valued with a < b."],
   "Text",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Newton", "AnimateNewton"}],

Cell["Examples", "Subsubsection",
  CellFrame->{{0, 0}, {0, 0.5}},
  CellTags->{"Newton", "AnimateNewton"}],

Cell[BoxData[
    \(Needs["\<ISP`Newton`\>"]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"Newton", "AnimateNewton"}],

Cell[BoxData[
    \(g[x_]\  := \ x^3\  - \ 9*x\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"Newton", "AnimateNewton"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnimateNewton[g, \ 1.3, \ 3, \ {\(-3.5\), \ 3.5}]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"Newton", "AnimateNewton"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.136054 0.311229 0.0228331 [
[.09184 .29873 -6 -9 ]
[.09184 .29873 6 0 ]
[.22789 .29873 -6 -9 ]
[.22789 .29873 6 0 ]
[.36395 .29873 -6 -9 ]
[.36395 .29873 6 0 ]
[.63605 .29873 -3 -9 ]
[.63605 .29873 3 0 ]
[.77211 .29873 -3 -9 ]
[.77211 .29873 3 0 ]
[.90816 .29873 -3 -9 ]
[.90816 .29873 3 0 ]
[.4875 .0829 -18 -4.5 ]
[.4875 .0829 0 4.5 ]
[.4875 .19706 -12 -4.5 ]
[.4875 .19706 0 4.5 ]
[.4875 .42539 -6 -4.5 ]
[.4875 .42539 0 4.5 ]
[.4875 .53956 -12 -4.5 ]
[.4875 .53956 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.09184 .31123 m
.09184 .31748 L
s
[(-3)] .09184 .29873 0 1 Mshowa
.22789 .31123 m
.22789 .31748 L
s
[(-2)] .22789 .29873 0 1 Mshowa
.36395 .31123 m
.36395 .31748 L
s
[(-1)] .36395 .29873 0 1 Mshowa
.63605 .31123 m
.63605 .31748 L
s
[(1)] .63605 .29873 0 1 Mshowa
.77211 .31123 m
.77211 .31748 L
s
[(2)] .77211 .29873 0 1 Mshowa
.90816 .31123 m
.90816 .31748 L
s
[(3)] .90816 .29873 0 1 Mshowa
.125 Mabswid
.11905 .31123 m
.11905 .31498 L
s
.14626 .31123 m
.14626 .31498 L
s
.17347 .31123 m
.17347 .31498 L
s
.20068 .31123 m
.20068 .31498 L
s
.2551 .31123 m
.2551 .31498 L
s
.28231 .31123 m
.28231 .31498 L
s
.30952 .31123 m
.30952 .31498 L
s
.33673 .31123 m
.33673 .31498 L
s
.39116 .31123 m
.39116 .31498 L
s
.41837 .31123 m
.41837 .31498 L
s
.44558 .31123 m
.44558 .31498 L
s
.47279 .31123 m
.47279 .31498 L
s
.52721 .31123 m
.52721 .31498 L
s
.55442 .31123 m
.55442 .31498 L
s
.58163 .31123 m
.58163 .31498 L
s
.60884 .31123 m
.60884 .31498 L
s
.66327 .31123 m
.66327 .31498 L
s
.69048 .31123 m
.69048 .31498 L
s
.71769 .31123 m
.71769 .31498 L
s
.7449 .31123 m
.7449 .31498 L
s
.79932 .31123 m
.79932 .31498 L
s
.82653 .31123 m
.82653 .31498 L
s
.85374 .31123 m
.85374 .31498 L
s
.88095 .31123 m
.88095 .31498 L
s
.06463 .31123 m
.06463 .31498 L
s
.03741 .31123 m
.03741 .31498 L
s
.0102 .31123 m
.0102 .31498 L
s
.93537 .31123 m
.93537 .31498 L
s
.96259 .31123 m
.96259 .31498 L
s
.9898 .31123 m
.9898 .31498 L
s
.25 Mabswid
0 .31123 m
1 .31123 L
s
.5 .0829 m
.50625 .0829 L
s
[(-10)] .4875 .0829 1 0 Mshowa
.5 .19706 m
.50625 .19706 L
s
[(-5)] .4875 .19706 1 0 Mshowa
.5 .42539 m
.50625 .42539 L
s
[(5)] .4875 .42539 1 0 Mshowa
.5 .53956 m
.50625 .53956 L
s
[(10)] .4875 .53956 1 0 Mshowa
.125 Mabswid
.5 .10573 m
.50375 .10573 L
s
.5 .12856 m
.50375 .12856 L
s
.5 .1514 m
.50375 .1514 L
s
.5 .17423 m
.50375 .17423 L
s
.5 .2199 m
.50375 .2199 L
s
.5 .24273 m
.50375 .24273 L
s
.5 .26556 m
.50375 .26556 L
s
.5 .2884 m
.50375 .2884 L
s
.5 .33406 m
.50375 .33406 L
s
.5 .35689 m
.50375 .35689 L
s
.5 .37973 m
.50375 .37973 L
s
.5 .40256 m
.50375 .40256 L
s
.5 .44823 m
.50375 .44823 L
s
.5 .47106 m
.50375 .47106 L
s
.5 .49389 m
.50375 .49389 L
s
.5 .51673 m
.50375 .51673 L
s
.5 .06007 m
.50375 .06007 L
s
.5 .03723 m
.50375 .03723 L
s
.5 .0144 m
.50375 .0144 L
s
.5 .56239 m
.50375 .56239 L
s
.5 .58523 m
.50375 .58523 L
s
.5 .60806 m
.50375 .60806 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .0515 m
.06244 .21262 L
.10458 .34794 L
.12507 .39968 L
.14415 .44018 L
.16371 .47437 L
.18221 .50025 L
.192 .51153 L
.20262 .52197 L
.2212 .53586 L
.23064 .54089 L
.24104 .54492 L
.24627 .54637 L
.24915 .54701 L
.2519 .54751 L
.25444 .54788 L
.25578 .54804 L
.25721 .54819 L
.25853 .5483 L
.25973 .54838 L
.26093 .54844 L
.26207 .54848 L
.26323 .54851 L
.26447 .54852 L
.26565 .54851 L
.26672 .54848 L
.26798 .54843 L
.26869 .5484 L
.26935 .54836 L
.27183 .54816 L
.27421 .5479 L
.27683 .54754 L
.28215 .54654 L
.28673 .54541 L
.29167 .54392 L
.30062 .54052 L
.32066 .52981 L
.34176 .51431 L
.37987 .47696 L
.42043 .42685 L
.45947 .37184 L
.50097 .30976 L
.54095 .25 L
.57942 .19582 L
.62034 .14527 L
.64069 .12397 L
.65974 .10691 L
.67918 .09275 L
.69763 .08271 L
.70733 .07888 L
Mistroke
.71246 .07728 L
.71792 .07591 L
.72052 .07538 L
.72296 .07495 L
.72535 .07461 L
.72753 .07436 L
.72962 .07417 L
.73067 .0741 L
.73182 .07403 L
.73298 .07399 L
.73422 .07395 L
.73539 .07394 L
.73645 .07394 L
.73764 .07397 L
.73875 .074 L
.73996 .07406 L
.74127 .07414 L
.74395 .07439 L
.74639 .07469 L
.75087 .07546 L
.75558 .07656 L
.76072 .07811 L
.7655 .07989 L
.77635 .08517 L
.7869 .09199 L
.79685 .10002 L
.81545 .11936 L
.8355 .14686 L
.85397 .17867 L
.89342 .26908 L
.93136 .38739 L
.97175 .55054 L
.97619 .57095 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3oooooohoQ00<0hN40S^4008oQ00<0hN40S^4008oQ
00<0hN40S^4008oQ00<0hN40S^4008oQ0P2?h@00Sn400`3Qh@2>h@00Sn400`3Qh@2>h@00Sn400`3Q
h@2>h@00Sn400`3Qh@2>h@00Sn400`3Qh@2>h@00Sn4208oQ002?h@030>7Q08kQ002?h@030>7Q08kQ
002?h@030>7Q08kQ0007h@030>7Q08GQ00<0hN40S^4000OQ00<0hN40QN400`3Qh@2>h@001n400`3Q
h@25h@030>7Q08kQ0008h@030>7Q08CQ0P2?h@002>400`3Qh@24h@030>7Q08kQ0008h@030>7Q08CQ
00<0hN40S^4000SQ00<0hN40Q>400`3Qh@2>h@002N400`3Qh@1eh@D00^400`3Qh@04h@030>7Q03_Q
3017h@002N400`3Qh@1gh@070>7Qh@3Q0005h@030>7Q03SQ0`0<h@<0A>4000WQ00<0hN40LN4300?Q
00L0hN7Q0>4000GQ0`0fh@804^4204;Q0009h@030>7Q07OQ00L0hN7Q0>4000GQ00<0hN40=>4201KQ
00<0hN40?n4000[Q00<0hN40M^401`3QhN40h@001N400`3Qh@0bh@806N4203oQ000:h@030>7Q07GQ
0P04h@030>7Q00CQ00<0hN40<>4201gQ00<0hN40?>4000[Q00<0hN40P^400`3Qh@0_h@030>7Q01kQ
0P0lh@002^400`3Qh@22h@030>7Q02gQ0P0Sh@030>7Q03WQ000;h@030>7Q087Q00<0hN40;>400`3Q
h@0Th@030>7Q03SQ000;h@030>7Q087Q0P0/h@030>7Q02KQ00<0hN40=n4000_Q00<0hN40PN400`3Q
h@0Zh@030>7Q02OQ00<0hN40=n4000_Q00<0hN40PN400`3Qh@0Yh@030>7Q02WQ00<0hN40=^4000cQ
00<0hN40P>400`3Qh@0Wh@80;N400`3Qh@0eh@003>400`3Qh@20h@030>7Q02KQ00<0hN40;^400`3Q
h@0dh@003>400`3Qh@20h@030>7Q02GQ00<0hN40<>400`3Qh@0ch@003>400`3Qh@20h@809N400`3Q
h@0ah@030>7Q03?Q000<h@030>7Q083Q00<0hN408n400`3Qh@0ch@030>7Q03;Q000=h@030>7Q07oQ
00<0hN408^400`3Qh@0eh@030>7Q037Q000=h@030>7Q07oQ00<0hN408N400`3Qh@0gh@030>7Q033Q
000=h@030>7Q07oQ00<0hN408>400`3Qh@0hh@030>7Q033Q000=h@030>7Q07oQ00<0hN407n400`3Q
h@0jh@030>7Q02oQ000>h@030>7Q07kQ00<0hN407^400`3Qh@0lh@030>7Q02kQ000>h@030>7Q07kQ
0P0Oh@030>7Q03gQ00<0hN40;N4000kQ00<0hN40O^400`3Qh@0Mh@030>7Q03kQ00<0hN40;N4000kQ
00<0hN40O^400`3Qh@0Lh@030>7Q043Q00<0hN40;>4000oQ00<0hN40ON400`3Qh@0Kh@030>7Q047Q
00<0hN40;>4000oQ00<0hN40ON400`3Qh@0Kh@030>7Q04;Q00<0hN40:n4000oQ00<0hN40ON400`3Q
h@0Jh@030>7Q04?Q00<0hN40:n4000oQ00<0hN40ON4201[Q00<0hN40AN400`3Qh@0Zh@004>400`3Q
h@1lh@030>7Q01SQ00<0hN40A^400`3Qh@0Zh@004>400`3Qh@1lh@030>7Q01SQ00<0hN40An400`3Q
h@0Yh@004>400`3Qh@1lh@030>7Q01OQ00<0hN40B>400`3Qh@0Yh@004>400`3Qh@1lh@030>7Q01KQ
00<0hN40B^400`3Qh@0Xh@004N400`3Qh@1dh@030>7Q00CQ00<0hN405N400`3Qh@1;h@030>7Q02SQ
000Ah@030>7Q07?Q00<0h@001N400`3Qh@0Eh@030>7Q04cQ00<0hN409n40017Q00<0hN40K^4300CQ
00<0hN400n4301CQ00<0hN40CN400`3Qh@0Wh@004N400`3Qh@1ch@801^400`3Qh@0Ch@030>7Q04oQ
00<0hN409^4001;Q00<0hN40L^400`3Qh@05h@030>7Q01?Q00<0hN40Cn400`3Qh@0Vh@004^400`3Q
h@1bh@<01N400`3Qh@0Bh@030>7Q057Q00<0hN409N4001;Q00<0hN40N^400`3Qh@0Ah@030>7Q05;Q
00<0hN409N4001;Q00<0hN40N^400`3Qh@0@h@030>7Q05CQ00<0hN409>4001?Q00<0hN40NN400`3Q
h@0@h@030>7Q05CQ00<0hN409>4001?Q00<0hN40NN42013Q00<0hN40E^400`3Qh@0Sh@004n400`3Q
h@1ih@030>7Q00kQ00<0hN40En400`3Qh@0Sh@005>400`3Qh@1hh@030>7Q00gQ00<0hN40F>400`3Q
h@0Sh@005>400`3Qh@1hh@030>7Q00gQ00<0hN40FN400`3Qh@0Rh@005>400`3Qh@1hh@030>7Q00cQ
00<0hN40F^400`3Qh@0Rh@005>400`3Qh@1hh@030>7Q00_Q00<0hN40G>400`3Qh@0Qh@005N400`3Q
h@1gh@802n400`3Qh@1Mh@030>7Q027Q000Eh@030>7Q07OQ00<0hN402^400`3Qh@1Nh@030>7Q023Q
000Eh@030>7Q07OQ00<0hN402N400`3Qh@1Oh@030>7Q023Q000Fh@030>7Q07KQ00<0hN402>400`3Q
h@1Qh@030>7Q01oQ000Fh@030>7Q07KQ00<0hN402>400`3Qh@1Qh@030>7Q01oQ000Fh@030>7Q07KQ
00<0hN401n400`3Qh@1Sh@030>7Q01kQ000Gh@030>7Q07GQ00<0hN401^400`3Qh@1Th@030>7Q01kQ
000Gh@030>7Q07GQ0P06h@030>7Q06KQ00<0hN407N4001OQ00@0hN7Q0P0Uh@<08n4502;Q00<0hN40
1N400`3Qh@0Ih@D08n4302;Q00@0h@007>4001SQ00<0hN400^400`3Qh@0Rh@030>7Q02GQ00<0hN40
8^400`3Qh@04h@030>7Q01cQ00<0hN408n400`3Qh@0Rh@050>7Qh@006n4001KQ0`03h@030>7Q01kQ
0`03h@030>7Q01kQ0`03h@030>7Q02;Q00<0hN400n400`3Qh@0Mh@030>7Q02CQ00<0hN408^400`3Q
000Lh@006>400`3Qh@02h@030>7Q02?Q00<0hN409>400`3Qh@0Rh@030>7Q00?Q00<0hN407N400`3Q
h@0Th@030>7Q02;Q00@0hN406n4001SQ00H0hN40h@0Th@030>4002GQ00<0hN408^400`3Qh@02h@03
0>7Q01kQ00<0hN408n400`3Q000Sh@8000?Q0>406^4001WQ00@0hN409^400`3Qh@0Sh@809>4200;Q
00<0hN407^4202KQ00<0hN408n4201cQ000Ih@030>7Q07?Q00D0hN7Q001^h@030>7Q01_Q000Ih@03
0>7Q07?Q00@0hN40Kn400`3Qh@0Kh@006^400`3Qh@1bh@030>40073Q00<0hN406n4001[Q00<0hN40
L^4207;Q00<0hN406^4001[Q00<0hN40L^4207;Q00<0hN406^4001_Q00<0hN40LN400`3Qh@1ah@03
0>7Q01[Q003o02000N4000?Q00<0hN401N400`3Qh@04h@030>7Q00GQ0P06h@030>7Q00GQ00<0hN40
1N400`3Qh@04h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00CQ00<0hN401N400`3Qh@05h@03
0>7Q00GQ00<0hN401N400`3Qh@04h@030>7Q00GQ00<0hN401>4200OQ00<0hN401N400`3Qh@04h@03
0>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00GQ00<0hN401>400`3Qh@05h@030>7Q00GQ00<0hN40
1N400`3Qh@04h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00GQ00<0hN401>400`3Qh@05h@03
0>7Q00;Q000Lh@030>7Q06kQ00<0h@00M>400`3Qh@0Ih@007>400`3Qh@1]h@040>7Q07GQ00<0hN40
6>4001cQ00<0hN40KN40103Qh@1eh@030>7Q01SQ000Lh@030>7Q06cQ00D0hN7Q001eh@030>7Q01SQ
000Mh@030>7Q06[Q00<0hN400^4207CQ00<0hN406>4001gQ00<0hN40J^400`3Qh@02h@030>7Q07CQ
00<0hN405n4001gQ00<0hN40JN400`3Qh@03h@030>7Q07CQ00<0hN405n4001kQ00<0hN40In400`3Q
h@04h@030>7Q07CQ00<0hN405n4001kQ00<0hN40In400`3Qh@04h@030>7Q07GQ00<0hN405^4001kQ
00<0hN40I^400`3Qh@05h@030>7Q07GQ00<0hN405^4001oQ00<0hN40I>400`3Qh@06h@030>7Q07GQ
00<0hN405^4001oQ00<0hN40I>400`3Qh@06h@80Mn400`3Qh@0Eh@008>400`3Qh@1Rh@030>7Q00OQ
00<0hN40M^400`3Qh@0Eh@008>400`3Qh@1Qh@030>7Q00SQ00<0hN40M^400`3Qh@0Eh@008>400`3Q
h@1Qh@030>7Q00SQ00<0hN40M^400`3Qh@0Eh@008N400`3Qh@1Oh@030>7Q00WQ00<0hN40Mn400`3Q
h@0Dh@008N400`3Qh@1Nh@030>7Q00[Q00<0hN40Mn400`3Qh@0Dh@008^400`3Qh@1Mh@030>7Q00[Q
0P1hh@030>7Q01CQ000Rh@030>7Q05cQ00<0hN402n400`3Qh@1hh@030>7Q01?Q000Rh@030>7Q05_Q
00<0hN403>400`3Qh@1hh@030>7Q01?Q000Sh@030>7Q05[Q00<0hN403>400`3Qh@1hh@030>7Q01?Q
000Sh@030>7Q05WQ00<0hN403N400`3Qh@1ih@030>7Q01;Q000Th@030>7Q05OQ00<0hN403^400`3Q
h@1ih@030>7Q01;Q000Th@030>7Q05KQ00<0hN403n400`3Qh@1ih@030>7Q01;Q000Th@030>7Q05KQ
00<0hN403n4207[Q00<0hN404^4002GQ00<0hN40E>400`3Qh@0@h@030>7Q07[Q00<0hN404N4002GQ
00<0hN40Dn400`3Qh@0Ah@030>7Q07[Q00<0hN404N4002KQ00<0hN40D^400`3Qh@0Ah@030>7Q07[Q
00<0hN404N4002KQ00<0hN40DN400`3Qh@0;h@030>7Q00CQ00<0hN40N^400`3Qh@0Ah@009^400`3Q
h@1@h@030>7Q00_Q00<0h@001N400`3Qh@1kh@030>7Q013Q000Wh@030>7Q04oQ00<0hN403N400`3Q
h@03h@<0Nn400`3Qh@0@h@009n400`3Qh@1>h@030>7Q00cQ0P06h@030>7Q07_Q00<0hN404>4002SQ
00<0hN40C>400`3Qh@0=h@030>7Q00GQ00<0hN40Nn400`3Qh@0@h@00:>400`3Qh@1;h@030>7Q00kQ
0`05h@030>7Q07cQ00<0hN403n4002WQ00<0hN40B^400`3Qh@0Fh@030>7Q07cQ00<0hN403n4002WQ
00<0hN40BN400`3Qh@0Gh@030>7Q07cQ00<0hN403n4002[Q00<0hN40An400`3Qh@0Hh@030>7Q07cQ
00<0hN403n4002[Q00<0hN40A^400`3Qh@0Ih@80O^400`3Qh@0>h@00:n400`3Qh@14h@030>7Q01[Q
00<0hN40ON400`3Qh@0>h@00;>400`3Qh@13h@030>7Q01[Q00<0hN40ON400`3Qh@0>h@00;>400`3Q
h@12h@030>7Q01_Q00<0hN40ON400`3Qh@0>h@00;N400`3Qh@10h@030>7Q01cQ00<0hN40O^400`3Q
h@0=h@00;^400`3Qh@0nh@030>7Q01gQ00<0hN40O^400`3Qh@0=h@00;^400`3Qh@0nh@030>7Q01gQ
0P1oh@030>7Q00gQ000_h@030>7Q03cQ00<0hN407^400`3Qh@1nh@030>7Q00gQ000`h@030>7Q03[Q
00<0hN407n400`3Qh@1oh@030>7Q00cQ000`h@030>7Q03WQ00<0hN408>400`3Qh@1oh@030>7Q00cQ
000ah@030>7Q03OQ00<0hN408N400`3Qh@1oh@030>7Q00cQ000ah@030>7Q03KQ00<0hN408^400`3Q
h@1oh@030>7Q00cQ000bh@030>7Q03CQ00<0hN408n400`3Qh@20h@030>7Q00_Q000ch@030>7Q03;Q
00<0hN409>42087Q00<0hN402n4003?Q00<0hN40<N400`3Qh@0Uh@030>7Q083Q00<0hN402n4003CQ
00<0hN40;n400`3Qh@0Vh@030>7Q083Q00<0hN402n4003GQ00<0hN40;N400`3Qh@0Wh@030>7Q087Q
00<0hN402^4003KQ00<0hN40:n400`3Qh@0Xh@030>7Q087Q00<0hN402^4003OQ00<0hN40:N400`3Q
h@0Yh@030>7Q087Q00<0hN402^4003SQ00<0hN409n400`3Qh@0Zh@80P^400`3Qh@0:h@00>N400`3Q
h@0Th@80;N400`3Qh@22h@030>7Q00WQ000jh@030>7Q02;Q00<0hN40;N400`3Qh@22h@030>7Q00WQ
000kh@030>7Q01oQ0P0`h@030>7Q08;Q00<0hN402N4003cQ0P0Nh@030>7Q033Q00<0hN40P^400`3Q
h@09h@00?^400`3Qh@0Ih@809N4500;Q00<0hN401>400`3Qh@23h@030>7Q00SQ000oh@805n4202WQ
00L0hN7Q0>4000GQ00<0hN40Pn400`3Qh@08h@00@N4201;Q0`0[h@070>7Qh@3Q0005h@<0Pn400`3Q
h@08h@00@n4200kQ0P0^h@070>7Qh@3Q0005h@030>7Q08?Q00<0hN402>4004GQ1@04h@D0<>401`3Q
hN40h@001N400`3Qh@24h@030>7Q00OQ0019h@D0=>4200CQ00<0hN401>400`3Qh@24h@030>7Q00OQ
002?h@030>7Q08CQ00<0hN401n4008oQ00<0hN40Q>400`3Qh@07h@00Sn400`3Qh@24h@030>7Q00OQ
002?h@80Q^400`3Qh@06h@00Sn400`3Qh@25h@030>7Q00KQ002?h@030>7Q08GQ00<0hN401^4008oQ
00<0hN40S^4008oQ00<0hN40S^4008oQ00<0hN40S^4008oQ0P2?h@00Sn400`3Qh@2>h@00Sn400`3Q
h@2>h@00Sn400`3Qh@2>h@00Sn400`3Qh@2>h@00Sn400`3Qh@2>h@00Sn400`3Qh@2>h@00Sn4208oQ
002?h@030>7Q08kQ002?h@030>7Q08kQ0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-3.68413, -13.6307, 
  0.0256734, 0.152978}},
  CellTags->{"Newton", "AnimateNewton"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.136054 0.311229 0.0228331 [
[.09184 .29873 -6 -9 ]
[.09184 .29873 6 0 ]
[.22789 .29873 -6 -9 ]
[.22789 .29873 6 0 ]
[.36395 .29873 -6 -9 ]
[.36395 .29873 6 0 ]
[.63605 .29873 -3 -9 ]
[.63605 .29873 3 0 ]
[.77211 .29873 -3 -9 ]
[.77211 .29873 3 0 ]
[.90816 .29873 -3 -9 ]
[.90816 .29873 3 0 ]
[.4875 .0829 -18 -4.5 ]
[.4875 .0829 0 4.5 ]
[.4875 .19706 -12 -4.5 ]
[.4875 .19706 0 4.5 ]
[.4875 .42539 -6 -4.5 ]
[.4875 .42539 0 4.5 ]
[.4875 .53956 -12 -4.5 ]
[.4875 .53956 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.09184 .31123 m
.09184 .31748 L
s
[(-3)] .09184 .29873 0 1 Mshowa
.22789 .31123 m
.22789 .31748 L
s
[(-2)] .22789 .29873 0 1 Mshowa
.36395 .31123 m
.36395 .31748 L
s
[(-1)] .36395 .29873 0 1 Mshowa
.63605 .31123 m
.63605 .31748 L
s
[(1)] .63605 .29873 0 1 Mshowa
.77211 .31123 m
.77211 .31748 L
s
[(2)] .77211 .29873 0 1 Mshowa
.90816 .31123 m
.90816 .31748 L
s
[(3)] .90816 .29873 0 1 Mshowa
.125 Mabswid
.11905 .31123 m
.11905 .31498 L
s
.14626 .31123 m
.14626 .31498 L
s
.17347 .31123 m
.17347 .31498 L
s
.20068 .31123 m
.20068 .31498 L
s
.2551 .31123 m
.2551 .31498 L
s
.28231 .31123 m
.28231 .31498 L
s
.30952 .31123 m
.30952 .31498 L
s
.33673 .31123 m
.33673 .31498 L
s
.39116 .31123 m
.39116 .31498 L
s
.41837 .31123 m
.41837 .31498 L
s
.44558 .31123 m
.44558 .31498 L
s
.47279 .31123 m
.47279 .31498 L
s
.52721 .31123 m
.52721 .31498 L
s
.55442 .31123 m
.55442 .31498 L
s
.58163 .31123 m
.58163 .31498 L
s
.60884 .31123 m
.60884 .31498 L
s
.66327 .31123 m
.66327 .31498 L
s
.69048 .31123 m
.69048 .31498 L
s
.71769 .31123 m
.71769 .31498 L
s
.7449 .31123 m
.7449 .31498 L
s
.79932 .31123 m
.79932 .31498 L
s
.82653 .31123 m
.82653 .31498 L
s
.85374 .31123 m
.85374 .31498 L
s
.88095 .31123 m
.88095 .31498 L
s
.06463 .31123 m
.06463 .31498 L
s
.03741 .31123 m
.03741 .31498 L
s
.0102 .31123 m
.0102 .31498 L
s
.93537 .31123 m
.93537 .31498 L
s
.96259 .31123 m
.96259 .31498 L
s
.9898 .31123 m
.9898 .31498 L
s
.25 Mabswid
0 .31123 m
1 .31123 L
s
.5 .0829 m
.50625 .0829 L
s
[(-10)] .4875 .0829 1 0 Mshowa
.5 .19706 m
.50625 .19706 L
s
[(-5)] .4875 .19706 1 0 Mshowa
.5 .42539 m
.50625 .42539 L
s
[(5)] .4875 .42539 1 0 Mshowa
.5 .53956 m
.50625 .53956 L
s
[(10)] .4875 .53956 1 0 Mshowa
.125 Mabswid
.5 .10573 m
.50375 .10573 L
s
.5 .12856 m
.50375 .12856 L
s
.5 .1514 m
.50375 .1514 L
s
.5 .17423 m
.50375 .17423 L
s
.5 .2199 m
.50375 .2199 L
s
.5 .24273 m
.50375 .24273 L
s
.5 .26556 m
.50375 .26556 L
s
.5 .2884 m
.50375 .2884 L
s
.5 .33406 m
.50375 .33406 L
s
.5 .35689 m
.50375 .35689 L
s
.5 .37973 m
.50375 .37973 L
s
.5 .40256 m
.50375 .40256 L
s
.5 .44823 m
.50375 .44823 L
s
.5 .47106 m
.50375 .47106 L
s
.5 .49389 m
.50375 .49389 L
s
.5 .51673 m
.50375 .51673 L
s
.5 .06007 m
.50375 .06007 L
s
.5 .03723 m
.50375 .03723 L
s
.5 .0144 m
.50375 .0144 L
s
.5 .56239 m
.50375 .56239 L
s
.5 .58523 m
.50375 .58523 L
s
.5 .60806 m
.50375 .60806 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .0515 m
.06244 .21262 L
.10458 .34794 L
.12507 .39968 L
.14415 .44018 L
.16371 .47437 L
.18221 .50025 L
.192 .51153 L
.20262 .52197 L
.2212 .53586 L
.23064 .54089 L
.24104 .54492 L
.24627 .54637 L
.24915 .54701 L
.2519 .54751 L
.25444 .54788 L
.25578 .54804 L
.25721 .54819 L
.25853 .5483 L
.25973 .54838 L
.26093 .54844 L
.26207 .54848 L
.26323 .54851 L
.26447 .54852 L
.26565 .54851 L
.26672 .54848 L
.26798 .54843 L
.26869 .5484 L
.26935 .54836 L
.27183 .54816 L
.27421 .5479 L
.27683 .54754 L
.28215 .54654 L
.28673 .54541 L
.29167 .54392 L
.30062 .54052 L
.32066 .52981 L
.34176 .51431 L
.37987 .47696 L
.42043 .42685 L
.45947 .37184 L
.50097 .30976 L
.54095 .25 L
.57942 .19582 L
.62034 .14527 L
.64069 .12397 L
.65974 .10691 L
.67918 .09275 L
.69763 .08271 L
.70733 .07888 L
Mistroke
.71246 .07728 L
.71792 .07591 L
.72052 .07538 L
.72296 .07495 L
.72535 .07461 L
.72753 .07436 L
.72962 .07417 L
.73067 .0741 L
.73182 .07403 L
.73298 .07399 L
.73422 .07395 L
.73539 .07394 L
.73645 .07394 L
.73764 .07397 L
.73875 .074 L
.73996 .07406 L
.74127 .07414 L
.74395 .07439 L
.74639 .07469 L
.75087 .07546 L
.75558 .07656 L
.76072 .07811 L
.7655 .07989 L
.77635 .08517 L
.7869 .09199 L
.79685 .10002 L
.81545 .11936 L
.8355 .14686 L
.85397 .17867 L
.89342 .26908 L
.93136 .38739 L
.97175 .55054 L
.97619 .57095 L
Mfstroke
.02381 .52497 m
.06244 .49949 L
.10458 .4717 L
.14415 .4456 L
.18221 .4205 L
.22272 .39378 L
.26171 .36806 L
.30316 .34073 L
.34309 .31439 L
.3815 .28906 L
.42237 .2621 L
.46172 .23615 L
.49955 .2112 L
.53984 .18462 L
.57861 .15905 L
.61984 .13186 L
.65954 .10567 L
.69774 .08048 L
.73838 .05368 L
.77751 .02787 L
.81909 .00044 L
s
.81909 .00044 m
.81977 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3oooooohoQ00<0hN40F>4203CQ002?h@030>7Q05KQ
0P0fh@00Sn400`3Qh@1Eh@030>7Q03KQ002?h@030>7Q05?Q0P0ih@00Sn4205?Q00<0hN40>N4008oQ
00<0hN40D>4203cQ002?h@030>7Q04oQ00<0hN40?>4008oQ00<0hN40CN4203oQ002?h@030>7Q04cQ
00<0hN40?n4008oQ00<0hN40B^4204;Q002?h@80B^400`3Qh@12h@00Sn400`3Qh@17h@80AN4008oQ
00<0hN40AN4204OQ002?h@030>7Q04CQ00<0hN40An4000OQ00<0hN40QN400`3Qh@12h@80B^4000OQ
00<0hN40QN400`3Qh@11h@030>7Q04[Q0007h@030>7Q08GQ00<0hN40?n4204gQ0008h@030>7Q08CQ
0P0oh@030>7Q04gQ0008h@030>7Q08CQ00<0hN40?N400`3Qh@1>h@002>400`3Qh@24h@030>7Q03_Q
0P1Ah@002>400`3Qh@24h@030>7Q03[Q00<0hN40DN4000WQ00<0hN40MN4500;Q00<0hN401>400`3Q
h@0ih@030>4000/0An4000WQ00<0hN40Mn401`3QhN40h@001N400`3Qh@0gh@@03>4304CQ0009h@03
0>7Q077Q0`03h@070>7Qh@3Q0005h@<0=^4201;Q0P12h@002N400`3Qh@1gh@070>7Qh@3Q0005h@03
0>7Q03CQ0P0Fh@030>7Q03oQ000:h@030>7Q07KQ00L0hN7Q0>4000GQ00<0hN40<^4201WQ0P0oh@00
2^400`3Qh@1eh@801>400`3Qh@04h@030>7Q033Q0`0Lh@030>7Q03cQ000:h@030>7Q08;Q00<0hN40
;n4201oQ0P0lh@002^400`3Qh@22h@030>7Q02gQ0P0Sh@030>7Q03WQ000;h@030>7Q087Q00<0hN40
;>4202GQ00<0hN40>>4000_Q00<0hN40PN4202cQ00<0hN409^400`3Qh@0gh@002n400`3Qh@21h@03
0>7Q02WQ0P0Yh@030>7Q03OQ000;h@030>7Q087Q00<0hN409n4302_Q00<0hN40=^4000cQ00<0hN40
P>400`3Qh@0Vh@<0;N400`3Qh@0eh@003>400`3Qh@20h@030>7Q02CQ0`0`h@030>7Q03CQ000<h@03
0>7Q083Q00<0hN408^420003h@3Q037Q00<0hN40<n4000cQ00<0hN40P>42027Q0P02h@030>7Q037Q
00<0hN40<n4000cQ00<0hN40P>400`3Qh@0Oh@050>7Qh@00=N400`3Qh@0bh@003N400`3Qh@1oh@03
0>7Q01gQ0P03h@030>7Q03GQ00<0hN40<N4000gQ00<0hN40On400`3Qh@0Lh@030>7Q00;Q00<0hN40
=n400`3Qh@0`h@003N400`3Qh@1oh@030>7Q01_Q00<0hN400^400`3Qh@0hh@030>7Q033Q000=h@03
0>7Q07oQ00<0hN406N4200CQ00<0hN40>^400`3Qh@0_h@003^400`3Qh@1nh@030>7Q01SQ00<0hN40
0n400`3Qh@0lh@030>7Q02kQ000>h@030>7Q07kQ0P0Hh@030>7Q00CQ00<0hN40?N400`3Qh@0]h@00
3^400`3Qh@1nh@030>7Q01GQ0P06h@030>7Q03kQ00<0hN40;N4000kQ00<0hN40O^400`3Qh@0Dh@03
0>7Q00GQ00<0hN40@>400`3Qh@0/h@003n400`3Qh@1mh@030>7Q01;Q0P07h@030>7Q047Q00<0hN40
;>4000oQ00<0hN40ON400`3Qh@0@h@802N400`3Qh@12h@030>7Q02_Q000?h@030>7Q07gQ00<0hN40
3n400`3Qh@08h@030>7Q04?Q00<0hN40:n4000oQ00<0hN40ON4200kQ0P0:h@030>7Q04GQ00<0hN40
:^40013Q00<0hN40O>400`3Qh@0;h@802n400`3Qh@16h@030>7Q02[Q000@h@030>7Q07cQ00<0hN40
2N4200gQ00<0hN40An400`3Qh@0Yh@004>400`3Qh@1lh@030>7Q00SQ00<0hN403>400`3Qh@18h@03
0>7Q02WQ000@h@030>7Q07cQ00<0hN401^4200kQ00<0hN40B^400`3Qh@0Xh@004N400`3Qh@1dh@03
0>7Q00CQ00<0hN401N400`3Qh@0=h@030>7Q04_Q00<0hN40:>40017Q00<0hN40Ln400`3Q0005h@03
0>7Q00CQ00<0hN403^400`3Qh@1<h@030>7Q02OQ000Ah@030>7Q06kQ0`04h@030>7Q00?Q0`02h@80
4>400`3Qh@1=h@030>7Q02OQ000Ah@030>7Q07?Q0P06h@050>7Qh@004N400`3Qh@1?h@030>7Q02KQ
000Bh@030>7Q07;Q00<0hN401N40103Qh@0Bh@030>7Q04oQ00<0hN409^4001;Q00<0hN40L^4300GQ
0`0Bh@030>7Q057Q00<0hN409N4001;Q00<0hN40N^400`3Qh@0Ah@030>7Q05;Q00<0hN409N4001;Q
00<0hN40N>4301;Q00<0hN40E>400`3Qh@0Th@004n400`3Qh@1fh@040>7Q01;Q00<0hN40E>400`3Q
h@0Th@004n400`3Qh@1dh@800n42013Q00<0hN40E^400`3Qh@0Sh@004n400`3Qh@1bh@801N400`3Q
h@0>h@030>7Q05OQ00<0hN408n4001CQ00<0hN40L>400`3Qh@05h@030>7Q00gQ00<0hN40F>400`3Q
h@0Sh@005>400`3Qh@1^h@802>400`3Qh@0=h@030>7Q05WQ00<0hN408^4001CQ00<0hN40KN400`3Q
h@08h@030>7Q00cQ00<0hN40F^400`3Qh@0Rh@005>400`3Qh@1[h@802n400`3Qh@0;h@030>7Q05cQ
00<0hN408N4001GQ00<0hN40JN400`3Qh@0;h@802n400`3Qh@1Mh@030>7Q027Q000Eh@030>7Q06SQ
00<0hN403>400`3Qh@0:h@030>7Q05kQ00<0hN408>4001GQ00<0hN40I^4200oQ00<0hN402N400`3Q
h@1Oh@030>7Q023Q000Fh@030>7Q06CQ00<0hN403n400`3Qh@08h@030>7Q067Q00<0hN407n4001KQ
00<0hN40Hn400`3Qh@0@h@030>7Q00SQ00<0hN40HN400`3Qh@0Oh@005^400`3Qh@1Qh@804n400`3Q
h@07h@030>7Q06?Q00<0hN407^4001OQ00<0hN40Gn400`3Qh@0Ch@030>7Q00KQ00<0hN40I>400`3Q
h@0Nh@005n400`3Qh@1Mh@805^4200KQ00<0hN40I^400`3Qh@0Mh@005n40103QhN4202GQ0`0Sh@D0
2>4201SQ00<0hN401N400`3Qh@0Ih@D08n4302;Q00@0h@007>4001SQ00<0hN400^400`3Qh@0Rh@03
0>7Q02GQ00<0hN401n400`3Qh@0Hh@030>7Q00CQ00<0hN407>400`3Qh@0Sh@030>7Q02;Q00D0hN7Q
000Kh@005^4300?Q00<0hN407^4300?Q00<0hN407^4300?Q00<0hN401N4201_Q00<0hN400n400`3Q
h@0Mh@030>7Q02CQ00<0hN408^400`3Q000Lh@006>400`3Qh@02h@030>7Q02?Q00<0hN409>400`3Q
h@03h@807N400`3Qh@03h@030>7Q01gQ00<0hN409>400`3Qh@0Rh@040>7Q01_Q000Hh@060>7Q0>40
9>400`3Q000Uh@040>7Qh@807n400`3Qh@02h@030>7Q01kQ00<0hN408n400`3Q000Sh@8000?Q0>40
6^4001WQ00@0hN409^400`3Qh@0Sh@800^400`3Qh@0Oh@800^400`3Qh@0Nh@809^400`3Qh@0Sh@80
7>4001WQ00<0hN40Cn4202;Q00D0hN7Q001^h@030>7Q01_Q000Ih@030>7Q04kQ00<0hN408^40103Q
h@1_h@030>7Q01_Q000Jh@030>7Q04cQ00<0hN408n400`3Q001`h@030>7Q01_Q000Jh@030>7Q04[Q
0P0Vh@80L^400`3Qh@0Jh@006^400`3Qh@19h@030>7Q02KQ0P1bh@030>7Q01[Q000Kh@030>7Q04OQ
00<0hN409n400`3Qh@1ah@030>7Q01[Q003o02000N4000?Q00<0hN401N400`3Qh@04h@030>7Q00GQ
0P06h@030>7Q00GQ00<0hN401N400`3Qh@04h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00CQ
00<0hN401N400`3Q0005h@030>7Q00GQ00<0hN401N400`3Qh@04h@030>7Q00GQ00<0hN401>4200OQ
00<0hN401N400`3Qh@04h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00GQ00<0hN401>400`3Q
h@05h@030>7Q00GQ00<0hN401N400`3Qh@04h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00GQ
00<0hN401>400`3Qh@05h@030>7Q00;Q000Lh@030>7Q047Q0P0[h@030>4007CQ00<0hN406N4001cQ
00<0hN40@>400`3Qh@0Zh@040>7Q07GQ00<0hN406>4001cQ00<0hN40?^4202gQ00@0hN40MN400`3Q
h@0Hh@007>400`3Qh@0lh@80;^401@3QhN4007GQ00<0hN406>4001gQ00<0hN40>^400`3Qh@0]h@03
0>7Q00;Q0P1dh@030>7Q01SQ000Mh@030>7Q03SQ0P0`h@030>7Q00;Q00<0hN40M>400`3Qh@0Gh@00
7N400`3Qh@0gh@030>7Q02oQ00<0hN400n400`3Qh@1dh@030>7Q01OQ000Nh@030>7Q03CQ0P0ah@03
0>7Q00CQ00<0hN40M>400`3Qh@0Gh@007^400`3Qh@0ch@030>7Q037Q00<0hN401>400`3Qh@1eh@03
0>7Q01KQ000Nh@030>7Q037Q0P0ch@030>7Q00GQ00<0hN40MN400`3Qh@0Fh@007n400`3Qh@0_h@03
0>7Q03;Q00<0hN401^400`3Qh@1eh@030>7Q01KQ000Oh@030>7Q02gQ0P0eh@030>7Q00KQ0P1gh@03
0>7Q01GQ000Ph@030>7Q02_Q00<0hN40=>400`3Qh@07h@030>7Q07KQ00<0hN405N40023Q00<0hN40
:N4203KQ00<0hN402>400`3Qh@1fh@030>7Q01GQ000Ph@030>7Q02SQ00<0hN40=^400`3Qh@08h@03
0>7Q07KQ00<0hN405N40027Q00<0hN409N4203SQ00<0hN402N400`3Qh@1gh@030>7Q01CQ000Qh@03
0>7Q02CQ00<0hN40=n400`3Qh@0:h@030>7Q07OQ00<0hN405>4002;Q00<0hN408N4203[Q00<0hN40
2^4207SQ00<0hN405>4002;Q00<0hN407n4203_Q00<0hN402n400`3Qh@1hh@030>7Q01?Q000Rh@03
0>7Q01kQ00<0hN40>^400`3Qh@0<h@030>7Q07SQ00<0hN404n4002?Q00<0hN406n4203gQ00<0hN40
3>400`3Qh@1hh@030>7Q01?Q000Sh@030>7Q01[Q00<0hN40?>400`3Qh@0=h@030>7Q07WQ00<0hN40
4^4002CQ00<0hN405n4203kQ00<0hN403^400`3Qh@1ih@030>7Q01;Q000Th@030>7Q01KQ00<0hN40
?N400`3Qh@0?h@030>7Q07WQ00<0hN404^4002CQ00<0hN405>42043Q00<0hN403n4207[Q00<0hN40
4^4002GQ00<0hN404^400`3Qh@0oh@030>7Q013Q00<0hN40N^400`3Qh@0Ah@009N400`3Qh@0@h@80
@N400`3Qh@0Ah@030>7Q07[Q00<0hN404N4002KQ00<0hN403^400`3Qh@11h@030>7Q017Q00<0hN40
N^400`3Qh@0Ah@009^400`3Qh@0<h@80@n400`3Qh@0;h@030>7Q00CQ00<0hN40N^400`3Qh@0Ah@00
9^400`3Qh@0;h@030>7Q04;Q00<0hN402n400`3Q0005h@030>7Q07_Q00<0hN404>4002OQ00<0hN40
2>4204GQ00<0hN403N400`3Qh@03h@<0Nn400`3Qh@0@h@009n400`3Qh@07h@030>7Q04CQ00<0hN40
3>4200KQ00<0hN40Nn400`3Qh@0@h@00:>400`3Qh@04h@80A^400`3Qh@0=h@030>7Q00GQ00<0hN40
Nn400`3Qh@0@h@00:>400`3Qh@02h@80An400`3Qh@0>h@<01N400`3Qh@1lh@030>7Q00oQ000Yh@04
0>7Q04WQ00<0hN405^400`3Qh@1lh@030>7Q00oQ000Yh@<0BN400`3Qh@0Gh@030>7Q07cQ00<0hN40
3n4002WQ0P19h@030>7Q01SQ00<0hN40O>400`3Qh@0?h@009n420003h@3Q04OQ00<0hN406N4207kQ
00<0hN403^4002KQ00<0hN400^400`3Qh@14h@030>7Q01[Q00<0hN40ON400`3Qh@0>h@009N400`3Q
h@04h@030>7Q04?Q00<0hN406^400`3Qh@1mh@030>7Q00kQ000Sh@801n400`3Qh@12h@030>7Q01_Q
00<0hN40ON400`3Qh@0>h@008^400`3Qh@08h@030>7Q043Q00<0hN407>400`3Qh@1nh@030>7Q00gQ
000Qh@030>7Q00[Q00<0hN40?^400`3Qh@0Mh@030>7Q07kQ00<0hN403N4001oQ0P0=h@030>7Q03kQ
00<0hN407N4207oQ00<0hN403N4001kQ00<0hN403^400`3Qh@0lh@030>7Q01kQ00<0hN40O^400`3Q
h@0=h@007>4201;Q00<0hN40>^400`3Qh@0Oh@030>7Q07oQ00<0hN403>4001_Q00<0hN404^400`3Q
h@0ih@030>7Q023Q00<0hN40On400`3Qh@0<h@006N4201KQ00<0hN40=n400`3Qh@0Qh@030>7Q07oQ
00<0hN403>4001SQ00<0hN405^400`3Qh@0fh@030>7Q02;Q00<0hN40On400`3Qh@0<h@005^4201[Q
00<0hN40=>400`3Qh@0Sh@030>7Q083Q00<0hN402n4001GQ00<0hN406n400`3Qh@0bh@030>7Q02CQ
0P21h@030>7Q00_Q000Ch@807^400`3Qh@0ah@030>7Q02GQ00<0hN40P>400`3Qh@0;h@004^400`3Q
h@0Oh@030>7Q02oQ00<0hN409^400`3Qh@20h@030>7Q00_Q000@h@808n400`3Qh@0]h@030>7Q02OQ
00<0hN40PN400`3Qh@0:h@003n400`3Qh@0Th@030>7Q02_Q00<0hN40:>400`3Qh@21h@030>7Q00[Q
000=h@80:>400`3Qh@0Yh@030>7Q02WQ00<0hN40PN400`3Qh@0:h@002n4202_Q00<0hN409n400`3Q
h@0Zh@80P^400`3Qh@0:h@002^400`3Qh@0/h@030>7Q02CQ0P0]h@030>7Q08;Q00<0hN402N4000SQ
0P0`h@030>7Q02;Q00<0hN40;N400`3Qh@22h@030>7Q00WQ0007h@030>7Q037Q00<0hN407n42033Q
00<0hN40P^400`3Qh@09h@00?>4201kQ00<0hN40<>400`3Qh@22h@030>7Q00WQ000nh@030>7Q01WQ
0P0Uh@D00^400`3Qh@04h@030>7Q08?Q00<0hN402>4003oQ0P0Gh@80:N401`3QhN40h@001N400`3Q
h@23h@030>7Q00SQ0011h@804^4302_Q00L0hN7Q0>4000GQ0`23h@030>7Q00SQ0013h@803^4202kQ
00L0hN7Q0>4000GQ00<0hN40Pn400`3Qh@08h@00AN4500CQ1@0`h@070>7Qh@3Q0005h@030>7Q08CQ
00<0hN401n4004WQ1@0dh@801>400`3Qh@04h@030>7Q08CQ00<0hN401n4008oQ00<0hN40Q>400`3Q
h@07h@00Sn400`3Qh@24h@030>7Q00OQ002?h@030>7Q08CQ00<0hN401n4008oQ0P26h@030>7Q00KQ
002?h@030>7Q08GQ00<0hN401^4008oQ00<0hN40QN400`3Qh@06h@00Sn400`3Qh@2>h@00Sn400`3Q
h@2>h@00Sn400`3Qh@2>h@00Sn4208oQ002?h@030>7Q08kQ002?h@030>7Q08kQ002?h@030>7Q08kQ
002?h@030>7Q08kQ002?h@030>7Q08kQ002?h@030>7Q08kQ002?h@80Sn4008oQ00<0hN40S^4008oQ
00<0hN40S^400001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-3.68413, -13.6307, 
  0.0256734, 0.152978}},
  CellTags->{"Newton", "AnimateNewton"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.136054 0.311229 0.0228331 [
[.09184 .29873 -6 -9 ]
[.09184 .29873 6 0 ]
[.22789 .29873 -6 -9 ]
[.22789 .29873 6 0 ]
[.36395 .29873 -6 -9 ]
[.36395 .29873 6 0 ]
[.63605 .29873 -3 -9 ]
[.63605 .29873 3 0 ]
[.77211 .29873 -3 -9 ]
[.77211 .29873 3 0 ]
[.90816 .29873 -3 -9 ]
[.90816 .29873 3 0 ]
[.4875 .0829 -18 -4.5 ]
[.4875 .0829 0 4.5 ]
[.4875 .19706 -12 -4.5 ]
[.4875 .19706 0 4.5 ]
[.4875 .42539 -6 -4.5 ]
[.4875 .42539 0 4.5 ]
[.4875 .53956 -12 -4.5 ]
[.4875 .53956 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.09184 .31123 m
.09184 .31748 L
s
[(-3)] .09184 .29873 0 1 Mshowa
.22789 .31123 m
.22789 .31748 L
s
[(-2)] .22789 .29873 0 1 Mshowa
.36395 .31123 m
.36395 .31748 L
s
[(-1)] .36395 .29873 0 1 Mshowa
.63605 .31123 m
.63605 .31748 L
s
[(1)] .63605 .29873 0 1 Mshowa
.77211 .31123 m
.77211 .31748 L
s
[(2)] .77211 .29873 0 1 Mshowa
.90816 .31123 m
.90816 .31748 L
s
[(3)] .90816 .29873 0 1 Mshowa
.125 Mabswid
.11905 .31123 m
.11905 .31498 L
s
.14626 .31123 m
.14626 .31498 L
s
.17347 .31123 m
.17347 .31498 L
s
.20068 .31123 m
.20068 .31498 L
s
.2551 .31123 m
.2551 .31498 L
s
.28231 .31123 m
.28231 .31498 L
s
.30952 .31123 m
.30952 .31498 L
s
.33673 .31123 m
.33673 .31498 L
s
.39116 .31123 m
.39116 .31498 L
s
.41837 .31123 m
.41837 .31498 L
s
.44558 .31123 m
.44558 .31498 L
s
.47279 .31123 m
.47279 .31498 L
s
.52721 .31123 m
.52721 .31498 L
s
.55442 .31123 m
.55442 .31498 L
s
.58163 .31123 m
.58163 .31498 L
s
.60884 .31123 m
.60884 .31498 L
s
.66327 .31123 m
.66327 .31498 L
s
.69048 .31123 m
.69048 .31498 L
s
.71769 .31123 m
.71769 .31498 L
s
.7449 .31123 m
.7449 .31498 L
s
.79932 .31123 m
.79932 .31498 L
s
.82653 .31123 m
.82653 .31498 L
s
.85374 .31123 m
.85374 .31498 L
s
.88095 .31123 m
.88095 .31498 L
s
.06463 .31123 m
.06463 .31498 L
s
.03741 .31123 m
.03741 .31498 L
s
.0102 .31123 m
.0102 .31498 L
s
.93537 .31123 m
.93537 .31498 L
s
.96259 .31123 m
.96259 .31498 L
s
.9898 .31123 m
.9898 .31498 L
s
.25 Mabswid
0 .31123 m
1 .31123 L
s
.5 .0829 m
.50625 .0829 L
s
[(-10)] .4875 .0829 1 0 Mshowa
.5 .19706 m
.50625 .19706 L
s
[(-5)] .4875 .19706 1 0 Mshowa
.5 .42539 m
.50625 .42539 L
s
[(5)] .4875 .42539 1 0 Mshowa
.5 .53956 m
.50625 .53956 L
s
[(10)] .4875 .53956 1 0 Mshowa
.125 Mabswid
.5 .10573 m
.50375 .10573 L
s
.5 .12856 m
.50375 .12856 L
s
.5 .1514 m
.50375 .1514 L
s
.5 .17423 m
.50375 .17423 L
s
.5 .2199 m
.50375 .2199 L
s
.5 .24273 m
.50375 .24273 L
s
.5 .26556 m
.50375 .26556 L
s
.5 .2884 m
.50375 .2884 L
s
.5 .33406 m
.50375 .33406 L
s
.5 .35689 m
.50375 .35689 L
s
.5 .37973 m
.50375 .37973 L
s
.5 .40256 m
.50375 .40256 L
s
.5 .44823 m
.50375 .44823 L
s
.5 .47106 m
.50375 .47106 L
s
.5 .49389 m
.50375 .49389 L
s
.5 .51673 m
.50375 .51673 L
s
.5 .06007 m
.50375 .06007 L
s
.5 .03723 m
.50375 .03723 L
s
.5 .0144 m
.50375 .0144 L
s
.5 .56239 m
.50375 .56239 L
s
.5 .58523 m
.50375 .58523 L
s
.5 .60806 m
.50375 .60806 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .0515 m
.06244 .21262 L
.10458 .34794 L
.12507 .39968 L
.14415 .44018 L
.16371 .47437 L
.18221 .50025 L
.192 .51153 L
.20262 .52197 L
.2212 .53586 L
.23064 .54089 L
.24104 .54492 L
.24627 .54637 L
.24915 .54701 L
.2519 .54751 L
.25444 .54788 L
.25578 .54804 L
.25721 .54819 L
.25853 .5483 L
.25973 .54838 L
.26093 .54844 L
.26207 .54848 L
.26323 .54851 L
.26447 .54852 L
.26565 .54851 L
.26672 .54848 L
.26798 .54843 L
.26869 .5484 L
.26935 .54836 L
.27183 .54816 L
.27421 .5479 L
.27683 .54754 L
.28215 .54654 L
.28673 .54541 L
.29167 .54392 L
.30062 .54052 L
.32066 .52981 L
.34176 .51431 L
.37987 .47696 L
.42043 .42685 L
.45947 .37184 L
.50097 .30976 L
.54095 .25 L
.57942 .19582 L
.62034 .14527 L
.64069 .12397 L
.65974 .10691 L
.67918 .09275 L
.69763 .08271 L
.70733 .07888 L
Mistroke
.71246 .07728 L
.71792 .07591 L
.72052 .07538 L
.72296 .07495 L
.72535 .07461 L
.72753 .07436 L
.72962 .07417 L
.73067 .0741 L
.73182 .07403 L
.73298 .07399 L
.73422 .07395 L
.73539 .07394 L
.73645 .07394 L
.73764 .07397 L
.73875 .074 L
.73996 .07406 L
.74127 .07414 L
.74395 .07439 L
.74639 .07469 L
.75087 .07546 L
.75558 .07656 L
.76072 .07811 L
.7655 .07989 L
.77635 .08517 L
.7869 .09199 L
.79685 .10002 L
.81545 .11936 L
.8355 .14686 L
.85397 .17867 L
.89342 .26908 L
.93136 .38739 L
.97175 .55054 L
.97619 .57095 L
Mfstroke
.22421 .61803 m
.26171 .585 L
.30316 .54848 L
.34309 .5133 L
.3815 .47946 L
.42237 .44345 L
.46172 .40878 L
.49955 .37545 L
.53984 .33995 L
.57861 .3058 L
.61984 .26948 L
.65954 .23449 L
.69774 .20084 L
.73838 .16503 L
.77751 .13056 L
.81909 .09392 L
.85916 .05862 L
.89771 .02466 L
s
.89771 .02466 m
.9257 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3oooooohoQ00<0hN40Mn400`3Qh@0Dh@00Sn400`3Q
h@1fh@030>7Q01GQ002?h@030>7Q07GQ00<0hN405^4008oQ00<0hN40M>400`3Qh@0Gh@00Sn4207?Q
0P0Jh@00Sn400`3Qh@1ah@030>7Q01[Q002?h@030>7Q073Q00<0hN406n4008oQ00<0hN40Kn400`3Q
h@0Lh@00Sn400`3Qh@1^h@030>7Q01gQ002?h@030>7Q06cQ0P0Ph@00Sn4206cQ00<0hN408>4008oQ
00<0hN40J^400`3Qh@0Qh@00Sn400`3Qh@1Yh@030>7Q02;Q002?h@030>7Q06SQ00<0hN408n4000OQ
00<0hN40QN400`3Qh@1Vh@809^4000OQ00<0hN40QN400`3Qh@1Uh@030>7Q02KQ0007h@030>7Q08GQ
00<0hN40I>400`3Qh@0Wh@002>400`3Qh@24h@80I>400`3Qh@0Xh@002>400`3Qh@24h@030>7Q06;Q
00<0hN40:N4000SQ00<0hN40Q>400`3Qh@1Ph@80;>4000SQ00<0hN40Q>400`3Qh@1Oh@030>7Q02cQ
0009h@030>7Q07GQ1@02h@030>7Q00CQ00<0hN40>n4<01OQ00<0hN40;N4000WQ00<0hN40Mn401`3Q
hN40h@001N400`3Qh@0hh@<03>4301?Q00<0hN40;^4000WQ00<0hN40LN4300?Q00L0hN7Q0>4000GQ
0`0fh@804^42013Q00<0hN40;n4000WQ00<0hN40Mn401`3QhN40h@001N400`3Qh@0dh@805^400`3Q
h@0;h@80<^4000[Q00<0hN40M^401`3QhN40h@001N400`3Qh@0bh@806N4200[Q00<0hN40<^4000[Q
00<0hN40MN4200CQ00<0hN401>400`3Qh@0`h@807N400`3Qh@06h@030>7Q03?Q000:h@030>7Q08;Q
00<0hN40;n400`3Qh@0Nh@801N400`3Qh@0dh@002^400`3Qh@22h@030>7Q02gQ0P0Sh@050>7Qh@00
=n4000_Q00<0hN40PN400`3Qh@0/h@030>7Q02CQ00<0h@00>>4000_Q00<0hN40PN4202cQ00<0hN40
9^400`3Qh@0gh@002n400`3Qh@21h@030>7Q02[Q00<0hN409^4203WQ000;h@030>7Q087Q00<0hN40
:N400`3Qh@0Uh@800^400`3Qh@0fh@003>400`3Qh@20h@030>7Q02OQ0P0Wh@030>7Q00?Q00<0hN40
=N4000cQ00<0hN40P>400`3Qh@0Vh@030>7Q02KQ00<0hN401N400`3Qh@0dh@003>400`3Qh@20h@03
0>7Q02GQ00<0hN409^400`3Qh@07h@030>7Q03?Q000<h@030>7Q083Q0P0Uh@030>7Q02KQ00<0hN40
2>400`3Qh@0ch@003>400`3Qh@20h@030>7Q02?Q00<0hN409^400`3Qh@0:h@030>7Q03;Q000=h@03
0>7Q07oQ00<0hN408^400`3Qh@0Vh@030>7Q00cQ00<0hN40<N4000gQ00<0hN40On400`3Qh@0Qh@03
0>7Q02KQ00<0hN403^400`3Qh@0`h@003N400`3Qh@1oh@030>7Q023Q00<0hN409^400`3Qh@0?h@03
0>7Q033Q000=h@030>7Q07oQ00<0hN407n400`3Qh@0Vh@030>7Q017Q00<0hN40;n4000kQ00<0hN40
O^400`3Qh@0Nh@030>7Q02GQ0P0Eh@030>7Q02kQ000>h@030>7Q07kQ0P0Oh@030>7Q02CQ00<0hN40
5^400`3Qh@0]h@003^400`3Qh@1nh@030>7Q01gQ00<0hN409>400`3Qh@0Gh@030>7Q02gQ000>h@03
0>7Q07kQ00<0hN407>400`3Qh@0Th@030>7Q01WQ00<0hN40;>4000oQ00<0hN40ON400`3Qh@0Kh@03
0>7Q02CQ00<0hN406^400`3Qh@0/h@003n400`3Qh@1mh@030>7Q01_Q00<0hN408n400`3Qh@0Lh@03
0>7Q02_Q000?h@030>7Q07gQ00<0hN406^400`3Qh@0Sh@030>7Q01gQ00<0hN40:n4000oQ00<0hN40
ON4201[Q00<0hN408n400`3Qh@0Oh@030>7Q02[Q000@h@030>7Q07cQ00<0hN406>400`3Qh@0Sh@03
0>7Q023Q00<0hN40:^40013Q00<0hN40O>400`3Qh@0Hh@030>7Q02;Q00<0hN408^400`3Qh@0Yh@00
4>400`3Qh@1lh@030>7Q01OQ00<0hN408N4202GQ00<0hN40:N40013Q00<0hN40O>400`3Qh@0Fh@03
0>7Q027Q00<0hN409^400`3Qh@0Xh@004N400`3Qh@1dh@030>7Q00CQ00<0hN405N400`3Qh@0Qh@03
0>7Q02OQ00<0hN40:>40017Q00<0hN40Ln400`3Q0005h@030>7Q01GQ00<0hN408>400`3Qh@0Yh@03
0>7Q02OQ000Ah@030>7Q06kQ0`04h@030>7Q00?Q0`0Dh@030>7Q023Q00<0hN40:^400`3Qh@0Wh@00
4N400`3Qh@1ch@801^400`3Qh@0Ch@030>7Q023Q00<0hN40;>400`3Qh@0Vh@004^400`3Qh@1bh@03
0>7Q00GQ00<0hN404n400`3Qh@0Oh@030>7Q02gQ00<0hN409^4001;Q00<0hN40L^4300GQ00<0hN40
4^400`3Qh@0Oh@030>7Q02oQ00<0hN409N4001;Q00<0hN40N^400`3Qh@0Ah@030>7Q01oQ00<0hN40
<>400`3Qh@0Uh@004^400`3Qh@1jh@030>7Q013Q00<0hN407n400`3Qh@0bh@030>7Q02CQ000Ch@03
0>7Q07WQ00<0hN404>400`3Qh@0Mh@80=N400`3Qh@0Th@004n400`3Qh@1ih@804>400`3Qh@0Mh@03
0>7Q03KQ00<0hN408n4001?Q00<0hN40NN400`3Qh@0>h@030>7Q01gQ00<0hN40=n400`3Qh@0Sh@00
5>400`3Qh@1hh@030>7Q00gQ00<0hN407N400`3Qh@0hh@030>7Q02?Q000Dh@030>7Q07SQ00<0hN40
3N400`3Qh@0Lh@030>7Q03[Q00<0hN408^4001CQ00<0hN40N>400`3Qh@0<h@030>7Q01cQ00<0hN40
>n400`3Qh@0Rh@005>400`3Qh@1hh@030>7Q00_Q00<0hN407>400`3Qh@0mh@030>7Q027Q000Eh@03
0>7Q07OQ0P0;h@030>7Q01cQ00<0hN40?^400`3Qh@0Qh@005N400`3Qh@1gh@030>7Q00[Q00<0hN40
6^4204;Q00<0hN408>4001GQ00<0hN40Mn400`3Qh@09h@030>7Q01[Q00<0hN40@^400`3Qh@0Ph@00
5^400`3Qh@1fh@030>7Q00SQ00<0hN406^400`3Qh@14h@030>7Q01oQ000Fh@030>7Q07KQ00<0hN40
2>400`3Qh@0Ih@030>7Q04GQ00<0hN407n4001KQ00<0hN40M^400`3Qh@07h@030>7Q01WQ00<0hN40
An400`3Qh@0Nh@005n400`3Qh@1eh@030>7Q00KQ00<0hN406>4204[Q00<0hN407^4001OQ00<0hN40
MN4200KQ00<0hN406>400`3Qh@1;h@030>7Q01gQ000Gh@040>7Qh@809N4302?Q1@0Rh@030>7Q00GQ
00<0hN405n400`3Q000402?Q0`0Rh@040>4001cQ000Hh@030>7Q00;Q00<0hN408^400`3Qh@0Uh@03
0>7Q02;Q00<0hN401>400`3Qh@0Gh@030>7Q00;Q00<0hN408n400`3Qh@0Rh@050>7Qh@006n4001KQ
0`03h@030>7Q01kQ0`03h@030>7Q01kQ0`03h@030>7Q02;Q00<0hN400n400`3Qh@0Gh@030>7Q00?Q
00<0hN409>400`3Qh@0Rh@030>4001cQ000Hh@030>7Q00;Q00<0hN408n400`3Qh@0Th@030>7Q02;Q
00<0hN400n400`3Qh@0Fh@030>7Q00CQ00<0hN409>400`3Qh@0Rh@040>7Q01_Q000Hh@060>7Q0>40
9>400`3Q000Uh@030>7Q02;Q00<0hN400^400`3Qh@0Fh@030>7Q00GQ00<0hN408n400`3Q000Sh@80
00?Q0>406^4001WQ00@0hN409^400`3Qh@0Sh@809>4200;Q00<0hN405N4200OQ0P0Vh@030>7Q02?Q
0P0Lh@006N400`3Qh@1ch@050>7Qh@005^400`3Qh@1Eh@030>7Q01_Q000Ih@030>7Q07?Q00@0hN40
5^400`3Qh@1Fh@030>7Q01_Q000Jh@030>7Q07;Q00<0h@005^400`3Qh@1Gh@030>7Q01_Q000Jh@03
0>7Q07;Q0P0Fh@030>7Q05WQ00<0hN406^4001[Q00<0hN40L^4201GQ00<0hN40F^400`3Qh@0Jh@00
6n400`3Qh@1ah@030>7Q01?Q00<0hN40Fn400`3Qh@0Jh@00o`0P007Q0003h@030>7Q00GQ00<0hN40
1>400`3Qh@05h@801^400`3Qh@05h@030>7Q00GQ00<0hN401>400`3Qh@05h@030>7Q00GQ00<0hN40
1N400`3Qh@04h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00GQ00<0hN401>400`3Qh@05h@03
0>7Q00CQ0P07h@030>7Q00GQ00<0hN400P02h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00GQ
00<0hN401>400`3Qh@05h@030>7Q00GQ00<0hN401N400`3Qh@04h@030>7Q00GQ00<0hN401N400`3Q
h@05h@030>7Q00GQ00<0hN401>400`3Qh@05h@030>7Q00;Q000Lh@030>7Q06kQ00<0h@004N400`3Q
h@1Ph@030>7Q01WQ000Lh@030>7Q06gQ00@0hN404>400`3Qh@1Rh@030>7Q01SQ000Lh@030>7Q06gQ
00@0hN403n400`3Qh@1Sh@030>7Q01SQ000Lh@030>7Q06cQ00D0hN7Q000>h@030>7Q06CQ00<0hN40
6>4001gQ00<0hN40J^400`3Qh@02h@802n4206OQ00<0hN406>4001gQ00<0hN40J^400`3Qh@02h@03
0>7Q00WQ00<0hN40J>400`3Qh@0Gh@007N400`3Qh@1Yh@030>7Q00?Q00<0hN402>400`3Qh@1Yh@03
0>7Q01OQ000Nh@030>7Q06OQ00<0hN401>400`3Qh@07h@030>7Q06[Q00<0hN405n4001kQ00<0hN40
In400`3Qh@04h@030>7Q00KQ00<0hN40K>400`3Qh@0Fh@007^400`3Qh@1Vh@030>7Q00GQ00<0hN40
1N400`3Qh@1]h@030>7Q01KQ000Oh@030>7Q06CQ00<0hN401^400`3Qh@04h@030>7Q06kQ00<0hN40
5^4001oQ00<0hN40I>400`3Qh@06h@800n4207;Q00<0hN405N40023Q00<0hN40H^400`3Qh@07h@05
0>7Qh@00M>400`3Qh@0Eh@008>400`3Qh@1Qh@030>7Q00SQ00@0hN40MN400`3Qh@0Eh@008>400`3Q
h@1Qh@030>7Q00SQ00<0h@00M^400`3Qh@0Eh@008N400`3Qh@1Oh@030>7Q00WQ0P1hh@030>7Q01CQ
000Qh@030>7Q05kQ00<0hN402^400`3Qh@1gh@030>7Q01CQ000Rh@030>7Q05gQ00<0hN402N4307SQ
00<0hN405>4002;Q00<0hN40G>400`3Qh@09h@030>4007[Q00<0hN404n4002;Q00<0hN40Fn400`3Q
h@09h@040>7Q07[Q00<0hN404n4002?Q00<0hN40F^400`3Qh@08h@050>7Qh@00N^400`3Qh@0Ch@00
8n400`3Qh@1Ih@030>7Q00OQ0P04h@030>7Q07WQ00<0hN404^4002CQ00<0hN40En400`3Qh@07h@03
0>7Q00CQ00<0hN40NN400`3Qh@0Bh@009>400`3Qh@1Fh@030>7Q00OQ00<0hN401N400`3Qh@1ih@03
0>7Q01;Q000Th@030>7Q05KQ00<0hN401^400`3Qh@06h@80N^400`3Qh@0Bh@009N400`3Qh@1Dh@03
0>7Q00KQ00<0hN401n400`3Qh@1jh@030>7Q017Q000Uh@030>7Q05?Q00<0hN401^400`3Qh@08h@03
0>7Q07[Q00<0hN404N4002KQ00<0hN40D^400`3Qh@05h@030>7Q00WQ00<0hN40N^400`3Qh@0Ah@00
9^400`3Qh@1Ah@030>7Q00GQ00<0hN400n400`3Qh@04h@030>7Q07[Q00<0hN404N4002KQ00<0hN40
D>400`3Qh@05h@030>7Q00?Q00<0h@001N400`3Qh@1kh@030>7Q013Q000Wh@030>7Q04oQ00<0hN40
1>400`3Qh@06h@030>7Q00?Q0`1kh@030>7Q013Q000Wh@030>7Q04kQ00<0hN400n4200OQ0P06h@03
0>7Q07_Q00<0hN404>4002SQ00<0hN40C>400`3Qh@03h@030>7Q00OQ00<0hN401N400`3Qh@1kh@03
0>7Q013Q000Xh@030>7Q04_Q00<0hN400n400`3Qh@08h@<01N400`3Qh@1lh@030>7Q00oQ000Yh@03
0>7Q04[Q00<0hN400^400`3Qh@0Ah@030>7Q07cQ00<0hN403n4002WQ00<0hN40BN400`3Qh@02h@03
0>7Q01;Q00<0hN40O>400`3Qh@0?h@00:^400`3Qh@17h@030>7Q00;Q00<0hN404n400`3Qh@1lh@03
0>7Q00oQ000Zh@030>7Q04KQ00<0hN400^400`3Qh@0Dh@80O^400`3Qh@0>h@00:n400`3Qh@14h@03
0>7Q00;Q00<0hN405N400`3Qh@1mh@030>7Q00kQ000/h@030>7Q04?Q00<0hN400P0Hh@030>7Q07gQ
00<0hN403^4002cQ00<0hN40@^40103Qh@0Jh@030>7Q07gQ00<0hN403^4002gQ00<0hN40@>40103Q
h@0Kh@030>7Q07kQ00<0hN403N4002kQ00<0hN40?^40103Qh@0Lh@030>7Q07kQ00<0hN403N4002kQ
00<0hN40?^400`3Q000Mh@80On400`3Qh@0=h@00;n400`3Qh@0lh@<07^400`3Qh@1nh@030>7Q00gQ
000`h@030>7Q03[Q0P0Ph@030>7Q07oQ00<0hN403>40033Q00<0hN40>N42027Q00<0hN40On400`3Q
h@0<h@00<N400`3Qh@0gh@808^400`3Qh@1oh@030>7Q00cQ000ah@030>7Q03KQ0P0Sh@030>7Q07oQ
00<0hN403>4003;Q00<0hN40=>4202CQ00<0hN40P>400`3Qh@0;h@00<n400`3Qh@0bh@809N42087Q
00<0hN402n4003?Q00<0hN40<N4202KQ00<0hN40P>400`3Qh@0;h@00=>400`3Qh@0_h@030>7Q02KQ
00<0hN40P>400`3Qh@0;h@00=N400`3Qh@0]h@030>7Q02OQ00<0hN40PN400`3Qh@0:h@00=^400`3Q
h@0[h@030>7Q02SQ00<0hN40PN400`3Qh@0:h@00=n400`3Qh@0Yh@030>7Q02WQ00<0hN40PN400`3Q
h@0:h@00>>400`3Qh@0Wh@030>7Q02[Q0P22h@030>7Q00[Q000ih@030>7Q02CQ0P0]h@030>7Q08;Q
00<0hN402N4003[Q00<0hN408^4202kQ00<0hN40P^400`3Qh@09h@00>n400`3Qh@0Oh@<0;n400`3Q
h@22h@030>7Q00WQ000lh@807^400`3Q000`h@030>7Q08;Q00<0hN402N4003kQ00<0hN406N4402?Q
1@02h@030>7Q00CQ00<0hN40Pn400`3Qh@08h@00?n4201OQ0P000n40h@0Vh@070>7Qh@3Q0005h@03
0>7Q08?Q00<0hN402>40047Q0P0Bh@<00^400`3Qh@0Vh@070>7Qh@3Q0005h@<0Pn400`3Qh@08h@00
@n4200kQ0P04h@030>7Q02OQ00L0hN7Q0>4000GQ00<0hN40Pn400`3Qh@08h@00AN4500CQ1@05h@03
0>7Q02SQ00L0hN7Q0>4000GQ00<0hN40Q>400`3Qh@07h@00BN4500WQ00<0hN40:>4200CQ00<0hN40
1>400`3Qh@24h@030>7Q00OQ001Fh@030>7Q03KQ00<0hN40Q>400`3Qh@07h@00EN400`3Qh@0gh@03
0>7Q08CQ00<0hN401n4005?Q0P0jh@030>7Q08CQ00<0hN401n4005;Q00<0hN40>^4208KQ00<0hN40
1^40057Q00<0hN40>n400`3Qh@25h@030>7Q00KQ001@h@030>7Q03cQ00<0hN40QN400`3Qh@06h@00
Cn400`3Qh@0mh@030>7Q08kQ001=h@80@>400`3Qh@2>h@00C>400`3Qh@10h@030>7Q08kQ001;h@03
0>7Q047Q0P2?h@00B^400`3Qh@12h@030>7Q08kQ0018h@80AN400`3Qh@2>h@00An400`3Qh@15h@03
0>7Q08kQ0016h@030>7Q04KQ00<0hN40S^4004GQ00<0hN40An400`3Qh@2>h@00A>400`3Qh@18h@03
0>7Q08kQ0012h@80Bn4208oQ0011h@030>7Q04_Q00<0hN40S^40043Q00<0hN40C>400`3Qh@2>h@00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-3.68413, -13.6307, 
  0.0256734, 0.152978}},
  CellTags->{"Newton", "AnimateNewton"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.136054 0.311229 0.0228331 [
[.09184 .29873 -6 -9 ]
[.09184 .29873 6 0 ]
[.22789 .29873 -6 -9 ]
[.22789 .29873 6 0 ]
[.36395 .29873 -6 -9 ]
[.36395 .29873 6 0 ]
[.63605 .29873 -3 -9 ]
[.63605 .29873 3 0 ]
[.77211 .29873 -3 -9 ]
[.77211 .29873 3 0 ]
[.90816 .29873 -3 -9 ]
[.90816 .29873 3 0 ]
[.4875 .0829 -18 -4.5 ]
[.4875 .0829 0 4.5 ]
[.4875 .19706 -12 -4.5 ]
[.4875 .19706 0 4.5 ]
[.4875 .42539 -6 -4.5 ]
[.4875 .42539 0 4.5 ]
[.4875 .53956 -12 -4.5 ]
[.4875 .53956 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.09184 .31123 m
.09184 .31748 L
s
[(-3)] .09184 .29873 0 1 Mshowa
.22789 .31123 m
.22789 .31748 L
s
[(-2)] .22789 .29873 0 1 Mshowa
.36395 .31123 m
.36395 .31748 L
s
[(-1)] .36395 .29873 0 1 Mshowa
.63605 .31123 m
.63605 .31748 L
s
[(1)] .63605 .29873 0 1 Mshowa
.77211 .31123 m
.77211 .31748 L
s
[(2)] .77211 .29873 0 1 Mshowa
.90816 .31123 m
.90816 .31748 L
s
[(3)] .90816 .29873 0 1 Mshowa
.125 Mabswid
.11905 .31123 m
.11905 .31498 L
s
.14626 .31123 m
.14626 .31498 L
s
.17347 .31123 m
.17347 .31498 L
s
.20068 .31123 m
.20068 .31498 L
s
.2551 .31123 m
.2551 .31498 L
s
.28231 .31123 m
.28231 .31498 L
s
.30952 .31123 m
.30952 .31498 L
s
.33673 .31123 m
.33673 .31498 L
s
.39116 .31123 m
.39116 .31498 L
s
.41837 .31123 m
.41837 .31498 L
s
.44558 .31123 m
.44558 .31498 L
s
.47279 .31123 m
.47279 .31498 L
s
.52721 .31123 m
.52721 .31498 L
s
.55442 .31123 m
.55442 .31498 L
s
.58163 .31123 m
.58163 .31498 L
s
.60884 .31123 m
.60884 .31498 L
s
.66327 .31123 m
.66327 .31498 L
s
.69048 .31123 m
.69048 .31498 L
s
.71769 .31123 m
.71769 .31498 L
s
.7449 .31123 m
.7449 .31498 L
s
.79932 .31123 m
.79932 .31498 L
s
.82653 .31123 m
.82653 .31498 L
s
.85374 .31123 m
.85374 .31498 L
s
.88095 .31123 m
.88095 .31498 L
s
.06463 .31123 m
.06463 .31498 L
s
.03741 .31123 m
.03741 .31498 L
s
.0102 .31123 m
.0102 .31498 L
s
.93537 .31123 m
.93537 .31498 L
s
.96259 .31123 m
.96259 .31498 L
s
.9898 .31123 m
.9898 .31498 L
s
.25 Mabswid
0 .31123 m
1 .31123 L
s
.5 .0829 m
.50625 .0829 L
s
[(-10)] .4875 .0829 1 0 Mshowa
.5 .19706 m
.50625 .19706 L
s
[(-5)] .4875 .19706 1 0 Mshowa
.5 .42539 m
.50625 .42539 L
s
[(5)] .4875 .42539 1 0 Mshowa
.5 .53956 m
.50625 .53956 L
s
[(10)] .4875 .53956 1 0 Mshowa
.125 Mabswid
.5 .10573 m
.50375 .10573 L
s
.5 .12856 m
.50375 .12856 L
s
.5 .1514 m
.50375 .1514 L
s
.5 .17423 m
.50375 .17423 L
s
.5 .2199 m
.50375 .2199 L
s
.5 .24273 m
.50375 .24273 L
s
.5 .26556 m
.50375 .26556 L
s
.5 .2884 m
.50375 .2884 L
s
.5 .33406 m
.50375 .33406 L
s
.5 .35689 m
.50375 .35689 L
s
.5 .37973 m
.50375 .37973 L
s
.5 .40256 m
.50375 .40256 L
s
.5 .44823 m
.50375 .44823 L
s
.5 .47106 m
.50375 .47106 L
s
.5 .49389 m
.50375 .49389 L
s
.5 .51673 m
.50375 .51673 L
s
.5 .06007 m
.50375 .06007 L
s
.5 .03723 m
.50375 .03723 L
s
.5 .0144 m
.50375 .0144 L
s
.5 .56239 m
.50375 .56239 L
s
.5 .58523 m
.50375 .58523 L
s
.5 .60806 m
.50375 .60806 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .0515 m
.06244 .21262 L
.10458 .34794 L
.12507 .39968 L
.14415 .44018 L
.16371 .47437 L
.18221 .50025 L
.192 .51153 L
.20262 .52197 L
.2212 .53586 L
.23064 .54089 L
.24104 .54492 L
.24627 .54637 L
.24915 .54701 L
.2519 .54751 L
.25444 .54788 L
.25578 .54804 L
.25721 .54819 L
.25853 .5483 L
.25973 .54838 L
.26093 .54844 L
.26207 .54848 L
.26323 .54851 L
.26447 .54852 L
.26565 .54851 L
.26672 .54848 L
.26798 .54843 L
.26869 .5484 L
.26935 .54836 L
.27183 .54816 L
.27421 .5479 L
.27683 .54754 L
.28215 .54654 L
.28673 .54541 L
.29167 .54392 L
.30062 .54052 L
.32066 .52981 L
.34176 .51431 L
.37987 .47696 L
.42043 .42685 L
.45947 .37184 L
.50097 .30976 L
.54095 .25 L
.57942 .19582 L
.62034 .14527 L
.64069 .12397 L
.65974 .10691 L
.67918 .09275 L
.69763 .08271 L
.70733 .07888 L
Mistroke
.71246 .07728 L
.71792 .07591 L
.72052 .07538 L
.72296 .07495 L
.72535 .07461 L
.72753 .07436 L
.72962 .07417 L
.73067 .0741 L
.73182 .07403 L
.73298 .07399 L
.73422 .07395 L
.73539 .07394 L
.73645 .07394 L
.73764 .07397 L
.73875 .074 L
.73996 .07406 L
.74127 .07414 L
.74395 .07439 L
.74639 .07469 L
.75087 .07546 L
.75558 .07656 L
.76072 .07811 L
.7655 .07989 L
.77635 .08517 L
.7869 .09199 L
.79685 .10002 L
.81545 .11936 L
.8355 .14686 L
.85397 .17867 L
.89342 .26908 L
.93136 .38739 L
.97175 .55054 L
.97619 .57095 L
Mfstroke
.27063 .61803 m
.30316 .57355 L
.34309 .51894 L
.3815 .4664 L
.42237 .41051 L
.46172 .35669 L
.49955 .30495 L
.53984 .24985 L
.57861 .19682 L
.61984 .14044 L
.65954 .08613 L
.69774 .0339 L
s
.69774 .0339 m
.72252 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[3]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/@P10`40000100000800002008200000
P800P020P<30`<3L`:K:l00003<006H009T00<`00?l0000c03<c06Hc09Tc0<`c0?lc001V03=V06IV
09UV0<aV0?mV002I03>I06JI09VI0<bI0?nI003<03?<06K<09W<0<c<0?o<003o03?o06Ko09Wo0<co
0?oo0000<c<0<fH0<iT0<l`0<ol0<`0c<c<c<fHc<iTc<l`c<olc<`1V<c=V<fIV<iUV<laV<omV<`2I
<c>I<fJI<iVI<lbI<onI<`3<<c?<<fK<<iW<<lc<<oo<<`3o<c?o<fKo<iWo<lco<ooo<`00IS<0IVH0
IYT0I/`0I_l0IP0cIS<cIVHcIYTcI/`cI_lcIP1VIS=VIVIVIYUVI/aVI_mVIP2IIS>IIVJIIYVII/bI
I_nIIP3<IS?<IVK<IYW<I/c<I_o<IP3oIS?oIVKoIYWoI/coI_ooIP00VC<0VFH0VIT0VL`0VOl0V@0c
VC<cVFHcVITcVL`cVOlcV@1VVC=VVFIVVIUVVLaVVOmVV@2IVC>IVFJIVIVIVLbIVOnIV@3<VC?<VFK<
VIW<VLc<VOo<V@3oVC?oVFKoVIWoVLcoVOooV@00c3<0c6H0c9T0c<`0c?l0c00cc3<cc6Hcc9Tcc<`c
c?lcc01Vc3=Vc6IVc9UVc<aVc?mVc02Ic3>Ic6JIc9VIc<bIc?nIc03<c3?<c6K<c9W<c<c<c?o<c03o
c3?oc6Koc9Woc<coc?ooc000oc<0ofH0oiT0ol`0ool0o`0coc<cofHcoiTcol`coolco`1Voc=VofIV
oiUVolaVoomVo`2Ioc>IofJIoiVIolbIoonIo`3<oc?<ofK<oiW<olc<ooo<o`3ooc?oofKooiWoolco
ooooo`0000410@820P<30`@410D51@H61PL71`P820T92@X:2P/;2``<30h>3Pl?3a0@414A4A8B4Q<C
4a@D5?okl:2PY820P?l0003o0?oo0000ool0o`3oooooohoQ00<0hN40?N400`3Qh@1>h@00Sn400`3Q
h@0lh@030>7Q04oQ002?h@030>7Q03_Q00<0hN40D>4008oQ00<0hN40>n400`3Qh@1@h@00Sn4203_Q
00<0hN40DN4008oQ00<0hN40>N400`3Qh@1Bh@00Sn400`3Qh@0hh@030>7Q05?Q002?h@030>7Q03SQ
00<0hN40Dn4008oQ00<0hN40=n400`3Qh@1Dh@00Sn400`3Qh@0fh@030>7Q05GQ002?h@80=^400`3Q
h@1Fh@00Sn400`3Qh@0eh@030>7Q05KQ002?h@030>7Q03CQ00<0hN40En4008oQ00<0hN40<n400`3Q
h@1Hh@001n400`3Qh@25h@030>7Q03;Q00<0hN40FN4000OQ00<0hN40QN400`3Qh@0bh@030>7Q05WQ
0007h@030>7Q08GQ00<0hN40<N400`3Qh@1Jh@002>400`3Qh@24h@80<N400`3Qh@1Kh@002>400`3Q
h@24h@030>7Q02oQ00<0hN40G>4000SQ00<0hN40Q>400`3Qh@0_h@030>7Q05cQ0008h@030>7Q08CQ
00<0hN40;^400`3Qh@1Mh@002N400`3Qh@1eh@D00^400`3Qh@04h@030>7Q02gQ00<0hN402n4<04OQ
0009h@030>7Q07OQ00L0hN7Q0>4000GQ00<0hN40;>400`3Qh@09h@<03>4304CQ0009h@030>7Q077Q
0`03h@070>7Qh@3Q0005h@<0;>400`3Qh@07h@804^4204;Q0009h@030>7Q07OQ00L0hN7Q0>4000GQ
00<0hN40:n400`3Qh@06h@805^400`3Qh@0oh@002^400`3Qh@1fh@070>7Qh@3Q0005h@030>7Q02[Q
00<0hN401N4201WQ0P0oh@002^400`3Qh@1eh@801>400`3Qh@04h@030>7Q02WQ00<0hN401>4201gQ
00<0hN40?>4000[Q00<0hN40P^400`3Qh@0Yh@030>7Q00?Q00<0hN407^4203cQ000:h@030>7Q08;Q
00<0hN40:>400`3Qh@02h@808n400`3Qh@0ih@002n400`3Qh@21h@030>7Q02OQ00<0hN400^400`3Q
h@0Th@030>7Q03SQ000;h@030>7Q087Q0P0Wh@030>7Q00;Q00<0hN409^400`3Qh@0gh@002n400`3Q
h@21h@030>7Q02KQ00D0hN7Q000Yh@030>7Q03OQ000;h@030>7Q087Q00<0hN409N401@3QhN4002_Q
00<0hN40=^4000cQ00<0hN40P>400`3Qh@0Th@030>7Q0080;N400`3Qh@0eh@003>400`3Qh@20h@03
0>7Q02?Q00@0hN40<>400`3Qh@0dh@003>400`3Qh@20h@030>7Q02?Q00<0h@00<^400`3Qh@0ch@00
3>400`3Qh@20h@808n400`3Q000ch@030>7Q03?Q000<h@030>7Q083Q00<0hN408N400`3Q000eh@03
0>7Q03;Q000=h@030>7Q07oQ00<0hN408>400`3Q000gh@030>7Q037Q000=h@030>7Q07oQ00<0hN40
8>4203WQ00<0hN40<>4000gQ00<0hN40On400`3Qh@0Oh@80>^400`3Qh@0`h@003N400`3Qh@1oh@03
0>7Q01kQ0P0lh@030>7Q02oQ000>h@030>7Q07kQ00<0hN407^400`3Qh@0lh@030>7Q02kQ000>h@03
0>7Q07kQ0P0Nh@80?n400`3Qh@0]h@003^400`3Qh@1nh@030>7Q01cQ0P10h@030>7Q02gQ000>h@03
0>7Q07kQ00<0hN407>400`3Qh@10h@030>7Q02cQ000?h@030>7Q07gQ00<0hN406n400`3Qh@11h@03
0>7Q02cQ000?h@030>7Q07gQ00<0hN406^4204CQ00<0hN40:n4000oQ00<0hN40ON400`3Qh@0Ih@80
AN400`3Qh@0[h@003n400`3Qh@1mh@806^400`3Qh@15h@030>7Q02[Q000@h@030>7Q07cQ00<0hN40
6>400`3Qh@16h@030>7Q02[Q000@h@030>7Q07cQ00<0hN405n4204WQ00<0hN40:N40013Q00<0hN40
O>400`3Qh@0Gh@030>7Q04SQ00<0hN40:N40013Q00<0hN40O>400`3Qh@0Fh@030>7Q04[Q00<0hN40
:>40017Q00<0hN40M>400`3Qh@04h@030>7Q01GQ00<0hN40Bn400`3Qh@0Xh@004N400`3Qh@1ch@03
0>4000GQ00<0hN405N400`3Qh@1<h@030>7Q02OQ000Ah@030>7Q06kQ0`04h@030>7Q00?Q0`0Dh@03
0>7Q04gQ00<0hN409n40017Q00<0hN40Ln4200KQ00<0hN404n400`3Qh@1?h@030>7Q02KQ000Bh@03
0>7Q07;Q00<0hN401N400`3Qh@0Bh@80DN400`3Qh@0Vh@004^400`3Qh@1bh@<01N400`3Qh@0Bh@03
0>7Q057Q00<0hN409N4001;Q00<0hN40N^400`3Qh@0Ah@030>7Q05;Q00<0hN409N4001;Q00<0hN40
N^400`3Qh@0@h@030>7Q05CQ00<0hN409>4001?Q00<0hN40NN400`3Qh@0?h@80E^400`3Qh@0Th@00
4n400`3Qh@1ih@803n4205SQ00<0hN408n4001?Q00<0hN40NN400`3Qh@0>h@030>7Q05OQ00<0hN40
8n4001CQ00<0hN40N>400`3Qh@0=h@030>7Q05SQ00<0hN408n4001CQ00<0hN40N>400`3Qh@0<h@80
Fn400`3Qh@0Rh@005>400`3Qh@1hh@030>7Q00_Q0P1Lh@030>7Q02;Q000Dh@030>7Q07SQ00<0hN40
2^4205kQ00<0hN408N4001GQ00<0hN40Mn4200_Q00<0hN40GN400`3Qh@0Qh@005N400`3Qh@1gh@03
0>7Q00WQ0P1Ph@030>7Q023Q000Eh@030>7Q07OQ00<0hN402>42067Q00<0hN408>4001KQ00<0hN40
M^400`3Qh@07h@80Hn400`3Qh@0Oh@005^400`3Qh@1fh@030>7Q00OQ0P1Sh@030>7Q01oQ000Fh@03
0>7Q07KQ00<0hN401^4206GQ00<0hN407^4001OQ00<0hN40MN400`3Qh@05h@80I^400`3Qh@0Nh@00
5n400`3Qh@1eh@801^400`3Qh@1Vh@030>7Q01gQ000Gh@040>7Qh@809N4302?Q1@0Rh@030>7Q00CQ
0P0Kh@D08n4302;Q00@0h@007>4001SQ00<0hN400^400`3Qh@0Rh@030>7Q02GQ00<0hN408^400`3Q
h@03h@807^400`3Qh@0Sh@030>7Q02;Q00D0hN7Q000Kh@005^4300?Q00<0hN407^4300?Q00<0hN40
7^4300?Q00<0hN408^400`3Qh@02h@807n400`3Qh@0Th@030>7Q02;Q00<0h@007>4001SQ00<0hN40
0^400`3Qh@0Sh@030>7Q02CQ00<0hN408^400`3Qh@02h@807n400`3Qh@0Th@030>7Q02;Q00@0hN40
6n4001SQ00H0hN40h@0Th@030>4002GQ00<0hN408^40103QhN42023Q00<0hN408n400`3Q000Sh@80
00?Q0>406^4001WQ00@0hN409^400`3Qh@0Sh@809>420003h@00023Q0P0Vh@030>7Q02?Q0P0Lh@00
6N400`3Qh@1ch@030>7Q0080K^400`3Qh@0Kh@006N400`3Qh@1ch@040>4006oQ00<0hN406n4001[Q
00<0hN40L^43073Q00<0hN406n4001[Q00<0hN40L^4207;Q00<0hN406^4001[Q00<0hN40L^4207;Q
00<0hN406^4001_Q00<0hN40L>4207?Q00<0hN406^400?l08001h@000n400`3Qh@05h@030>7Q00CQ
00<0hN401N4200KQ00<0hN401N400`3Qh@05h@030>7Q00CQ00<0hN401N400`3Qh@05h@030>7Q00GQ
00<0hN401>400`3Qh@05h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00CQ00<0hN401N400`3Q
h@03h@<01n400`3Qh@05h@030>7Q00CQ00<0hN401N400`3Qh@05h@030>7Q00GQ00<0hN401N400`3Q
h@04h@030>7Q00GQ00<0hN401N400`3Qh@05h@030>7Q00CQ00<0hN401N400`3Qh@05h@030>7Q00GQ
00<0hN401N400`3Qh@04h@030>7Q00GQ00<0hN400^4001cQ00<0hN40KN420003h@3Q07?Q00<0hN40
6N4001cQ00<0hN40K>4200;Q00<0hN40Ln400`3Qh@0Hh@007>400`3Qh@1/h@800^400`3Qh@1ch@03
0>7Q01SQ000Lh@030>7Q06_Q0P03h@030>7Q07?Q00<0hN406>4001gQ00<0hN40JN4200CQ0P1dh@03
0>7Q01SQ000Mh@030>7Q06SQ00<0h@001>400`3Qh@1dh@030>7Q01OQ000Mh@030>7Q06SQ0P05h@03
0>7Q07CQ00<0hN405n4001kQ00<0hN40I^4200KQ00<0hN40M>400`3Qh@0Gh@007^400`3Qh@1Uh@03
0>4000KQ00<0hN40MN400`3Qh@0Fh@007^400`3Qh@1Th@030>4000OQ00<0hN40MN400`3Qh@0Fh@00
7n400`3Qh@1Sh@802>400`3Qh@1eh@030>7Q01KQ000Oh@030>7Q06;Q00<0h@002>4207OQ00<0hN40
5N40023Q00<0hN40H>400`3Q0009h@030>7Q07KQ00<0hN405N40023Q00<0hN40H>4200[Q00<0hN40
M^400`3Qh@0Eh@008>400`3Qh@1Oh@030>4000[Q00<0hN40M^400`3Qh@0Eh@008N400`3Qh@1Mh@03
0>4000_Q00<0hN40Mn400`3Qh@0Dh@008N400`3Qh@1Lh@030>4000cQ00<0hN40Mn400`3Qh@0Dh@00
8^400`3Qh@1Kh@030>4000cQ0P1hh@030>7Q01CQ000Rh@030>7Q05[Q00<0h@003N400`3Qh@1hh@03
0>7Q01?Q000Rh@030>7Q05WQ00<0h@003^400`3Qh@1hh@030>7Q01?Q000Sh@030>7Q05OQ00@0hN40
3^400`3Qh@1hh@030>7Q01?Q000Sh@030>7Q05OQ00<0h@003n400`3Qh@1ih@030>7Q01;Q000Th@03
0>7Q05GQ00<0h@004>400`3Qh@1ih@030>7Q01;Q000Th@030>7Q05CQ00<0h@004N400`3Qh@1ih@03
0>7Q01;Q000Th@030>7Q05?Q00@0hN404N4207[Q00<0hN404^4002GQ00<0hN40D^400`3Q000Bh@03
0>7Q07[Q00<0hN404N4002GQ00<0hN40DN400`3Q000Ch@030>7Q07[Q00<0hN404N4002KQ00<0hN40
Cn40103Qh@0Ch@030>7Q07[Q00<0hN404N4002KQ00<0hN40C^40103Qh@0=h@030>7Q00CQ00<0hN40
N^400`3Qh@0Ah@009^400`3Qh@1>h@030>4000gQ00<0h@001N400`3Qh@1kh@030>7Q013Q000Wh@03
0>7Q04cQ00@0hN403n400`3Qh@03h@<0Nn400`3Qh@0@h@009n400`3Qh@1;h@040>7Q00kQ0P06h@03
0>7Q07_Q00<0hN404>4002SQ00<0hN40BN40103Qh@0?h@030>7Q00GQ00<0hN40Nn400`3Qh@0@h@00
:>400`3Qh@19h@030>40013Q0`05h@030>7Q07cQ00<0hN403n4002WQ00<0hN40An40103Qh@0Hh@03
0>7Q07cQ00<0hN403n4002WQ00<0hN40A^40103Qh@0Ih@030>7Q07cQ00<0hN403n4002[Q00<0hN40
A>40103Qh@0Jh@030>7Q07cQ00<0hN403n4002[Q00<0hN40A>400`3Q000Kh@80O^400`3Qh@0>h@00
:n400`3Qh@12h@030>4001cQ00<0hN40ON400`3Qh@0>h@00;>400`3Qh@10h@040>7Q01cQ00<0hN40
ON400`3Qh@0>h@00;>400`3Qh@0oh@040>7Q01gQ00<0hN40ON400`3Qh@0>h@00;N400`3Qh@0nh@03
0>4001kQ00<0hN40O^400`3Qh@0=h@00;^400`3Qh@0lh@030>4001oQ00<0hN40O^400`3Qh@0=h@00
;^400`3Qh@0kh@040>7Q01oQ0P1oh@030>7Q00gQ000_h@030>7Q03[Q00<0h@008>400`3Qh@1nh@03
0>7Q00gQ000`h@030>7Q03SQ00<0h@008N400`3Qh@1oh@030>7Q00cQ000`h@030>7Q03OQ00<0h@00
8^400`3Qh@1oh@030>7Q00cQ000ah@030>7Q03GQ00<0h@008n400`3Qh@1oh@030>7Q00cQ000ah@03
0>7Q03GQ0P0Th@030>7Q07oQ00<0hN403>4003;Q00<0hN40<n4202GQ00<0hN40P>400`3Qh@0;h@00
<n400`3Qh@0ah@809^42087Q00<0hN402n4003?Q00<0hN40<>4202OQ00<0hN40P>400`3Qh@0;h@00
=>400`3Qh@0_h@030>7Q02KQ00<0hN40P>400`3Qh@0;h@00=N400`3Qh@0]h@030>7Q02OQ00<0hN40
PN400`3Qh@0:h@00=^400`3Qh@0[h@030>7Q02SQ00<0hN40PN400`3Qh@0:h@00=n400`3Qh@0Yh@03
0>7Q02WQ00<0hN40PN400`3Qh@0:h@00>>400`3Qh@0Wh@80:n4208;Q00<0hN402^4003WQ00<0hN40
9>4302cQ00<0hN40P^400`3Qh@09h@00>^400`3Qh@0Rh@030>4002gQ00<0hN40P^400`3Qh@09h@00
>n400`3Qh@0Oh@800^400`3Qh@0[h@030>7Q08;Q00<0hN402N4003cQ0P0Nh@050>7Qh@00;^400`3Q
h@22h@030>7Q00WQ000nh@030>7Q01WQ0P03h@030>7Q01oQ1@02h@030>7Q00CQ00<0hN40Pn400`3Q
h@08h@00?n4201OQ0P05h@030>7Q027Q00L0hN7Q0>4000GQ00<0hN40Pn400`3Qh@08h@00@N4201;Q
0`06h@030>7Q02;Q00L0hN7Q0>4000GQ0`23h@030>7Q00SQ0013h@803^4200SQ00<0hN408n401`3Q
hN40h@001N400`3Qh@23h@030>7Q00SQ0015h@D01>4500WQ00<0hN409>401`3QhN40h@001N400`3Q
h@24h@030>7Q00OQ0019h@D03^400`3Qh@0Sh@801>400`3Qh@04h@030>7Q08CQ00<0hN401n4005_Q
00<0hN40<N400`3Qh@24h@030>7Q00OQ001Jh@030>7Q03;Q00<0hN40Q>400`3Qh@07h@00F^400`3Q
h@0bh@030>7Q08CQ00<0hN401n4005WQ00<0hN40<n4208KQ00<0hN401^4005SQ00<0hN40=>400`3Q
h@25h@030>7Q00KQ001Hh@030>7Q03CQ00<0hN40QN400`3Qh@06h@00En400`3Qh@0eh@030>7Q08kQ
001Fh@030>7Q03KQ00<0hN40S^4005GQ00<0hN40=n400`3Qh@2>h@00E>400`3Qh@0hh@80Sn4005CQ
00<0hN40>>400`3Qh@2>h@00Dn400`3Qh@0ih@030>7Q08kQ001Bh@030>7Q03[Q00<0hN40S^40057Q
00<0hN40>n400`3Qh@2>h@00D>400`3Qh@0lh@030>7Q08kQ001?h@030>7Q03gQ00<0hN40S^4004oQ
00<0hN40?N4208oQ001>h@030>7Q03kQ00<0hN40S^4004gQ00<0hN40?n400`3Qh@2>h@00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-3.68413, -13.6307, 
  0.0256734, 0.152978}},
  CellTags->{"Newton", "AnimateNewton"}]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1600}, {0, 1144}},
AutoGeneratedPackage->Automatic,
CellGrouping->Manual,
WindowSize->{395, 910},
WindowMargins->{{65, Automatic}, {Automatic, 20}},
StyleDefinitions -> "HelpBrowser.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Blocks"->{
    Cell[1986, 61, 47, 1, 56, "Section",
      CellTags->"Blocks"],
    Cell[2036, 64, 591, 18, 130, "Text",
      CellTags->"Blocks"],
    Cell[2630, 84, 103, 3, 29, "Input",
      CellTags->"Blocks"],
    Cell[2758, 91, 68, 1, 39, "Subsection",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[2829, 94, 251, 5, 50, "Text",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3083, 101, 168, 4, 33, "Text",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3276, 109, 103, 2, 43, "Subsubsection",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3382, 113, 143, 4, 29, "Input",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3550, 121, 132, 4, 29, "Input",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3685, 127, 115, 3, 23, "Output",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3861, 137, 70, 1, 39, "Subsection",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[3956, 142, 218, 4, 50, "Text",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4177, 148, 169, 4, 33, "Text",
      CellTags->{"Blocks", "BlockFloat"}],
    Cell[4371, 156, 104, 2, 43, "Subsubsection",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4478, 160, 119, 3, 29, "Input",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4622, 167, 109, 3, 29, "Input",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4734, 172, 116, 3, 23, "Output",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4923, 183, 78, 1, 39, "Subsection",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[5026, 188, 374, 6, 92, "Text",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[5403, 196, 821, 20, 159, "Text",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6249, 220, 108, 2, 43, "Subsubsection",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6360, 224, 148, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6533, 232, 123, 3, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6659, 237, 223, 7, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6919, 249, 153, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7075, 255, 222, 7, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7334, 267, 145, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7482, 273, 120, 3, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7639, 281, 149, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7791, 287, 136, 3, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[8000, 298, 66, 1, 39, "Subsection",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8069, 301, 208, 4, 50, "Text",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8280, 307, 167, 4, 33, "Text",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8472, 315, 102, 2, 43, "Subsubsection",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8577, 319, 117, 3, 29, "Input",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8719, 326, 105, 3, 29, "Input",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8827, 331, 109, 3, 36, "Output",
      CellTags->{"Blocks", "BlockRat"}]},
  "BlockFast"->{
    Cell[2758, 91, 68, 1, 39, "Subsection",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[2829, 94, 251, 5, 50, "Text",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3083, 101, 168, 4, 33, "Text",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3276, 109, 103, 2, 43, "Subsubsection",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3382, 113, 143, 4, 29, "Input",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3550, 121, 132, 4, 29, "Input",
      CellTags->{"Blocks", "BlockFast"}],
    Cell[3685, 127, 115, 3, 23, "Output",
      CellTags->{"Blocks", "BlockFast"}]},
  "BlockFloat"->{
    Cell[3861, 137, 70, 1, 39, "Subsection",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[3956, 142, 218, 4, 50, "Text",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4177, 148, 169, 4, 33, "Text",
      CellTags->{"Blocks", "BlockFloat"}],
    Cell[4371, 156, 104, 2, 43, "Subsubsection",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4478, 160, 119, 3, 29, "Input",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4622, 167, 109, 3, 29, "Input",
      CellTags->{"BlockFloat", "Blocks"}],
    Cell[4734, 172, 116, 3, 23, "Output",
      CellTags->{"BlockFloat", "Blocks"}]},
  "BlockPrecision"->{
    Cell[4923, 183, 78, 1, 39, "Subsection",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[5026, 188, 374, 6, 92, "Text",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[5403, 196, 821, 20, 159, "Text",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6249, 220, 108, 2, 43, "Subsubsection",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6360, 224, 148, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6533, 232, 123, 3, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6659, 237, 223, 7, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[6919, 249, 153, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7075, 255, 222, 7, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7334, 267, 145, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7482, 273, 120, 3, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7639, 281, 149, 4, 29, "Input",
      CellTags->{"Blocks", "BlockPrecision"}],
    Cell[7791, 287, 136, 3, 23, "Output",
      CellTags->{"Blocks", "BlockPrecision"}]},
  "BlockRat"->{
    Cell[8000, 298, 66, 1, 39, "Subsection",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8069, 301, 208, 4, 50, "Text",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8280, 307, 167, 4, 33, "Text",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8472, 315, 102, 2, 43, "Subsubsection",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8577, 319, 117, 3, 29, "Input",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8719, 326, 105, 3, 29, "Input",
      CellTags->{"Blocks", "BlockRat"}],
    Cell[8827, 331, 109, 3, 36, "Output",
      CellTags->{"Blocks", "BlockRat"}]},
  "Integ"->{
    Cell[8994, 341, 45, 1, 56, "Section",
      CellTags->"Integ"],
    Cell[9042, 344, 271, 6, 84, "Text",
      CellTags->"Integ"],
    Cell[9316, 352, 107, 3, 49, "Input",
      CellTags->"Integ"],
    Cell[9426, 357, 85, 1, 39, "Subsection",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[9514, 360, 299, 5, 67, "Text",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[9816, 367, 340, 6, 83, "Text",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10159, 375, 111, 2, 43, "Subsubsection",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10273, 379, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10401, 384, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10551, 391, 138, 3, 29, "Input",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10692, 396, 15765, 475, 184, 4814, 334, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[26460, 873, 18246, 510, 184, 4902, 340, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[44709, 1385, 23209, 581, 184, 5094, 352, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[67921, 1968, 501, 13, 23, "Output",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[68437, 1984, 85, 1, 39, "Subsection",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[68525, 1987, 287, 5, 67, "Text",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[68815, 1994, 341, 6, 83, "Text",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69159, 2002, 78, 1, 35, "Subsubsection",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69240, 2005, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69368, 2010, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69518, 2017, 138, 3, 29, "Input",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69659, 2022, 15708, 474, 184, 4814, 334, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[85370, 2498, 18457, 513, 184, 4902, 340, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[103830, 3013, 22071, 567, 184, 5094, 352, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[125904, 3582, 501, 13, 23, "Output",
      CellTags->{"Integ", "AnimateTrapezoidal"}]},
  "AnimateRectangular"->{
    Cell[9426, 357, 85, 1, 39, "Subsection",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[9514, 360, 299, 5, 67, "Text",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[9816, 367, 340, 6, 83, "Text",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10159, 375, 111, 2, 43, "Subsubsection",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10273, 379, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10401, 384, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10551, 391, 138, 3, 29, "Input",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[10692, 396, 15765, 475, 184, 4814, 334, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[26460, 873, 18246, 510, 184, 4902, 340, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[44709, 1385, 23209, 581, 184, 5094, 352, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateRectangular"}],
    Cell[67921, 1968, 501, 13, 23, "Output",
      CellTags->{"Integ", "AnimateRectangular"}]},
  "AnimateTrapezoidal"->{
    Cell[68437, 1984, 85, 1, 39, "Subsection",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[68525, 1987, 287, 5, 67, "Text",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[68815, 1994, 341, 6, 83, "Text",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69159, 2002, 78, 1, 35, "Subsubsection",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69240, 2005, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69368, 2010, 125, 3, 29, "Input",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69518, 2017, 138, 3, 29, "Input",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[69659, 2022, 15708, 474, 184, 4814, 334, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[85370, 2498, 18457, 513, 184, 4902, 340, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[103830, 3013, 22071, 567, 184, 5094, 352, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Integ", "AnimateTrapezoidal"}],
    Cell[125904, 3582, 501, 13, 23, "Output",
      CellTags->{"Integ", "AnimateTrapezoidal"}]},
  "Newton"->{
    Cell[126439, 3600, 47, 1, 56, "Section",
      CellTags->"Newton"],
    Cell[126489, 3603, 262, 5, 67, "Text",
      CellTags->"Newton"],
    Cell[126754, 3610, 103, 3, 29, "Input",
      CellTags->"Newton"],
    Cell[126860, 3615, 76, 1, 39, "Subsection",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[126939, 3618, 337, 6, 67, "Text",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127279, 3626, 404, 7, 108, "Text",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127686, 3635, 107, 2, 43, "Subsubsection",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127796, 3639, 122, 3, 29, "Input",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127921, 3644, 124, 3, 29, "Input",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[128070, 3651, 147, 3, 29, "Input",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[128220, 3656, 14073, 469, 184, 4735, 348, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[142296, 4127, 15386, 506, 184, 5096, 373, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[157685, 4635, 15727, 508, 184, 5048, 370, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[173415, 5145, 14733, 490, 184, 4954, 364, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}]},
  "AnimateNewton"->{
    Cell[126860, 3615, 76, 1, 39, "Subsection",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[126939, 3618, 337, 6, 67, "Text",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127279, 3626, 404, 7, 108, "Text",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127686, 3635, 107, 2, 43, "Subsubsection",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127796, 3639, 122, 3, 29, "Input",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[127921, 3644, 124, 3, 29, "Input",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[128070, 3651, 147, 3, 29, "Input",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[128220, 3656, 14073, 469, 184, 4735, 348, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[142296, 4127, 15386, 506, 184, 5096, 373, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[157685, 4635, 15727, 508, 184, 5048, 370, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}],
    Cell[173415, 5145, 14733, 490, 184, 4954, 364, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"Newton", "AnimateNewton"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Blocks", 188856, 5657},
  {"BlockFast", 192011, 5732},
  {"BlockFloat", 192620, 5747},
  {"BlockPrecision", 193242, 5762},
  {"BlockRat", 194419, 5789},
  {"Integ", 195018, 5804},
  {"AnimateRectangular", 197629, 5861},
  {"AnimateTrapezoidal", 198841, 5887},
  {"Newton", 200054, 5913},
  {"AnimateNewton", 201506, 5946}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1709, 49, 142, 5, 214, "Chapter"],
Cell[1854, 56, 129, 3, 42, "Text"],
Cell[1986, 61, 47, 1, 56, "Section",
  CellTags->"Blocks"],
Cell[2036, 64, 591, 18, 130, "Text",
  CellTags->"Blocks"],
Cell[2630, 84, 103, 3, 29, "Input",
  CellTags->"Blocks"],

Cell[CellGroupData[{
Cell[2758, 91, 68, 1, 39, "Subsection",
  CellTags->{"Blocks", "BlockFast"}],
Cell[2829, 94, 251, 5, 50, "Text",
  CellTags->{"Blocks", "BlockFast"}],
Cell[3083, 101, 168, 4, 33, "Text",
  CellTags->{"Blocks", "BlockFast"}],

Cell[CellGroupData[{
Cell[3276, 109, 103, 2, 43, "Subsubsection",
  CellTags->{"Blocks", "BlockFast"}],
Cell[3382, 113, 143, 4, 29, "Input",
  CellTags->{"Blocks", "BlockFast"}],

Cell[CellGroupData[{
Cell[3550, 121, 132, 4, 29, "Input",
  CellTags->{"Blocks", "BlockFast"}],
Cell[3685, 127, 115, 3, 23, "Output",
  CellTags->{"Blocks", "BlockFast"}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3861, 137, 70, 1, 39, "Subsection",
  CellTags->{"BlockFloat", "Blocks"}],

Cell[CellGroupData[{
Cell[3956, 142, 218, 4, 50, "Text",
  CellTags->{"BlockFloat", "Blocks"}],
Cell[4177, 148, 169, 4, 33, "Text",
  CellTags->{"Blocks", "BlockFloat"}],

Cell[CellGroupData[{
Cell[4371, 156, 104, 2, 43, "Subsubsection",
  CellTags->{"BlockFloat", "Blocks"}],
Cell[4478, 160, 119, 3, 29, "Input",
  CellTags->{"BlockFloat", "Blocks"}],

Cell[CellGroupData[{
Cell[4622, 167, 109, 3, 29, "Input",
  CellTags->{"BlockFloat", "Blocks"}],
Cell[4734, 172, 116, 3, 23, "Output",
  CellTags->{"BlockFloat", "Blocks"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4923, 183, 78, 1, 39, "Subsection",
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[CellGroupData[{
Cell[5026, 188, 374, 6, 92, "Text",
  CellTags->{"Blocks", "BlockPrecision"}],
Cell[5403, 196, 821, 20, 159, "Text",
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[CellGroupData[{
Cell[6249, 220, 108, 2, 43, "Subsubsection",
  CellTags->{"Blocks", "BlockPrecision"}],
Cell[6360, 224, 148, 4, 29, "Input",
  CellTags->{"Blocks", "BlockPrecision"}],

Cell[CellGroupData[{
Cell[6533, 232, 123, 3, 29, "Input",
  CellTags->{"Blocks", "BlockPrecision"}],
Cell[6659, 237, 223, 7, 23, "Output",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[6919, 249, 153, 4, 29, "Input",
  CellTags->{"Blocks", "BlockPrecision"}],
Cell[7075, 255, 222, 7, 23, "Output",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[7334, 267, 145, 4, 29, "Input",
  CellTags->{"Blocks", "BlockPrecision"}],
Cell[7482, 273, 120, 3, 23, "Output",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[7639, 281, 149, 4, 29, "Input",
  CellTags->{"Blocks", "BlockPrecision"}],
Cell[7791, 287, 136, 3, 23, "Output",
  CellTags->{"Blocks", "BlockPrecision"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8000, 298, 66, 1, 39, "Subsection",
  CellTags->{"Blocks", "BlockRat"}],
Cell[8069, 301, 208, 4, 50, "Text",
  CellTags->{"Blocks", "BlockRat"}],
Cell[8280, 307, 167, 4, 33, "Text",
  CellTags->{"Blocks", "BlockRat"}],

Cell[CellGroupData[{
Cell[8472, 315, 102, 2, 43, "Subsubsection",
  CellTags->{"Blocks", "BlockRat"}],
Cell[8577, 319, 117, 3, 29, "Input",
  CellTags->{"Blocks", "BlockRat"}],

Cell[CellGroupData[{
Cell[8719, 326, 105, 3, 29, "Input",
  CellTags->{"Blocks", "BlockRat"}],
Cell[8827, 331, 109, 3, 36, "Output",
  CellTags->{"Blocks", "BlockRat"}]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[8975, 339, 16, 0, 25, "Text"],
Cell[8994, 341, 45, 1, 56, "Section",
  CellTags->"Integ"],
Cell[9042, 344, 271, 6, 84, "Text",
  CellTags->"Integ"],
Cell[9316, 352, 107, 3, 49, "Input",
  CellTags->"Integ"],
Cell[9426, 357, 85, 1, 39, "Subsection",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[9514, 360, 299, 5, 67, "Text",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[9816, 367, 340, 6, 83, "Text",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[10159, 375, 111, 2, 43, "Subsubsection",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[10273, 379, 125, 3, 29, "Input",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[10401, 384, 125, 3, 29, "Input",
  CellTags->{"Integ", "AnimateRectangular"}],

Cell[CellGroupData[{
Cell[10551, 391, 138, 3, 29, "Input",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[10692, 396, 15765, 475, 184, 4814, 334, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[26460, 873, 18246, 510, 184, 4902, 340, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[44709, 1385, 23209, 581, 184, 5094, 352, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Integ", "AnimateRectangular"}],
Cell[67921, 1968, 501, 13, 23, "Output",
  CellTags->{"Integ", "AnimateRectangular"}]
}, Open  ]],
Cell[68437, 1984, 85, 1, 39, "Subsection",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[68525, 1987, 287, 5, 67, "Text",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[68815, 1994, 341, 6, 83, "Text",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[69159, 2002, 78, 1, 35, "Subsubsection",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[69240, 2005, 125, 3, 29, "Input",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[69368, 2010, 125, 3, 29, "Input",
  CellTags->{"Integ", "AnimateTrapezoidal"}],

Cell[CellGroupData[{
Cell[69518, 2017, 138, 3, 29, "Input",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[69659, 2022, 15708, 474, 184, 4814, 334, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[85370, 2498, 18457, 513, 184, 4902, 340, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[103830, 3013, 22071, 567, 184, 5094, 352, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Integ", "AnimateTrapezoidal"}],
Cell[125904, 3582, 501, 13, 23, "Output",
  CellTags->{"Integ", "AnimateTrapezoidal"}]
}, Open  ]],
Cell[126420, 3598, 16, 0, 25, "Text"],
Cell[126439, 3600, 47, 1, 56, "Section",
  CellTags->"Newton"],
Cell[126489, 3603, 262, 5, 67, "Text",
  CellTags->"Newton"],
Cell[126754, 3610, 103, 3, 29, "Input",
  CellTags->"Newton"],
Cell[126860, 3615, 76, 1, 39, "Subsection",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[126939, 3618, 337, 6, 67, "Text",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[127279, 3626, 404, 7, 108, "Text",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[127686, 3635, 107, 2, 43, "Subsubsection",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[127796, 3639, 122, 3, 29, "Input",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[127921, 3644, 124, 3, 29, "Input",
  CellTags->{"Newton", "AnimateNewton"}],

Cell[CellGroupData[{
Cell[128070, 3651, 147, 3, 29, "Input",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[128220, 3656, 14073, 469, 184, 4735, 348, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[142296, 4127, 15386, 506, 184, 5096, 373, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[157685, 4635, 15727, 508, 184, 5048, 370, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Newton", "AnimateNewton"}],
Cell[173415, 5145, 14733, 490, 184, 4954, 364, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->{"Newton", "AnimateNewton"}]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

