(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     16571,        626]*)
(*NotebookOutlinePosition[     17243,        650]*)
(*  CellTagsIndexPosition[     17199,        646]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Arithmetic Expressions", "Section"],

Cell[TextData[{
  "This notebook\.18\.13 is designed to accompany Chapter 2 of \"Introduction \
to Scientific Programming: Computational Problem Solving Using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and C\" by Joseph L. Zachary.  In it, we will explore the use of basic \
arithmetic expressions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  We will also explore some of the differences between the rational and \
floating-point numbers that are supplied by ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  (25Jul1997)"
}], "Text"],

Cell[CellGroupData[{

Cell["Numbers", "Subsection"],

Cell[TextData[{
  "As you already know, you interact with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by entering a command and looking at the value that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " then displays.  The simplest kind of command is a number.  When you enter \
it, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " echoes its value, which is the number itself.  There are various ways to \
write numbers in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text"],

Cell["\<\
There are integers, which are written without decimal points or \
exponents:\
\>", "Text"],

Cell[BoxData[
    \(154\)], "Input"],

Cell["\<\
There are fractions, which are written as ratios of two integers:\
\
\>", "Text"],

Cell[BoxData[
    \(4/16\)], "Input"],

Cell["\<\
Integers and fractions are collectively called rational \
numbers.\
\>", "Text"],

Cell["\<\
There are floating-point numbers, which are written with decimal \
points:\
\>", "Text"],

Cell[BoxData[
    \(152.35\)], "Input"],

Cell["\<\
Floating-point numbers can also be written using scientific \
notation:\
\>", "Text"],

Cell[BoxData[
    \(152.35*^15\)], "Input"],

Cell["\<\
The \"*^15\" part means \"times 10 to the 15th power\".  Notice \
that the result is also written in scientific notation, although the \
appearance is more conventional.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Exercises", "Subsubsection"],

Cell["\<\
In the space provided, enter rational numbers that are equal \
to:\
\>", "Text"],

Cell["(1)  175.", "Text"],

Cell[BoxData[""], "Input"],

Cell["(2)  17.5", "Text"],

Cell[BoxData[""], "Input"],

Cell["(3)  175.*^-3", "Text"],

Cell[BoxData[""], "Input"],

Cell["\<\
In the space provided, enter floating-point numbers that are equal \
to:\
\>", "Text"],

Cell["(4)  16", "Text"],

Cell[BoxData[""], "Input"],

Cell["(5)  1/16", "Text"],

Cell[BoxData[""], "Input"],

Cell["(6)  5 1/4", "Text"],

Cell[BoxData[""], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Simple Expressions", "Subsection"],

Cell["\<\
Numbers can be combined with the five arithmetic operations of \
addition (+), subtraction (-), multiplication (*), division (/), and \
exponentiation (^).  They are used in exactly the way with which you are \
familiar from arithmetic.\
\>", "Text"],

Cell["We add two rational numbers:", "Text"],

Cell[BoxData[
    \(115\  + \ 72\)], "Input"],

Cell["We subtract two floating-point numbers:", "Text"],

Cell[BoxData[
    \(191.3\  - \ 72.3\)], "Input"],

Cell["We multiply two rational numbers:", "Text"],

Cell[BoxData[
    \(18723474387\ *\ 2398723497\)], "Input"],

Cell["We multiply two floating-point numbers:", "Text"],

Cell[BoxData[
    \(18723474387.\ *\ 2398723497.\)], "Input"],

Cell[TextData[{
  "Compare the results of the two multiplications.  When ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " operates on rational numbers, it gives an exact result no matter how many \
digits are required.  When ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " operates on two floating-point numbers, it gives the answer in scientific \
notation rounded to six decimal places."
}], "Text"],

Cell["We divide two rational numbers:", "Text"],

Cell[BoxData[
    \(8383832\ /\ 283742\)], "Input"],

Cell["We divide two floating-point numbers:", "Text"],

Cell[BoxData[
    \(8383832.\ /\ 283742.\)], "Input"],

Cell[TextData[{
  "Again, notice that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " gives exact results for the rational division and a result rounded to six \
digits for the floating-point division."
}], "Text"],

Cell["We exponentiate two rational numbers:", "Text"],

Cell[BoxData[
    \(2\ ^\ 100\)], "Input"],

Cell["We exponentiate two floating-point numbers:", "Text"],

Cell[BoxData[
    \(2.\ ^\ 100.\)], "Input"],

Cell[CellGroupData[{

Cell["Exercises", "Subsubsection"],

Cell["\<\
Compute the following values.  Are the results rational numbers or \
floating-point numbers?\
\>", "Text"],

Cell["\<\
(7)  The number of inches in a mile (there are 5280 feet in a mile \
and 12 inches in a foot).\
\>", "Text"],

Cell[BoxData[""], "Input"],

Cell["\<\
(8)  The number of bytes in a megabyte (it's 2 to the 20th \
power).\
\>", "Text"],

Cell[BoxData[""], "Input"],

Cell["\<\
(9)  The number of inches in a meter (there are .0254 meters in an \
inch). \
\>", "Text"],

Cell[BoxData[""], "Input"],

Cell["\<\
(10)  The difference between your height in feet and your best \
friend's height in feet.\
\>", "Text"],

Cell[BoxData[""], "Input"],

Cell["\<\
(11)  The combined weight of you and George Foreman (let's say he \
weighs 260 pounds).\
\>", "Text"],

Cell[BoxData[""], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Mixed Expressions", "Subsection"],

Cell[TextData[{
  "If an expression contains a mixture of rational and floating-point \
numbers, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " first converts the rational number into floating-point form and then does \
the operation.  Compare the results of the following operations to the ones \
in the previous section."
}], "Text"],

Cell["We multiply a rational number and a floating-point number:", "Text"],

Cell[BoxData[
    \(18723474387\ *\ 2398723497.\)], "Input"],

Cell["We divide a floating-point number and a rational number:", "Text"],

Cell[BoxData[
    \(8383832.\ /\ 283742\)], "Input"],

Cell["\<\
We exponentiate a floating-point number and a rational number:\
\>",
   "Text"],

Cell[BoxData[
    \(2.\ ^\ 100\)], "Input"],

Cell["\<\
In each case, the result is the same as it would have been if both \
of the numbers had been floating-point numbers.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Exercises", "Subsubsection"],

Cell[TextData[{
  "We have been careful not to make any mistakes in our expressions to this \
point, but they will be very common when you experiment with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  Mistakes can be things like forgetting an operator symbol, forgetting a \
panthesis, putting in too many parentheses, and the like.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " tries to explain what is wrong, but the explanations are not\nalways \
clear.  Use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to evaluate each of the expressions below, and then correct the syntax \
error."
}], "Text"],

Cell["(12)  Average of 16 and 39.", "Text"],

Cell[BoxData[
    \(0.5\ *\ \((16\  + \ 39\)\)], "Input"],

Cell["(13)  Sum of 10 and 200.", "Text"],

Cell[BoxData[
    \(10\ 5 + \ 200\)], "Input"],

Cell["(14)  Product of first ten even numbers.", "Text"],

Cell[BoxData[
    \(2\ *\ 4\ *\ 6\ *\ 8\ *\ 10\ , \ 12\ *\ 14\ *\ 16\ *\ 18\ *\ 20\)], 
  "Input"],

Cell["(15)  11 raised to the 11th power.", "Text"],

Cell[BoxData[
    \(\(11\ &\)\ 11\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Compound Expressions", "Subsection"],

Cell[TextData[{
  "Just as in arithmetic, more than one operation can be carried out in a \
single expression.  The order in which the operations are carried out depends \
both on the way they are grouped with parentheses and on ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s precedence rules."
}], "Text"],

Cell["Here we take the average of two numbers:", "Text"],

Cell[BoxData[
    \(\.03\((158\  + \ 64)\)\ /\ 2\)], "Input"],

Cell["\<\
Notice that parentheses are used, just as in mathematics, to \
indicate the order in which operations are carried out.  In the absence of \
parentheses, the operations are not necessarily carried out from left to \
right:\
\>", "Text"],

Cell[BoxData[
    \(158\  + \ 64\ /\ 2\)], "Input"],

Cell[TextData[{
  "In the absence of parentheses, the division was performed first, followed \
by the addition.  Once you have learned ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s precedence rules, you will be able to predict the order in which \
arithmetic operations will be carried out.  We will not discuss precedence \
rules in this notebook.  You should use parentheses whenever you are in \
doubt."
}], "Text"],

Cell["\<\
In a compound expression, a rational number is converted to \
floating-point form whenever it is involved in an operation with a \
floating-point number.  Consider:\
\>", "Text"],

Cell[BoxData[
    \(\((158\  + \ 64)\)\ /\ 2.0\)], "Input"],

Cell["\<\
The addition involves two rational numbers, so rational number \
addition is done.  The sum is then converted to floating-point form so that \
it can be divided by a floating-point number.  On ther other hand, \
consider:\
\>", "Text"],

Cell[BoxData[
    \(\((158.\  + \ 64)\)\ /\ 2\)], "Input"],

Cell["\<\
The sum involves a floating-point and a rational number, so the \
rational number is first converted to floating-point form.  The sum is a \
floating-point number, so the divisor must be converted into floating-point \
form before the division can be carried out.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Exercises", "Subsubsection"],

Cell["\<\
(16)  Write an expression to calculate the number of centimeters in \
a mile.  (There are 5280 feet in a mile, 12 inches in a foot, and 2.54 \
centimeters in an inch.)\
\>", "Text"],

Cell[BoxData[""], "Input"],

Cell["\<\
(17)  Write an expression to compute the grade point average of a \
class of 11 students, where 4 make an A (4.0), 3 make a B+ (3.3), 3 make a C \
(2.0), and 1 makes a C- (1.7).\
\>", "Text"],

Cell[BoxData[""], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Function Applications", "Subsection"],

Cell[TextData[{
  "Besides the five arithmetic operators, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " provides many built-in functions that operate on numbers.  Whereas \
operator symbols are written between the numbers that they operate on (their \
parameters), built-in function names are written in front of their \
parameters, just as is the convention in mathematics."
}], "Text"],

Cell["\<\
The \"N\" function expects one parameter, which is typically a \
rational number, and produces the closest floating-point equivalent:\
\>", 
  "Text"],

Cell[BoxData[
    \(N[1/7]\)], "Input"],

Cell["\<\
The \"Sqrt\" function produces the square root of its parameter.  \
We can take the square root of a floating-point number:\
\>", "Text"],

Cell[BoxData[
    \(Sqrt[18.0]\)], "Input"],

Cell["Or of a rational number:", "Text"],

Cell[BoxData[
    \(Sqrt[18]\)], "Input"],

Cell["\<\
Notice how the square root of a floating-point number is written as \
a six-digit approximation whereas the square root of a rational number is \
written exactly, even if it requires a radical to do so.\
\>", "Text"],

Cell["\<\
Just as we can write expressions with more than one arithmetic \
operation, we can write expressions with more than one built-in \
function:\
\>", "Text"],

Cell[BoxData[
    \(N[Sqrt[18]]\)], "Input"],

Cell["\<\
This takes the square root of a rational number and then converts \
it into floating-point form.\
\>", "Text"],

Cell["This expression amounts to the same thing:", "Text"],

Cell[BoxData[
    \(N[Sqrt[9 + 9]]\)], "Input"],

Cell["\<\
The \"Cos\" function produces the cosine of its parameter.  (The \
parameter should be expressed in radians.)\
\>", "Text"],

Cell[BoxData[
    \(Cos \((3.141592654\ /\ 3)\)\)], "Input"],

Cell["\<\
Some functions take more than one parameter.  For example, \"N\" \
will take an optional second argument, which should be an integer.  The \
second parameter specifies the number of digits that should appear in the \
mantissa of the resulting floating-point number:\
\>", "Text"],

Cell[BoxData[
    \(N[1/7, \ 25]\)], "Input"],

Cell["\<\
Notice that when more than one parameter is passed to a function, \
they are separated with commas.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Exercises", "Subsubsection"],

Cell["Compute the following values.", "Text"],

Cell["\<\
(18)  The exact sum of the square roots of 1, 2, 3, 4, and 5.\
\>", 
  "Text"],

Cell[BoxData[""], "Input"],

Cell["(19)  A floating-point approximation to the sum above.", "Text"],

Cell[BoxData[""], "Input"],

Cell["\<\
(20)  The square root of the sum of the sine and the cosine of 1/4 \
Pi.  (Use a value of 3.14159 for Pi.)\
\>", "Text"],

Cell[BoxData[""], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Help With Functions", "Subsection"],

Cell[TextData[{
  "If you happen to know the name of a function and wish to know more about \
it, you can ask ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to display help information for it.  For example, let's see what ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can tell us about the \"Sqrt\" function.  To ask for help, issue a \
command consisting of a question mark followed by the function name."
}], "Text"],

Cell[BoxData[
    \(\(?Sqrt\)\)], "Input"],

Cell["\<\
You can get more detailed information by using the help browser.  \
Choose the \"Help...\" option from the \"Help\" menu, enter \"Sqrt\" to the \
left of the \"Go To:\" button, and then click the button.\
\>", "Text"],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will help you explore even if you have no idea what you're looking for.   \
When you select one of the topics displayed in the help browser, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will display a variety of subtopics.  For example, selecting \"Algebraic \
Computation\" in the first column displays some subtopics.  Selecting \
\"Formula Manipulation\" in the second column displays more subtopics.    \
Finally, selecting \"Denominator\" in the third column displays help \
information for that built-in function."
}], "Text"],

Cell[BoxData[
    \(Denominator[3/15]\)], "Input"],

Cell[CellGroupData[{

Cell["Exercises", "Subsubsection"],

Cell[TextData[{
  "(21) Use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s help facility to find a way to compute the base 2 logarithm of a \
number.  (The base 2 logarithm of 16 is 4 and of 19.2 is 4.263034406.)"
}], "Text"]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowSize->{520, 600},
WindowMargins->{{260, Automatic}, {160, Automatic}},
StyleDefinitions -> "Default.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 41, 0, 54, "Section"],
Cell[1775, 53, 579, 14, 104, "Text"],

Cell[CellGroupData[{
Cell[2379, 71, 29, 0, 45, "Subsection"],
Cell[2411, 73, 531, 16, 86, "Text"],
Cell[2945, 91, 100, 3, 32, "Text"],
Cell[3048, 96, 36, 1, 27, "Input"],
Cell[3087, 99, 91, 3, 32, "Text"],
Cell[3181, 104, 37, 1, 27, "Input"],
Cell[3221, 107, 90, 3, 32, "Text"],
Cell[3314, 112, 98, 3, 32, "Text"],
Cell[3415, 117, 39, 1, 27, "Input"],
Cell[3457, 120, 95, 3, 32, "Text"],
Cell[3555, 125, 43, 1, 27, "Input"],
Cell[3601, 128, 193, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[3819, 136, 34, 0, 42, "Subsubsection"],
Cell[3856, 138, 90, 3, 32, "Text"],
Cell[3949, 143, 25, 0, 32, "Text"],
Cell[3977, 145, 26, 0, 27, "Input"],
Cell[4006, 147, 25, 0, 32, "Text"],
Cell[4034, 149, 26, 0, 27, "Input"],
Cell[4063, 151, 29, 0, 32, "Text"],
Cell[4095, 153, 26, 0, 27, "Input"],
Cell[4124, 155, 96, 3, 32, "Text"],
Cell[4223, 160, 23, 0, 32, "Text"],
Cell[4249, 162, 26, 0, 27, "Input"],
Cell[4278, 164, 25, 0, 32, "Text"],
Cell[4306, 166, 26, 0, 27, "Input"],
Cell[4335, 168, 26, 0, 32, "Text"],
Cell[4364, 170, 26, 0, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[4439, 176, 40, 0, 29, "Subsection"],
Cell[4482, 178, 260, 5, 68, "Text"],
Cell[4745, 185, 44, 0, 32, "Text"],
Cell[4792, 187, 45, 1, 27, "Input"],
Cell[4840, 190, 55, 0, 32, "Text"],
Cell[4898, 192, 49, 1, 27, "Input"],
Cell[4950, 195, 49, 0, 32, "Text"],
Cell[5002, 197, 59, 1, 27, "Input"],
Cell[5064, 200, 55, 0, 32, "Text"],
Cell[5122, 202, 61, 1, 27, "Input"],
Cell[5186, 205, 422, 10, 86, "Text"],
Cell[5611, 217, 47, 0, 32, "Text"],
Cell[5661, 219, 51, 1, 27, "Input"],
Cell[5715, 222, 53, 0, 32, "Text"],
Cell[5771, 224, 53, 1, 27, "Input"],
Cell[5827, 227, 225, 6, 50, "Text"],
Cell[6055, 235, 53, 0, 32, "Text"],
Cell[6111, 237, 42, 1, 27, "Input"],
Cell[6156, 240, 59, 0, 32, "Text"],
Cell[6218, 242, 44, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[6287, 247, 34, 0, 42, "Subsubsection"],
Cell[6324, 249, 116, 3, 32, "Text"],
Cell[6443, 254, 118, 3, 32, "Text"],
Cell[6564, 259, 26, 0, 27, "Input"],
Cell[6593, 261, 92, 3, 32, "Text"],
Cell[6688, 266, 26, 0, 27, "Input"],
Cell[6717, 268, 100, 3, 32, "Text"],
Cell[6820, 273, 26, 0, 27, "Input"],
Cell[6849, 275, 113, 3, 32, "Text"],
Cell[6965, 280, 26, 0, 27, "Input"],
Cell[6994, 282, 111, 3, 32, "Text"],
Cell[7108, 287, 26, 0, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[7183, 293, 39, 0, 29, "Subsection"],
Cell[7225, 295, 347, 8, 68, "Text"],
Cell[7575, 305, 74, 0, 32, "Text"],
Cell[7652, 307, 60, 1, 27, "Input"],
Cell[7715, 310, 72, 0, 32, "Text"],
Cell[7790, 312, 52, 1, 27, "Input"],
Cell[7845, 315, 89, 3, 32, "Text"],
Cell[7937, 320, 43, 1, 27, "Input"],
Cell[7983, 323, 140, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[8148, 330, 34, 0, 42, "Subsubsection"],
Cell[8185, 332, 642, 15, 122, "Text"],
Cell[8830, 349, 43, 0, 32, "Text"],
Cell[8876, 351, 57, 1, 27, "Input"],
Cell[8936, 354, 40, 0, 32, "Text"],
Cell[8979, 356, 46, 1, 27, "Input"],
Cell[9028, 359, 56, 0, 32, "Text"],
Cell[9087, 361, 98, 2, 27, "Input"],
Cell[9188, 365, 50, 0, 32, "Text"],
Cell[9241, 367, 46, 1, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[9336, 374, 42, 0, 29, "Subsection"],
Cell[9381, 376, 320, 7, 68, "Text"],
Cell[9704, 385, 56, 0, 32, "Text"],
Cell[9763, 387, 61, 1, 27, "Input"],
Cell[9827, 390, 245, 5, 68, "Text"],
Cell[10075, 397, 51, 1, 27, "Input"],
Cell[10129, 400, 433, 9, 86, "Text"],
Cell[10565, 411, 188, 4, 50, "Text"],
Cell[10756, 417, 59, 1, 27, "Input"],
Cell[10818, 420, 245, 5, 68, "Text"],
Cell[11066, 427, 58, 1, 27, "Input"],
Cell[11127, 430, 287, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[11439, 439, 34, 0, 42, "Subsubsection"],
Cell[11476, 441, 191, 4, 50, "Text"],
Cell[11670, 447, 26, 0, 27, "Input"],
Cell[11699, 449, 201, 4, 50, "Text"],
Cell[11903, 455, 26, 0, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[11978, 461, 43, 0, 29, "Subsection"],
Cell[12024, 463, 400, 8, 86, "Text"],
Cell[12427, 473, 160, 4, 50, "Text"],
Cell[12590, 479, 39, 1, 27, "Input"],
Cell[12632, 482, 147, 3, 50, "Text"],
Cell[12782, 487, 43, 1, 27, "Input"],
Cell[12828, 490, 40, 0, 32, "Text"],
Cell[12871, 492, 41, 1, 27, "Input"],
Cell[12915, 495, 226, 4, 68, "Text"],
Cell[13144, 501, 164, 4, 50, "Text"],
Cell[13311, 507, 44, 1, 27, "Input"],
Cell[13358, 510, 120, 3, 32, "Text"],
Cell[13481, 515, 58, 0, 32, "Text"],
Cell[13542, 517, 47, 1, 27, "Input"],
Cell[13592, 520, 133, 3, 50, "Text"],
Cell[13728, 525, 60, 1, 27, "Input"],
Cell[13791, 528, 289, 5, 68, "Text"],
Cell[14083, 535, 45, 1, 27, "Input"],
Cell[14131, 538, 123, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[14279, 545, 34, 0, 42, "Subsubsection"],
Cell[14316, 547, 45, 0, 32, "Text"],
Cell[14364, 549, 88, 3, 32, "Text"],
Cell[14455, 554, 26, 0, 27, "Input"],
Cell[14484, 556, 70, 0, 32, "Text"],
Cell[14557, 558, 26, 0, 27, "Input"],
Cell[14586, 560, 130, 3, 50, "Text"],
Cell[14719, 565, 26, 0, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[14794, 571, 41, 0, 29, "Subsection"],
Cell[14838, 573, 443, 10, 86, "Text"],
Cell[15284, 585, 42, 1, 27, "Input"],
Cell[15329, 588, 227, 4, 50, "Text"],
Cell[15559, 594, 618, 12, 122, "Text"],
Cell[16180, 608, 50, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[16255, 613, 34, 0, 42, "Subsubsection"],
Cell[16292, 615, 239, 6, 50, "Text"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

